/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package ability;

import java.util.List;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.math.Circle;

import game.GameScreen;
import game.QuestMain;
import gameobject.HealingSparkles;
import gameobject.ObjectType;
import gameobject.character.Character;

/**
 * Class representing the mage's special ability
 * @author jordan
 * @version Jan 6 2014
 */
public class MageSpecial extends SpecialAbility {

	private float SPARKLES_WIDTH = 0.1f;
	private float SPARKLES_HEIGHT = 0.1f;

	private int amountHealed;
	private boolean boss;

	/**
	 * Constructor
	 * @param range the range of the ability
	 * @param energyUse the energy used up by the ability
	 * @param amountHealed the amount of healing caused by the ability
	 * @param character the character to whom this ability belongs
	 */
	public MageSpecial(boolean boss, float range, int energyUse, int amountHealed, int cooldown, Character character) {
		super(range, energyUse, cooldown, character);
		this.amountHealed = amountHealed;
		this.boss = boss;
	}
	
	//initialize the audio file
		Sound mageSpecialSound = Gdx.audio.newSound(Gdx.files.internal("assets/data/sound/magespecial.wav"));
		

	/*
	 * @see ability.SpecialAbility#activate()
	 */
	@Override
	public void activate()
	{
		if(cooldownTimer <= 0){
			heal();
		}
	}

	/*
	 * @see ability.SpecialAbility#update(float)
	 */
	public void update(float delta)
	{
		// update cooldown timer
		if(cooldownTimer > 0)
		{
			cooldownTimer--;
		}
		if(cooldownTimer <= 0)
		{
			cooldownTimer = 0;
		}
	}

	/**
	 * Heals any allied characters within the range of the mage
	 */
	private void heal() {
		if(character.getEnergy() >= energyUse)
		{
			cooldownTimer = cooldown;

			//subtract energy
			character.useEnergy(energyUse);
			
			//play sound
			mageSpecialSound.play(1.0f);
			
			//make sparkles
			makeHealySparkles();
		}
	}
	
	
	public void makeHealySparkles()
	{
		GameScreen gameScreen = QuestMain.getCurrentScreen();
		
		List<Character> allies;
		// get correct list of allies
		if(character.getObjectType() == ObjectType.PRIMARY || character.getObjectType() == ObjectType.SECONDARY ||
				character.getObjectType() == ObjectType.TERTIARY)
		{
			allies = gameScreen.getAllyList();
		}
		else
		{
			allies = gameScreen.getEnemyList();
		}

		Circle attackZone = new Circle(character.getX(), character.getY(), range);
		
		// if there is an opponent within the attack zone, apply damage
		for(Character ally : allies)
		{
			if(attackZone.contains(ally.getX() + ally.getWidth()/2,
					ally.getY() + ally.getHeight()/2) && !ally.isDead())
			{
				if(!boss)
				{
					ally.heal(amountHealed);
				}
				else
				{
					character.heal(amountHealed);
					ally.takeDamage(amountHealed);
				}

				HealingSparkles sparkles = new HealingSparkles(boss, ally.getX() - SPARKLES_WIDTH/16.0f,
						ally.getY() - SPARKLES_HEIGHT/2.0f, SPARKLES_WIDTH, SPARKLES_HEIGHT,
						ObjectType.HEALING_SPARKLES);
				gameScreen.addNewObject(sparkles);
			}
		}	
	}

}
