/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package ability;

import java.util.List;

import manager.CollisionManager;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.audio.*;

import game.GameScreen;
import game.QuestMain;
import gameobject.GameObject;
import gameobject.ObjectType;
import gameobject.character.Character;
import gameobject.character.DeadTree;
import gameobject.character.Direction;

/**
 * Class representing the warrior's attack ability
 * @author jordan
 * @version Jan 6 2014
 */
public class WarriorAttack extends AttackAbility {

	/**
	 * Constructor
	 * @param range the range of the attack
	 * @param damage the damage caused by the attack
	 * @param energyUse the energy used up by the attack
	 * @param cooldown the cooldown time for the attack
	 * @param character the character to whom this ability belongs
	 */
	public WarriorAttack(float range, int damage, int energyUse,
			int cooldown, Character character) {
		super(range, damage, energyUse, cooldown, character);
	}
	
	//initialize the audio file
	Sound warriorAttackSound = Gdx.audio.newSound(Gdx.files.internal("assets/data/sound/warriorattack.wav"));
	
	/*
	 * @see ability.AttackAbility#sendAttack(float, float, float)
	 */
	@Override
	protected void sendAttack(float destX, float destY, float range)
	{
		// check that there is enough energy
		if (character.getEnergy() >= energyUse)
		{
			GameScreen gameScreen = QuestMain.getCurrentScreen();
			
			cooldownTimer = cooldown;
			
			character.useEnergy(energyUse);
			
			//play sound
			warriorAttackSound.play(1.0f);
			
			List<Character> opponents;
			List<DeadTree> trees;
			// get correct list of opponents
			if(character.getObjectType() == ObjectType.PRIMARY || character.getObjectType() == ObjectType.SECONDARY ||
					character.getObjectType() == ObjectType.TERTIARY)
			{
				opponents = gameScreen.getEnemyList();
			}
			else
			{
				opponents = gameScreen.getAllyList();
			}
			
			Rectangle attackZone;
			// place attack zone based on the direction the character is facing
			if(character.getDirection() == Direction.RIGHT)
			{
				attackZone = new Rectangle(character.getX() + character.getWidth(),
						character.getY(), range, character.getHeight());
			}
			else if (character.getDirection() == Direction.LEFT)
			{
				attackZone = new Rectangle(character.getX() - character.getWidth(),
						character.getY(), range, character.getHeight());
			}
			else if (character.getDirection() == Direction.UP)
			{
				attackZone = new Rectangle(character.getX(),
						character.getY() + character.getHeight(),
						character.getWidth(), range);
			}
			else
			{
				attackZone = new Rectangle(character.getX(),
						character.getY() - character.getHeight(),
						character.getWidth(), range);
			}
			
			// if there is an opponent within the attack zone, apply damage
			for(Character opponent : opponents)
			{
				if(attackZone.overlaps(opponent.getCollider()))
				{
					opponent.attacked(damage, (GameObject) character);
				}
			}
			
			
			trees = gameScreen.getDeadTreeList();
			
			for(DeadTree object : trees)
			{
				if(attackZone.overlaps(object.getCollider()))
				{
					object.attacked(damage);
				}
			}
			
		}
		
		//destroy sound
		//warriorAttackSound.dispose();
		
	}

}
