/*******************************************************************************
f * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package game;

import gameobject.character.CharacterType;

import java.util.ArrayList;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.TextureRegion;

/**
 * Loads images and animations from the file system
 * @author jordan
 * @version Apr 16 2014
 */
public class AssetLoader {

	private static AssetLoader instance = null;

	private final int IMAGE_WIDTH = 64;
	private final int IMAGE_HEIGHT = 64;

	private final float ANIMATION_FRAME_TIME = 0.1f;

	private static ArrayList<Animation> warriorAnimations;
	private static ArrayList<Animation> mageAnimations;
	private static ArrayList<Animation> rogueAnimations;
	private static ArrayList<Animation> fighterAnimations;
	private static ArrayList<Animation> wizardAnimations;
	private static ArrayList<Animation> thiefAnimations;
	private static ArrayList<Animation> wolfAnimations;
	private static ArrayList<Animation> bossAnimations;
	private static ArrayList<Animation> npcMaleAnimations;
	private static ArrayList<Animation> npcFemaleAnimations;
	private static ArrayList<Animation> npcKidAnimations;
	private static Animation mageExplosion;
	private static Animation bossExplosion;
	private static Animation mageSparkles;
	private static Animation bossSparkles;
	private static Animation gateAnimation;
	private static Animation poisonCloudAnimation;
	private static TextureRegion arrowImage;
	
	private ArrayList<Texture> textures;

	/**
	 * Constructor
	 */
	private AssetLoader()
	{
		textures = new ArrayList<Texture>();
		
		makeAllyAnimations(CharacterType.WARRIOR);
		makeAllyAnimations(CharacterType.MAGE);
		makeAllyAnimations(CharacterType.ROGUE);
		makeEnemyAnimations(CharacterType.WARRIOR);
		makeEnemyAnimations(CharacterType.MAGE);
		makeEnemyAnimations(CharacterType.ROGUE);
		makeNPCAnimations("male");
		makeNPCAnimations("female");
		makeNPCAnimations("kid");
		makeWolfAnimations();
		makeBossAnimations();
		makeExplosionAnimation(true);
		makeExplosionAnimation(false);
		makeSparkleAnimation(true);
		makeSparkleAnimation(false);
		makeGateAnimation();
		makePoisonCloudAnimation();
		makeArrowImage();
	}

	/**
	 * Make new instance
	 */
	public static void makeNewInstance()
	{
		AssetLoader imageLoader = new AssetLoader();
		instance = imageLoader;
	}

	/**
	 * Get instance
	 * @return instance
	 */
	public static AssetLoader getInstance()
	{
		return instance;
	}

	/**
	 * Gets an image from the assets folder
	 * @param filePath the path to the image
	 * @return the image as a TextureRegion
	 */
	private TextureRegion getImage(String filePath)
	{
		Texture texture = new Texture(Gdx.files.internal(filePath));
		textures.add(texture);
		TextureRegion region = new TextureRegion(texture, 0, 0, IMAGE_WIDTH, IMAGE_HEIGHT);
		return region;
	}
	
	/**
	 * Get an image from the assets folder
	 * @param filePath the path to the image
	 * @return the image as a TextureRegion
	 */
	private TextureRegion getLargeImage(String filePath)
	{
		Texture texture = new Texture(Gdx.files.internal(filePath));
		textures.add(texture);
		TextureRegion region = new TextureRegion(texture, 0, 0, IMAGE_WIDTH*2, IMAGE_HEIGHT*2);
		return region;
	}
	
	/**
	 * Gets an image from the assets folder
	 * @param filePath the path to the image
	 * @return the image as a TextureRegion
	 */
	private TextureRegion getLongImage(String filePath)
	{
		Texture texture = new Texture(Gdx.files.internal(filePath));
		textures.add(texture);
		TextureRegion region = new TextureRegion(texture, 0, 0, IMAGE_WIDTH*2, IMAGE_HEIGHT);
		return region;
	}

	/**
	 * Gets an image from the assets folder for NPC child
	 * @param filePath the path to the image
	 * @return the image as a TextureRegion
	 */
	private TextureRegion getNPCImage(String filePath, boolean adult)
	{
		Texture texture = new Texture(Gdx.files.internal(filePath));
		textures.add(texture);
		TextureRegion region;
		if (adult)
		{
			region = new TextureRegion(texture, 0, 0, IMAGE_WIDTH, IMAGE_HEIGHT);
		}
		else
		{
			region = new TextureRegion(texture, 0, 0, IMAGE_WIDTH, IMAGE_WIDTH);
		}
		return region;

	}

	private void makeEnemyAnimations(CharacterType type)
	{
		ArrayList<Animation> animations = new ArrayList<Animation>();

		String imagePath;
		float idleOffset;
		if(type == CharacterType.WARRIOR)
		{
			imagePath = "assets/data/enemy_warrior_images/fighter";
			idleOffset = 4.0f;
		}
		else if(type == CharacterType.MAGE)
		{
			imagePath = "assets/data/enemy_mage_images/wizard";
			idleOffset = 5.0f;
		}
		else
		{
			imagePath = "assets/data/enemy_rogue_images/thief";
			idleOffset = 3.0f;
		}

		TextureRegion moveRight1 = getImage(imagePath + "_right_01.png");
		TextureRegion moveRight2 = getImage(imagePath + "_right_02.png");
		TextureRegion moveRight3 = getImage(imagePath + "_right_03.png");
		TextureRegion moveRight4 = getImage(imagePath + "_right_04.png");
		TextureRegion moveRight5 = getImage(imagePath + "_right_05.png");
		TextureRegion moveRight6 = getImage(imagePath + "_right_06.png");
		TextureRegion moveRight7 = getImage(imagePath + "_right_07.png");
		TextureRegion moveRight8 = getImage(imagePath + "_right_08.png");
		Animation movingRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveRight1, moveRight2, moveRight3,
				moveRight4, moveRight5, moveRight6, moveRight7, moveRight8);
		animations.add(movingRightAnimation);
		TextureRegion idleRight1 = getImage(imagePath + "_right_idle_01.png");
		TextureRegion idleRight2 = getImage(imagePath + "_right_idle_02.png");
		Animation facingRightAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleRight1, idleRight2);
		animations.add(facingRightAnimation);

		TextureRegion moveLeft1 = getImage(imagePath + "_left_01.png");
		TextureRegion moveLeft2 = getImage(imagePath + "_left_02.png");
		TextureRegion moveLeft3 = getImage(imagePath + "_left_03.png");
		TextureRegion moveLeft4 = getImage(imagePath + "_left_04.png");
		TextureRegion moveLeft5 = getImage(imagePath + "_left_05.png");
		TextureRegion moveLeft6 = getImage(imagePath + "_left_06.png");
		TextureRegion moveLeft7 = getImage(imagePath + "_left_07.png");
		TextureRegion moveLeft8 = getImage(imagePath + "_left_08.png");
		Animation movingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveLeft1, moveLeft2, moveLeft3,
				moveLeft4, moveLeft5, moveLeft6, moveLeft7, moveLeft8);
		animations.add(movingLeftAnimation);
		TextureRegion idleLeft1 = getImage(imagePath + "_left_idle_01.png");
		TextureRegion idleLeft2 = getImage(imagePath + "_left_idle_02.png");
		Animation facingLeftAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleLeft1, idleLeft2);
		animations.add(facingLeftAnimation);

		TextureRegion moveUp1 = getImage(imagePath + "_back_01.png");
		TextureRegion moveUp2 = getImage(imagePath + "_back_02.png");
		TextureRegion moveUp3 = getImage(imagePath + "_back_03.png");
		TextureRegion moveUp4 = getImage(imagePath + "_back_04.png");
		TextureRegion moveUp5 = getImage(imagePath + "_back_05.png");
		TextureRegion moveUp6 = getImage(imagePath + "_back_06.png");
		TextureRegion moveUp7 = getImage(imagePath + "_back_07.png");
		TextureRegion moveUp8 = getImage(imagePath + "_back_08.png");
		Animation movingUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveUp1, moveUp2, moveUp3,
				moveUp4, moveUp5, moveUp6, moveUp7, moveUp8);
		animations.add(movingUpAnimation);
		TextureRegion idleUp1 = getImage(imagePath + "_back_idle_01.png");
		TextureRegion idleUp2 = getImage(imagePath + "_back_idle_02.png");
		Animation facingUpAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleUp1, idleUp2);
		animations.add(facingUpAnimation);

		TextureRegion moveDown1 = getImage(imagePath + "_front_01.png");
		TextureRegion moveDown2 = getImage(imagePath + "_front_02.png");
		TextureRegion moveDown3 = getImage(imagePath + "_front_03.png");
		TextureRegion moveDown4 = getImage(imagePath + "_front_04.png");
		TextureRegion moveDown5 = getImage(imagePath + "_front_05.png");
		TextureRegion moveDown6 = getImage(imagePath + "_front_06.png");
		TextureRegion moveDown7 = getImage(imagePath + "_front_07.png");
		TextureRegion moveDown8 = getImage(imagePath + "_front_08.png");
		Animation movingDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1, moveDown2, moveDown3,
				moveDown4, moveDown5, moveDown6, moveDown7, moveDown8);
		animations.add(movingDownAnimation);
		TextureRegion idleDown1 = getImage(imagePath + "_front_idle_01.png");
		TextureRegion idleDown2 = getImage(imagePath + "_front_idle_02.png");
		Animation facingDownAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleDown1, idleDown2);
		animations.add(facingDownAnimation);

		//Attack animations

		if(type == CharacterType.WARRIOR)
		{
			TextureRegion attackRight1 = getImage(imagePath + "_right_attack_01.png");
			TextureRegion attackRight2 = getImage(imagePath + "_right_attack_02.png");
			TextureRegion attackRight3 = getImage(imagePath + "_right_attack_03.png");
			Animation attackingRightAnimation = new Animation(ANIMATION_FRAME_TIME, attackRight1, attackRight2,
					attackRight3);
			animations.add(attackingRightAnimation);

			TextureRegion attackLeft1 = getImage(imagePath + "_left_attack_01.png");
			TextureRegion attackLeft2 = getImage(imagePath + "_left_attack_02.png");
			TextureRegion attackLeft3 = getImage(imagePath + "_left_attack_03.png");
			Animation attackingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, attackLeft1, attackLeft2,
					attackLeft3);
			animations.add(attackingLeftAnimation);

			TextureRegion attackUp1 = getImage(imagePath + "_back_attack_01.png");
			TextureRegion attackUp2 = getImage(imagePath + "_back_attack_02.png");
			Animation attackingUpAnimation = new Animation(ANIMATION_FRAME_TIME, attackUp1, attackUp2);
			animations.add(attackingUpAnimation);

			TextureRegion attackDown1 = getImage(imagePath + "_front_attack_01.png");
			TextureRegion attackDown2 = getImage(imagePath + "_front_attack_02.png");
			Animation attackingDownAnimation = new Animation(ANIMATION_FRAME_TIME, attackDown1, attackDown2);
			animations.add(attackingDownAnimation);
		}
		else if(type == CharacterType.MAGE)
		{
			TextureRegion attackRight1 = getImage(imagePath + "_right_heal_01.png");
			TextureRegion attackRight2 = getImage(imagePath + "_right_heal_02.png");
			TextureRegion attackRight3 = getImage(imagePath + "_right_heal_03.png");
			Animation attackingRightAnimation = new Animation(ANIMATION_FRAME_TIME, attackRight1, attackRight2,
					attackRight3);
			animations.add(attackingRightAnimation);

			TextureRegion attackLeft1 = getImage(imagePath + "_left_heal_01.png");
			TextureRegion attackLeft2 = getImage(imagePath + "_left_heal_02.png");
			TextureRegion attackLeft3 = getImage(imagePath + "_left_heal_03.png");
			Animation attackingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, attackLeft1, attackLeft2,
					attackLeft3);
			animations.add(attackingLeftAnimation);

			TextureRegion attackUp1 = getImage(imagePath + "_back_heal_01.png");
			TextureRegion attackUp2 = getImage(imagePath + "_back_heal_02.png");
			TextureRegion attackUp3 = getImage(imagePath + "_back_heal_03.png");
			Animation attackingUpAnimation = new Animation(ANIMATION_FRAME_TIME, attackUp1, attackUp2,
					attackUp3);
			animations.add(attackingUpAnimation);

			TextureRegion attackDown1 = getImage(imagePath + "_front_heal_01.png");
			TextureRegion attackDown2 = getImage(imagePath + "_front_heal_02.png");
			TextureRegion attackDown3 = getImage(imagePath + "_front_heal_03.png");
			Animation attackingDownAnimation = new Animation(ANIMATION_FRAME_TIME, attackDown1, attackDown2,
					attackDown3);
			animations.add(attackingDownAnimation);
		}
		else
		{
			TextureRegion attackRight1 = getImage(imagePath + "_right_attack_01.png");
			TextureRegion attackRight2 = getImage(imagePath + "_right_attack_02.png");
			TextureRegion attackRight3 = getImage(imagePath + "_right_attack_03.png");
			//TextureRegion attackRight4 = getCharacterImage(imagePath + "_right_attack_04.png");
			Animation attackingRightAnimation = new Animation(ANIMATION_FRAME_TIME, attackRight1, attackRight2,
					attackRight3);
			animations.add(attackingRightAnimation);

			TextureRegion attackLeft1 = getImage(imagePath + "_left_attack_01.png");
			TextureRegion attackLeft2 = getImage(imagePath + "_left_attack_02.png");
			TextureRegion attackLeft3 = getImage(imagePath + "_left_attack_03.png");
			//TextureRegion attackLeft4 = getCharacterImage(imagePath + "_left_attack_04.png");
			Animation attackingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, attackLeft1, attackLeft2,
					attackLeft3);
			animations.add(attackingLeftAnimation);

			TextureRegion attackUp1 = getImage(imagePath + "_back_attack_01.png");
			TextureRegion attackUp2 = getImage(imagePath + "_back_attack_02.png");
			TextureRegion attackUp3 = getImage(imagePath + "_back_attack_03.png");
			Animation attackingUpAnimation = new Animation(ANIMATION_FRAME_TIME, attackUp1, attackUp2,
					attackUp3);
			animations.add(attackingUpAnimation);

			TextureRegion attackDown1 = getImage(imagePath + "_front_attack_01.png");
			TextureRegion attackDown2 = getImage(imagePath + "_front_attack_02.png");
			TextureRegion attackDown3 = getImage(imagePath + "_front_attack_03.png");
			Animation attackingDownAnimation = new Animation(ANIMATION_FRAME_TIME, attackDown1, attackDown2,
					attackDown3);
			animations.add(attackingDownAnimation);
		}

		// Defending Animations

		if(type == CharacterType.WARRIOR)
		{
			TextureRegion defendRight1 = getImage(imagePath + "_right_shield_01.png");
			TextureRegion defendRight2 = getImage(imagePath + "_right_shield_02.png");
			Animation defendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendRight1, defendRight2);
			animations.add(defendingRightAnimation);

			TextureRegion defendLeft1 = getImage(imagePath + "_left_shield_01.png");
			TextureRegion defendLeft2 = getImage(imagePath + "_left_shield_02.png");
			Animation defendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendLeft1, defendLeft2);
			animations.add(defendingLeftAnimation);

			TextureRegion defendUp1 = getImage(imagePath + "_back_shield_01.png");
			TextureRegion defendUp2 = getImage(imagePath + "_back_shield_02.png");
			Animation defendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendUp1, defendUp2);
			animations.add(defendingUpAnimation);

			TextureRegion defendDown1 = getImage(imagePath + "_front_shield_01.png");
			TextureRegion defendDown2 = getImage(imagePath + "_front_shield_02.png");
			Animation defendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendDown1, defendDown2);
			animations.add(defendingDownAnimation);
		}
		else if(type == CharacterType.MAGE)
		{
			TextureRegion defendRight1 = getImage(imagePath + "_right_shield_01.png");
			TextureRegion defendRight2 = getImage(imagePath + "_right_shield_02.png");
			TextureRegion defendRight3 = getImage(imagePath + "_right_shield_03.png");
			Animation defendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendRight1, defendRight2,
					defendRight3);
			animations.add(defendingRightAnimation);

			TextureRegion defendLeft1 = getImage(imagePath + "_left_shield_01.png");
			TextureRegion defendLeft2 = getImage(imagePath + "_left_shield_02.png");
			TextureRegion defendLeft3 = getImage(imagePath + "_left_shield_03.png");
			Animation defendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendLeft1, defendLeft2,
					defendLeft3);
			animations.add(defendingLeftAnimation);

			TextureRegion defendUp1 = getImage(imagePath + "_back_shield_01.png");
			TextureRegion defendUp2 = getImage(imagePath + "_back_shield_02.png");
			TextureRegion defendUp3 = getImage(imagePath + "_back_shield_03.png");
			Animation defendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendUp1, defendUp2,
					defendUp3);
			animations.add(defendingUpAnimation);

			TextureRegion defendDown1 = getImage(imagePath + "_front_shield_01.png");
			TextureRegion defendDown2 = getImage(imagePath + "_front_shield_02.png");
			TextureRegion defendDown3 = getImage(imagePath + "_front_shield_03.png");
			Animation defendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendDown1, defendDown2,
					defendDown3);
			animations.add(defendingDownAnimation);
		}
		else
		{
			TextureRegion defendRight1 = getImage(imagePath + "_right_cloaking_01.png");
			TextureRegion defendRight2 = getImage(imagePath + "_right_cloaking_02.png");
			TextureRegion defendRight3 = getImage(imagePath + "_right_cloaking_03.png");
			TextureRegion defendRight4 = getImage(imagePath + "_right_cloaking_04.png");
			TextureRegion defendRight5 = getImage(imagePath + "_right_cloaked.png");
			Animation defendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendRight1, defendRight2,
					defendRight3, defendRight4, defendRight5);
			animations.add(defendingRightAnimation);

			TextureRegion defendLeft1 = getImage(imagePath + "_left_cloaking_01.png");
			TextureRegion defendLeft2 = getImage(imagePath + "_left_cloaking_02.png");
			TextureRegion defendLeft3 = getImage(imagePath + "_left_cloaking_03.png");
			TextureRegion defendLeft4 = getImage(imagePath + "_left_cloaking_04.png");
			TextureRegion defendLeft5 = getImage(imagePath + "_left_cloaked.png");
			Animation defendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendLeft1, defendLeft2,
					defendLeft3, defendLeft4, defendLeft5);
			animations.add(defendingLeftAnimation);

			TextureRegion defendUp1 = getImage(imagePath + "_back_cloaking_01.png");
			TextureRegion defendUp2 = getImage(imagePath + "_back_cloaking_02.png");
			TextureRegion defendUp3 = getImage(imagePath + "_back_cloaking_03.png");
			TextureRegion defendUp4 = getImage(imagePath + "_back_cloaking_04.png");
			TextureRegion defendUp5 = getImage(imagePath + "_back_cloaked.png");
			Animation defendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendUp1, defendUp2,
					defendUp3, defendUp4, defendUp5);
			animations.add(defendingUpAnimation);

			TextureRegion defendDown1 = getImage(imagePath + "_front_cloaking_01.png");
			TextureRegion defendDown2 = getImage(imagePath + "_front_cloaking_02.png");
			TextureRegion defendDown3 = getImage(imagePath + "_front_cloaking_03.png");
			TextureRegion defendDown4 = getImage(imagePath + "_front_cloaking_04.png");
			TextureRegion defendDown5 = getImage(imagePath + "_front_cloaked.png");
			Animation defendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendDown1, defendDown2,
					defendDown3, defendDown4, defendDown5);
			animations.add(defendingDownAnimation);
		}

		//dying animations


		TextureRegion deadRight1 = getImage(imagePath + "_right_dying_01.png");
		TextureRegion deadRight2 = getImage(imagePath + "_right_dying_02.png");
		TextureRegion deadRight3 = getImage(imagePath + "_right_dying_03.png");
		TextureRegion deadRight4 = getImage(imagePath + "_right_dead.png");
		Animation deadRightAnimation = new Animation(ANIMATION_FRAME_TIME, deadRight1, deadRight2, deadRight3, 
				deadRight4);
		animations.add(deadRightAnimation);

		TextureRegion deadLeft1 = getImage(imagePath + "_left_dying_01.png");
		TextureRegion deadLeft2 = getImage(imagePath + "_left_dying_02.png");
		TextureRegion deadLeft3 = getImage(imagePath + "_left_dying_03.png");
		TextureRegion deadLeft4 = getImage(imagePath + "_left_dead.png");
		Animation deadLeftAnimation = new Animation(ANIMATION_FRAME_TIME, deadLeft1, deadLeft2, 
				deadLeft3, deadLeft4);
		animations.add(deadLeftAnimation);

		TextureRegion deadUp1 = getImage(imagePath + "_back_dying_01.png");
		TextureRegion deadUp2 = getImage(imagePath + "_back_dying_02.png");
		TextureRegion deadUp3 = getImage(imagePath + "_back_dying_03.png");
		TextureRegion deadUp4 = getImage(imagePath + "_back_dead.png");
		Animation deadUpAnimation = new Animation(ANIMATION_FRAME_TIME, deadUp1, deadUp2, deadUp3, 
				deadUp4);
		animations.add(deadUpAnimation);

		TextureRegion deadDown1 = getImage(imagePath + "_front_dying_01.png");
		TextureRegion deadDown2 = getImage(imagePath + "_front_dying_02.png");
		TextureRegion deadDown3 = getImage(imagePath + "_front_dying_03.png");
		TextureRegion deadDown4 = getImage(imagePath + "_front_dead.png");
		Animation deadDownAnimation = new Animation(ANIMATION_FRAME_TIME, deadDown1, deadDown2, 
				deadDown3, deadDown4);
		animations.add(deadDownAnimation);

		//special animations

		if(type == CharacterType.WARRIOR)
		{
			TextureRegion specialRight1 = getImage(imagePath + "_right_yell_01.png");
			TextureRegion specialRight2 = getImage(imagePath + "_right_yell_02.png");
			Animation specialRightAnimation = new Animation(ANIMATION_FRAME_TIME, specialRight1, specialRight2);
			animations.add(specialRightAnimation);

			TextureRegion specialLeft1 = getImage(imagePath + "_left_yell_01.png");
			TextureRegion specialLeft2 = getImage(imagePath + "_left_yell_02.png");
			Animation specialLeftAnimation = new Animation(ANIMATION_FRAME_TIME, specialLeft1, specialLeft2);
			animations.add(specialLeftAnimation);

			TextureRegion specialUp1 = getImage(imagePath + "_back_yell_01.png");
			TextureRegion specialUp2 = getImage(imagePath + "_back_yell_02.png");
			Animation specialUpAnimation = new Animation(ANIMATION_FRAME_TIME, specialUp1, specialUp2);
			animations.add(specialUpAnimation);

			TextureRegion specialDown1 = getImage(imagePath + "_front_yell_01.png");
			TextureRegion specialDown2 = getImage(imagePath + "_front_yell_02.png");
			Animation specialDownAnimation = new Animation(ANIMATION_FRAME_TIME, specialDown1, specialDown2);
			animations.add(specialDownAnimation);
		}
		else if(type == CharacterType.MAGE)
		{
			TextureRegion specialRight1 = getImage(imagePath + "_right_heal_01.png");
			TextureRegion specialRight2 = getImage(imagePath + "_right_heal_02.png");
			TextureRegion specialRight3 = getImage(imagePath + "_right_heal_03.png");
			Animation specialRightAnimation = new Animation(ANIMATION_FRAME_TIME, specialRight1, specialRight2,
					specialRight3);
			animations.add(specialRightAnimation);

			TextureRegion specialLeft1 = getImage(imagePath + "_left_heal_01.png");
			TextureRegion specialLeft2 = getImage(imagePath + "_left_heal_02.png");
			TextureRegion specialLeft3 = getImage(imagePath + "_left_heal_03.png");
			Animation specialLeftAnimation = new Animation(ANIMATION_FRAME_TIME, specialLeft1, specialLeft2,
					specialLeft3);
			animations.add(specialLeftAnimation);

			TextureRegion specialUp1 = getImage(imagePath + "_back_heal_01.png");
			TextureRegion specialUp2 = getImage(imagePath + "_back_heal_02.png");
			TextureRegion specialUp3 = getImage(imagePath + "_back_heal_03.png");
			Animation specialUpAnimation = new Animation(ANIMATION_FRAME_TIME, specialUp1, specialUp2,
					specialUp3);
			animations.add(specialUpAnimation);

			TextureRegion specialDown1 = getImage(imagePath + "_front_heal_01.png");
			TextureRegion specialDown2 = getImage(imagePath + "_front_heal_02.png");
			TextureRegion specialDown3 = getImage(imagePath + "_front_heal_03.png");
			Animation specialDownAnimation = new Animation(ANIMATION_FRAME_TIME, specialDown1, specialDown2,
					specialDown3);
			animations.add(specialDownAnimation);
		}
		else
		{
			TextureRegion specialRight1 = getImage(imagePath + "_right_special_01.png");
			Animation specialRightAnimation = new Animation(ANIMATION_FRAME_TIME, specialRight1);
			animations.add(specialRightAnimation);

			TextureRegion specialLeft1 = getImage(imagePath + "_left_special_01.png");
			Animation specialLeftAnimation = new Animation(ANIMATION_FRAME_TIME, specialLeft1);
			animations.add(specialLeftAnimation);

			TextureRegion specialUp1 = getImage(imagePath + "_back_special_01.png");
			Animation specialUpAnimation = new Animation(ANIMATION_FRAME_TIME, specialUp1);
			animations.add(specialUpAnimation);

			TextureRegion specialDown1 = getImage(imagePath + "_front_special_01.png");
			Animation specialDownAnimation = new Animation(ANIMATION_FRAME_TIME, specialDown1);
			animations.add(specialDownAnimation);
		}

		// potion animations
		if(type == CharacterType.WARRIOR)
		{
			TextureRegion potionRight1 = getImage(imagePath + "_right_potion_01.png");
			TextureRegion potionRight2 = getImage(imagePath + "_right_potion_02.png");
			Animation potionRightAnimation = new Animation(ANIMATION_FRAME_TIME, potionRight1, potionRight2);
			animations.add(potionRightAnimation);

			TextureRegion potionLeft1 = getImage(imagePath + "_left_potion_01.png");
			TextureRegion potionLeft2 = getImage(imagePath + "_left_potion_02.png");
			Animation potionLeftAnimation = new Animation(ANIMATION_FRAME_TIME, potionLeft1, potionLeft2);
			animations.add(potionLeftAnimation);

			TextureRegion potionUp1 = getImage(imagePath + "_back_potion_01.png");
			TextureRegion potionUp2 = getImage(imagePath + "_back_potion_02.png");
			Animation potionUpAnimation = new Animation(ANIMATION_FRAME_TIME, potionUp1, potionUp2);
			animations.add(potionUpAnimation);

			TextureRegion potionDown1 = getImage(imagePath + "_front_potion_01.png");
			TextureRegion potionDown2 = getImage(imagePath + "_front_potion_02.png");
			Animation potionDownAnimation = new Animation(ANIMATION_FRAME_TIME, potionDown1, potionDown2);
			animations.add(potionDownAnimation);
		}
		else
		{
			TextureRegion potionRight1 = getImage(imagePath + "_right_potion_01.png");
			TextureRegion potionRight2 = getImage(imagePath + "_right_potion_02.png");
			TextureRegion potionRight3 = getImage(imagePath + "_right_potion_03.png");
			Animation potionRightAnimation = new Animation(ANIMATION_FRAME_TIME, potionRight1, potionRight2,
					potionRight3);
			animations.add(potionRightAnimation);

			TextureRegion potionLeft1 = getImage(imagePath + "_left_potion_01.png");
			TextureRegion potionLeft2 = getImage(imagePath + "_left_potion_02.png");
			TextureRegion potionLeft3 = getImage(imagePath + "_left_potion_03.png");
			Animation potionLeftAnimation = new Animation(ANIMATION_FRAME_TIME, potionLeft1, potionLeft2,
					potionLeft3);
			animations.add(potionLeftAnimation);

			TextureRegion potionUp1 = getImage(imagePath + "_back_potion_01.png");
			TextureRegion potionUp2 = getImage(imagePath + "_back_potion_02.png");
			TextureRegion potionUp3 = getImage(imagePath + "_back_potion_03.png");
			Animation potionUpAnimation = new Animation(ANIMATION_FRAME_TIME, potionUp1, potionUp2,
					potionUp3);
			animations.add(potionUpAnimation);

			TextureRegion potionDown1 = getImage(imagePath + "_front_potion_01.png");
			TextureRegion potionDown2 = getImage(imagePath + "_front_potion_02.png");
			TextureRegion potionDown3 = getImage(imagePath + "_front_potion_03.png");
			Animation potionDownAnimation = new Animation(ANIMATION_FRAME_TIME, potionDown1, potionDown2,
					potionDown3);
			animations.add(potionDownAnimation);
		}

		//revive animations

		TextureRegion reviveRight1 = getImage(imagePath + "_right_revive_01.png");
		TextureRegion reviveRight2 = getImage(imagePath + "_right_revive_02.png");
		TextureRegion reviveRight3 = getImage(imagePath + "_right_revive_03.png");
		Animation reviveRightAnimation = new Animation(ANIMATION_FRAME_TIME, reviveRight1, reviveRight2, 
				reviveRight3);
		animations.add(reviveRightAnimation);

		TextureRegion reviveLeft1 = getImage(imagePath + "_left_revive_01.png");
		TextureRegion reviveLeft2 = getImage(imagePath + "_left_revive_02.png");
		TextureRegion reviveLeft3 = getImage(imagePath + "_left_revive_03.png");
		Animation reviveLeftAnimation = new Animation(ANIMATION_FRAME_TIME, reviveLeft1, reviveLeft2, 
				reviveLeft3);
		animations.add(reviveLeftAnimation);

		TextureRegion reviveUp1 = getImage(imagePath + "_back_revive_01.png");
		TextureRegion reviveUp2 = getImage(imagePath + "_back_revive_02.png");
		TextureRegion reviveUp3 = getImage(imagePath + "_back_revive_03.png");
		Animation reviveUpAnimation = new Animation(ANIMATION_FRAME_TIME, reviveUp1, reviveUp2, reviveUp3);
		animations.add(reviveUpAnimation);

		TextureRegion reviveDown1 = getImage(imagePath + "_front_revive_01.png");
		TextureRegion reviveDown2 = getImage(imagePath + "_front_revive_02.png");
		TextureRegion reviveDown3 = getImage(imagePath + "_front_revive_03.png");
		Animation reviveDownAnimation = new Animation(ANIMATION_FRAME_TIME, reviveDown1, reviveDown2, 
				reviveDown3);
		animations.add(reviveDownAnimation);

		// end defending animations
		if(type == CharacterType.WARRIOR)
		{
			TextureRegion defendRight1 = getImage(imagePath + "_right_shield_01.png");
			TextureRegion defendRight2 = getImage(imagePath + "_right_shield_02.png");
			Animation endDefendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendRight2,
					defendRight1);
			animations.add(endDefendingRightAnimation);

			TextureRegion defendLeft1 = getImage(imagePath + "_left_shield_01.png");
			TextureRegion defendLeft2 = getImage(imagePath + "_left_shield_02.png");
			Animation endDefendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendLeft2,
					defendLeft1);
			animations.add(endDefendingLeftAnimation);

			TextureRegion defendUp1 = getImage(imagePath + "_back_shield_01.png");
			TextureRegion defendUp2 = getImage(imagePath + "_back_shield_02.png");
			Animation endDefendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendUp2,
					defendUp1);
			animations.add(endDefendingUpAnimation);

			TextureRegion defendDown1 = getImage(imagePath + "_front_shield_01.png");
			TextureRegion defendDown2 = getImage(imagePath + "_front_shield_02.png");
			Animation endDefendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendDown2,
					defendDown1);
			animations.add(endDefendingDownAnimation);
		}
		else if(type == CharacterType.MAGE)
		{
			TextureRegion defendRight1 = getImage(imagePath + "_right_shield_01.png");
			TextureRegion defendRight2 = getImage(imagePath + "_right_shield_02.png");
			TextureRegion defendRight3 = getImage(imagePath + "_right_shield_03.png");
			Animation endDefendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendRight3,
					defendRight2, defendRight1);
			animations.add(endDefendingRightAnimation);

			TextureRegion defendLeft1 = getImage(imagePath + "_left_shield_01.png");
			TextureRegion defendLeft2 = getImage(imagePath + "_left_shield_02.png");
			TextureRegion defendLeft3 = getImage(imagePath + "_left_shield_03.png");
			Animation endDefendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendLeft3,
					defendLeft2, defendLeft1);
			animations.add(endDefendingLeftAnimation);

			TextureRegion defendUp1 = getImage(imagePath + "_back_shield_01.png");
			TextureRegion defendUp2 = getImage(imagePath + "_back_shield_02.png");
			TextureRegion defendUp3 = getImage(imagePath + "_back_shield_03.png");
			Animation endDefendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendUp3,
					defendUp2, defendUp1);
			animations.add(endDefendingUpAnimation);

			TextureRegion defendDown1 = getImage(imagePath + "_front_shield_01.png");
			TextureRegion defendDown2 = getImage(imagePath + "_front_shield_02.png");
			TextureRegion defendDown3 = getImage(imagePath + "_front_shield_03.png");
			Animation endDefendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendDown3,
					defendDown2, defendDown1);
			animations.add(endDefendingDownAnimation);
		}
		else
		{
			TextureRegion defendRight1 = getImage(imagePath + "_right_cloaking_01.png");
			TextureRegion defendRight2 = getImage(imagePath + "_right_cloaking_02.png");
			TextureRegion defendRight3 = getImage(imagePath + "_right_cloaking_03.png");
			TextureRegion defendRight4 = getImage(imagePath + "_right_cloaking_04.png");
			TextureRegion defendRight5 = getImage(imagePath + "_right_cloaked.png");
			Animation endDefendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendRight5, defendRight4,
					defendRight3, defendRight2, defendRight1);
			animations.add(endDefendingRightAnimation);

			TextureRegion defendLeft1 = getImage(imagePath + "_left_cloaking_01.png");
			TextureRegion defendLeft2 = getImage(imagePath + "_left_cloaking_02.png");
			TextureRegion defendLeft3 = getImage(imagePath + "_left_cloaking_03.png");
			TextureRegion defendLeft4 = getImage(imagePath + "_left_cloaking_04.png");
			TextureRegion defendLeft5 = getImage(imagePath + "_left_cloaked.png");
			Animation endDefendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendLeft5, defendLeft4, 
					defendLeft3, defendLeft2, defendLeft1);
			animations.add(endDefendingLeftAnimation);

			TextureRegion defendUp1 = getImage(imagePath + "_back_cloaking_01.png");
			TextureRegion defendUp2 = getImage(imagePath + "_back_cloaking_02.png");
			TextureRegion defendUp3 = getImage(imagePath + "_back_cloaking_03.png");
			TextureRegion defendUp4 = getImage(imagePath + "_back_cloaking_04.png");
			TextureRegion defendUp5 = getImage(imagePath + "_back_cloaked.png");
			Animation endDefendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendUp5, defendUp4,
					defendUp3, defendUp2, defendUp1);
			animations.add(endDefendingUpAnimation);

			TextureRegion defendDown1 = getImage(imagePath + "_front_cloaking_01.png");
			TextureRegion defendDown2 = getImage(imagePath + "_front_cloaking_02.png");
			TextureRegion defendDown3 = getImage(imagePath + "_front_cloaking_03.png");
			TextureRegion defendDown4 = getImage(imagePath + "_front_cloaking_04.png");
			TextureRegion defendDown5 = getImage(imagePath + "_front_cloaked.png");
			Animation endDefendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendDown5, defendDown4,
					defendDown3, defendDown2, defendDown1);
			animations.add(endDefendingDownAnimation);
		}

		// pain animations

		TextureRegion rightPain = getImage(imagePath + "_right_pain.png");
		Animation rightPainAnimation = new Animation(ANIMATION_FRAME_TIME, rightPain);
		animations.add(rightPainAnimation);
		TextureRegion leftPain = getImage(imagePath + "_left_pain.png");
		Animation leftPainAnimation = new Animation(ANIMATION_FRAME_TIME, leftPain);
		animations.add(leftPainAnimation);
		TextureRegion backPain = getImage(imagePath + "_back_pain.png");
		Animation backPainAnimation = new Animation(ANIMATION_FRAME_TIME, backPain);
		animations.add(backPainAnimation);
		TextureRegion frontPain = getImage(imagePath + "_front_pain.png");
		Animation frontPainAnimation = new Animation(ANIMATION_FRAME_TIME, frontPain);
		animations.add(frontPainAnimation);

		if(type == CharacterType.WARRIOR)
		{
			fighterAnimations = animations;
		}
		else if(type == CharacterType.MAGE)
		{
			wizardAnimations = animations;
		}
		else
		{
			thiefAnimations = animations;
		}
	}

	private void makeAllyAnimations(CharacterType type)
	{
		// animations should match the order in Character.makeAnimations
		ArrayList<Animation> animations = new ArrayList<Animation> ();

		String imagePath;
		float idleOffset;
		if(type == CharacterType.WARRIOR)
		{
			imagePath = "assets/data/warrior_images/warrior";
			idleOffset = 4.0f;
		}
		else if(type == CharacterType.MAGE)
		{
			imagePath = "assets/data/mage_images/mage";
			idleOffset = 5.0f;
		}
		else
		{
			imagePath = "assets/data/rogue_images/rogue";
			idleOffset = 3.0f;
		}

		TextureRegion moveRight1 = getImage(imagePath + "_right_01.png");
		TextureRegion moveRight2 = getImage(imagePath + "_right_02.png");
		TextureRegion moveRight3 = getImage(imagePath + "_right_03.png");
		TextureRegion moveRight4 = getImage(imagePath + "_right_04.png");
		TextureRegion moveRight5 = getImage(imagePath + "_right_05.png");
		TextureRegion moveRight6 = getImage(imagePath + "_right_06.png");
		TextureRegion moveRight7 = getImage(imagePath + "_right_07.png");
		TextureRegion moveRight8 = getImage(imagePath + "_right_08.png");
		TextureRegion idleRight1 = getImage(imagePath + "_right_stand_01.png");
		TextureRegion idleRight2 = getImage(imagePath + "_right_stand_02.png");
		Animation movingRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveRight1, moveRight2, moveRight3,
				moveRight4, moveRight5, moveRight6, moveRight7, moveRight8);
		animations.add(movingRightAnimation);
		Animation facingRightAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleRight1, idleRight2);
		animations.add(facingRightAnimation);

		TextureRegion moveLeft1 = getImage(imagePath + "_left_01.png");
		TextureRegion moveLeft2 = getImage(imagePath + "_left_02.png");
		TextureRegion moveLeft3 = getImage(imagePath + "_left_03.png");
		TextureRegion moveLeft4 = getImage(imagePath + "_left_04.png");
		TextureRegion moveLeft5 = getImage(imagePath + "_left_05.png");
		TextureRegion moveLeft6 = getImage(imagePath + "_left_06.png");
		TextureRegion moveLeft7 = getImage(imagePath + "_left_07.png");
		TextureRegion moveLeft8 = getImage(imagePath + "_left_08.png");
		TextureRegion idleLeft1 = getImage(imagePath + "_left_stand_01.png");
		TextureRegion idleLeft2 = getImage(imagePath + "_left_stand_02.png");
		Animation movingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveLeft1, moveLeft2, moveLeft3,
				moveLeft4, moveLeft5, moveLeft6, moveLeft7, moveLeft8);
		animations.add(movingLeftAnimation);
		Animation facingLeftAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleLeft1, idleLeft2);
		animations.add(facingLeftAnimation);

		TextureRegion moveUp1 = getImage(imagePath + "_back_01.png");
		TextureRegion moveUp2 = getImage(imagePath + "_back_02.png");
		TextureRegion moveUp3 = getImage(imagePath + "_back_03.png");
		TextureRegion moveUp4 = getImage(imagePath + "_back_04.png");
		TextureRegion moveUp5 = getImage(imagePath + "_back_05.png");
		TextureRegion moveUp6 = getImage(imagePath + "_back_06.png");
		TextureRegion moveUp7 = getImage(imagePath + "_back_07.png");
		TextureRegion moveUp8 = getImage(imagePath + "_back_08.png");
		TextureRegion idleUp1 = getImage(imagePath + "_back_stand_01.png");
		TextureRegion idleUp2 = getImage(imagePath + "_back_stand_02.png");
		Animation movingUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveUp1, moveUp2, moveUp3,
				moveUp4, moveUp5, moveUp6, moveUp7, moveUp8);
		animations.add(movingUpAnimation);
		Animation facingUpAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleUp1, idleUp2);
		animations.add(facingUpAnimation);

		TextureRegion moveDown1 = getImage(imagePath + "_front_01.png");
		TextureRegion moveDown2 = getImage(imagePath + "_front_02.png");
		TextureRegion moveDown3 = getImage(imagePath + "_front_03.png");
		TextureRegion moveDown4 = getImage(imagePath + "_front_04.png");
		TextureRegion moveDown5 = getImage(imagePath + "_front_05.png");
		TextureRegion moveDown6 = getImage(imagePath + "_front_06.png");
		TextureRegion moveDown7 = getImage(imagePath + "_front_07.png");
		TextureRegion moveDown8 = getImage(imagePath + "_front_08.png");
		TextureRegion idleDown1 = getImage(imagePath + "_front_stand_01.png");
		TextureRegion idleDown2 = getImage(imagePath + "_front_stand_02.png");
		Animation movingDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1, moveDown2, moveDown3,
				moveDown4, moveDown5, moveDown6, moveDown7, moveDown8);
		animations.add(movingDownAnimation);
		Animation facingDownAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleDown1, idleDown2);
		animations.add(facingDownAnimation);

		//Attack animations

		if(type == CharacterType.WARRIOR)
		{
			TextureRegion attackRight1 = getImage(imagePath + "_right_attack_01.png");
			TextureRegion attackRight2 = getImage(imagePath + "_right_attack_02.png");
			TextureRegion attackRight3 = getImage(imagePath + "_right_attack_03.png");
			TextureRegion attackRight4 = getImage(imagePath + "_right_attack_04.png");
			Animation attackingRightAnimation = new Animation(ANIMATION_FRAME_TIME, attackRight1, attackRight2,
					attackRight3, attackRight4);
			animations.add(attackingRightAnimation);

			TextureRegion attackLeft1 = getImage(imagePath + "_left_attack_01.png");
			TextureRegion attackLeft2 = getImage(imagePath + "_left_attack_02.png");
			TextureRegion attackLeft3 = getImage(imagePath + "_left_attack_03.png");
			TextureRegion attackLeft4 = getImage(imagePath + "_left_attack_04.png");
			Animation attackingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, attackLeft1, attackLeft2,
					attackLeft3, attackLeft4);
			animations.add(attackingLeftAnimation);

			TextureRegion attackUp1 = getImage(imagePath + "_back_attack_01.png");
			TextureRegion attackUp2 = getImage(imagePath + "_back_attack_02.png");
			Animation attackingUpAnimation = new Animation(ANIMATION_FRAME_TIME, attackUp2, attackUp1,
					attackUp2);
			animations.add(attackingUpAnimation);

			TextureRegion attackDown1 = getImage(imagePath + "_front_attack_01.png");
			TextureRegion attackDown2 = getImage(imagePath + "_front_attack_02.png");
			TextureRegion attackDown3 = getImage(imagePath + "_front_attack_01.png");
			Animation attackingDownAnimation = new Animation(ANIMATION_FRAME_TIME, attackDown2, attackDown1,
					attackDown2);
			animations.add(attackingDownAnimation);
		}
		else if(type == CharacterType.MAGE)
		{
			TextureRegion attackRight1 = getImage(imagePath + "_right_heal_01.png");
			TextureRegion attackRight2 = getImage(imagePath + "_right_heal_02.png");
			TextureRegion attackRight3 = getImage(imagePath + "_right_heal_03.png");
			TextureRegion attackRight4 = getImage(imagePath + "_right_heal_04.png");
			TextureRegion attackRight5 = getImage(imagePath + "_right_heal_05.png");
			Animation attackingRightAnimation = new Animation(ANIMATION_FRAME_TIME, attackRight1, attackRight2,
					attackRight3, attackRight4, attackRight5);
			animations.add(attackingRightAnimation);

			TextureRegion attackLeft1 = getImage(imagePath + "_left_heal_01.png");
			TextureRegion attackLeft2 = getImage(imagePath + "_left_heal_02.png");
			TextureRegion attackLeft3 = getImage(imagePath + "_left_heal_03.png");
			TextureRegion attackLeft4 = getImage(imagePath + "_left_heal_04.png");
			TextureRegion attackLeft5 = getImage(imagePath + "_left_heal_05.png");
			Animation attackingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, attackLeft1, attackLeft2,
					attackLeft3, attackLeft4, attackLeft5);
			animations.add(attackingLeftAnimation);

			TextureRegion attackUp1 = getImage(imagePath + "_back_heal_01.png");
			TextureRegion attackUp2 = getImage(imagePath + "_back_heal_02.png");
			TextureRegion attackUp3 = getImage(imagePath + "_back_heal_03.png");
			TextureRegion attackUp4 = getImage(imagePath + "_back_heal_04.png");
			TextureRegion attackUp5 = getImage(imagePath + "_back_heal_05.png");
			Animation attackingUpAnimation = new Animation(ANIMATION_FRAME_TIME, attackUp1, attackUp2,
					attackUp3, attackUp4, attackUp5);
			animations.add(attackingUpAnimation);

			TextureRegion attackDown1 = getImage(imagePath + "_front_heal_01.png");
			TextureRegion attackDown2 = getImage(imagePath + "_front_heal_02.png");
			TextureRegion attackDown3 = getImage(imagePath + "_front_heal_03.png");
			TextureRegion attackDown4 = getImage(imagePath + "_front_heal_04.png");
			TextureRegion attackDown5 = getImage(imagePath + "_front_heal_05.png");
			Animation attackingDownAnimation = new Animation(ANIMATION_FRAME_TIME, attackDown1, attackDown2,
					attackDown3, attackDown4, attackDown5);
			animations.add(attackingDownAnimation);
		}
		else
		{
			TextureRegion attackRight1 = getImage(imagePath + "_right_attack_01.png");
			TextureRegion attackRight2 = getImage(imagePath + "_right_attack_02.png");
			TextureRegion attackRight3 = getImage(imagePath + "_right_attack_03.png");
			//TextureRegion attackRight4 = getCharacterImage(imagePath + "_right_attack_04.png");
			Animation attackingRightAnimation = new Animation(ANIMATION_FRAME_TIME, attackRight1, attackRight2,
					attackRight3);
			animations.add(attackingRightAnimation);

			TextureRegion attackLeft1 = getImage(imagePath + "_left_attack_01.png");
			TextureRegion attackLeft2 = getImage(imagePath + "_left_attack_02.png");
			TextureRegion attackLeft3 = getImage(imagePath + "_left_attack_03.png");
			//TextureRegion attackLeft4 = getCharacterImage(imagePath + "_left_attack_04.png");
			Animation attackingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, attackLeft1, attackLeft2,
					attackLeft3);
			animations.add(attackingLeftAnimation);

			TextureRegion attackUp1 = getImage(imagePath + "_back_attack_01.png");
			TextureRegion attackUp2 = getImage(imagePath + "_back_attack_02.png");
			TextureRegion attackUp3 = getImage(imagePath + "_back_attack_03.png");
			Animation attackingUpAnimation = new Animation(ANIMATION_FRAME_TIME, attackUp1, attackUp2,
					attackUp3);
			animations.add(attackingUpAnimation);

			TextureRegion attackDown1 = getImage(imagePath + "_front_attack_01.png");
			TextureRegion attackDown2 = getImage(imagePath + "_front_attack_02.png");
			TextureRegion attackDown3 = getImage(imagePath + "_front_attack_03.png");
			Animation attackingDownAnimation = new Animation(ANIMATION_FRAME_TIME, attackDown1, attackDown2,
					attackDown3);
			animations.add(attackingDownAnimation);
		}

		// Defending Animations

		if(type == CharacterType.WARRIOR)
		{
			TextureRegion defendRight1 = getImage("assets/data/warrior_images/warrior_right_block_01.png");
			TextureRegion defendRight2 = getImage("assets/data/warrior_images/warrior_right_block_02.png");
			//TextureRegion defendRight3 = getCharacterImage("assets/data/warrior_images/warrior_right_block_03.png");
			Animation defendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendRight1, defendRight2);
			animations.add(defendingRightAnimation);

			TextureRegion defendLeft1 = getImage("assets/data/warrior_images/warrior_left_block_01.png");
			TextureRegion defendLeft2 = getImage("assets/data/warrior_images/warrior_left_block_02.png");
			//TextureRegion defendLeft3 = getCharacterImage("assets/data/warrior_images/warrior_left_block_03.png");
			Animation defendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendLeft1, defendLeft2);
			animations.add(defendingLeftAnimation);

			TextureRegion defendUp1 = getImage("assets/data/warrior_images/warrior_back_block_01.png");
			TextureRegion defendUp2 = getImage("assets/data/warrior_images/warrior_back_block_02.png");
			//TextureRegion defendUp3 = getCharacterImage("assets/data/warrior_images/warrior_back_block_03.png");
			Animation defendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendUp1, defendUp2);
			animations.add(defendingUpAnimation);

			TextureRegion defendDown1 = getImage("assets/data/warrior_images/warrior_front_block_01.png");
			TextureRegion defendDown2 = getImage("assets/data/warrior_images/warrior_front_block_02.png");
			//TextureRegion defendDown3 = getCharacterImage("assets/data/warrior_images/warrior_front_block_03.png");
			Animation defendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendDown1, defendDown2);
			animations.add(defendingDownAnimation);
		}
		else if(type == CharacterType.MAGE)
		{
			TextureRegion defendRight1 = getImage("assets/data/mage_images/mage_right_shield_01.png");
			TextureRegion defendRight2 = getImage("assets/data/mage_images/mage_right_shield_02.png");
			TextureRegion defendRight3 = getImage("assets/data/mage_images/mage_right_shield_03.png");
			TextureRegion defendRight4 = getImage("assets/data/mage_images/mage_right_shield_04.png");
			Animation defendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendRight1, defendRight2,
					defendRight3, defendRight4);
			animations.add(defendingRightAnimation);

			TextureRegion defendLeft1 = getImage("assets/data/mage_images/mage_left_shield_01.png");
			TextureRegion defendLeft2 = getImage("assets/data/mage_images/mage_left_shield_02.png");
			TextureRegion defendLeft3 = getImage("assets/data/mage_images/mage_left_shield_03.png");
			TextureRegion defendLeft4 = getImage("assets/data/mage_images/mage_left_shield_04.png");
			Animation defendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendLeft1, defendLeft2,
					defendLeft3, defendLeft4);
			animations.add(defendingLeftAnimation);

			TextureRegion defendUp1 = getImage("assets/data/mage_images/mage_back_shield_01.png");
			TextureRegion defendUp2 = getImage("assets/data/mage_images/mage_back_shield_02.png");
			TextureRegion defendUp3 = getImage("assets/data/mage_images/mage_back_shield_03.png");
			TextureRegion defendUp4 = getImage("assets/data/mage_images/mage_back_shield_04.png");
			Animation defendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendUp1, defendUp2,
					defendUp3, defendUp4);
			animations.add(defendingUpAnimation);

			TextureRegion defendDown1 = getImage("assets/data/mage_images/mage_front_shield_01.png");
			TextureRegion defendDown2 = getImage("assets/data/mage_images/mage_front_shield_02.png");
			TextureRegion defendDown3 = getImage("assets/data/mage_images/mage_front_shield_03.png");
			TextureRegion defendDown4 = getImage("assets/data/mage_images/mage_front_shield_04.png");
			Animation defendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendDown1, defendDown2,
					defendDown3, defendDown4);
			animations.add(defendingDownAnimation);
		}
		else
		{
			TextureRegion defendRight1 = getImage("assets/data/rogue_images/rogue_right_cloaking_01.png");
			TextureRegion defendRight2 = getImage("assets/data/rogue_images/rogue_right_cloaking_02.png");
			TextureRegion defendRight3 = getImage("assets/data/rogue_images/rogue_right_cloaking_03.png");
			TextureRegion defendRight4 = getImage("assets/data/rogue_images/rogue_right_cloaking_04.png");
			TextureRegion defendRight5 = getImage("assets/data/rogue_images/rogue_right_cloaked.png");
			Animation defendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendRight1, defendRight2,
					defendRight3, defendRight4, defendRight5);
			animations.add(defendingRightAnimation);

			TextureRegion defendLeft1 = getImage("assets/data/rogue_images/rogue_left_cloaking_01.png");
			TextureRegion defendLeft2 = getImage("assets/data/rogue_images/rogue_left_cloaking_02.png");
			TextureRegion defendLeft3 = getImage("assets/data/rogue_images/rogue_left_cloaking_03.png");
			TextureRegion defendLeft4 = getImage("assets/data/rogue_images/rogue_left_cloaking_04.png");
			TextureRegion defendLeft5 = getImage("assets/data/rogue_images/rogue_left_cloaked.png");
			Animation defendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendLeft1, defendLeft2,
					defendLeft3, defendLeft4, defendLeft5);
			animations.add(defendingLeftAnimation);

			TextureRegion defendUp1 = getImage("assets/data/rogue_images/rogue_back_cloaking_01.png");
			TextureRegion defendUp2 = getImage("assets/data/rogue_images/rogue_back_cloaking_02.png");
			TextureRegion defendUp3 = getImage("assets/data/rogue_images/rogue_back_cloaking_03.png");
			TextureRegion defendUp4 = getImage("assets/data/rogue_images/rogue_back_cloaking_04.png");
			TextureRegion defendUp5 = getImage("assets/data/rogue_images/rogue_back_cloaked.png");
			Animation defendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendUp1, defendUp2,
					defendUp3, defendUp4, defendUp5);
			animations.add(defendingUpAnimation);

			TextureRegion defendDown1 = getImage("assets/data/rogue_images/rogue_front_cloaking_01.png");
			TextureRegion defendDown2 = getImage("assets/data/rogue_images/rogue_front_cloaking_02.png");
			TextureRegion defendDown3 = getImage("assets/data/rogue_images/rogue_front_cloaking_03.png");
			TextureRegion defendDown4 = getImage("assets/data/rogue_images/rogue_front_cloaking_04.png");
			TextureRegion defendDown5 = getImage("assets/data/rogue_images/rogue_front_cloaked.png");
			Animation defendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendDown1, defendDown2,
					defendDown3, defendDown4, defendDown5);
			animations.add(defendingDownAnimation);
		}

		//dying animations

		if(type == CharacterType.WARRIOR)
		{
			TextureRegion deadRight1 = getImage("assets/data/warrior_images/warrior_right_dying_01.png");
			TextureRegion deadRight2 = getImage("assets/data/warrior_images/warrior_right_dying_02.png");
			TextureRegion deadRight3 = getImage("assets/data/warrior_images/warrior_right_dying_03.png");
			TextureRegion deadRight4 = getImage("assets/data/warrior_images/warrior_right_dead.png");
			Animation deadRightAnimation = new Animation(ANIMATION_FRAME_TIME, deadRight1, deadRight2, deadRight3, 
					deadRight4);
			animations.add(deadRightAnimation);

			TextureRegion deadLeft1 = getImage("assets/data/warrior_images/warrior_left_dying_01.png");
			TextureRegion deadLeft2 = getImage("assets/data/warrior_images/warrior_left_dying_02.png");
			TextureRegion deadLeft3 = getImage("assets/data/warrior_images/warrior_left_dying_03.png");
			TextureRegion deadLeft4 = getImage("assets/data/warrior_images/warrior_left_dead.png");
			Animation deadLeftAnimation = new Animation(ANIMATION_FRAME_TIME, deadLeft1, deadLeft2, 
					deadLeft3, deadLeft4);
			animations.add(deadLeftAnimation);

			TextureRegion deadUp1 = getImage("assets/data/warrior_images/warrior_back_dying_01.png");
			TextureRegion deadUp2 = getImage("assets/data/warrior_images/warrior_back_dying_02.png");
			TextureRegion deadUp3 = getImage("assets/data/warrior_images/warrior_back_dying_03.png");
			TextureRegion deadUp4 = getImage("assets/data/warrior_images/warrior_back_dead.png");
			Animation deadUpAnimation = new Animation(ANIMATION_FRAME_TIME, deadUp1, deadUp2, deadUp3, 
					deadUp4);
			animations.add(deadUpAnimation);

			TextureRegion deadDown1 = getImage("assets/data/warrior_images/warrior_front_dying_01.png");
			TextureRegion deadDown2 = getImage("assets/data/warrior_images/warrior_front_dying_02.png");
			TextureRegion deadDown3 = getImage("assets/data/warrior_images/warrior_front_dying_03.png");
			TextureRegion deadDown4 = getImage("assets/data/warrior_images/warrior_front_dead.png");
			Animation deadDownAnimation = new Animation(ANIMATION_FRAME_TIME, deadDown1, deadDown2, 
					deadDown3, deadDown4);
			animations.add(deadDownAnimation);
		}
		else if(type == CharacterType.MAGE)
		{
			TextureRegion deadRight1 = getImage("assets/data/mage_images/mage_right_dying_01.png");
			TextureRegion deadRight2 = getImage("assets/data/mage_images/mage_right_dying_02.png");
			TextureRegion deadRight3 = getImage("assets/data/mage_images/mage_right_dying_03.png");
			Animation deadRightAnimation = new Animation(ANIMATION_FRAME_TIME, deadRight1, deadRight2, 
					deadRight3);
			animations.add(deadRightAnimation);

			TextureRegion deadLeft1 = getImage("assets/data/mage_images/mage_left_dying_01.png");
			TextureRegion deadLeft2 = getImage("assets/data/mage_images/mage_left_dying_02.png");
			TextureRegion deadLeft3 = getImage("assets/data/mage_images/mage_left_dying_03.png");
			Animation deadLeftAnimation = new Animation(ANIMATION_FRAME_TIME, deadLeft1, deadLeft2, 
					deadLeft3);
			animations.add(deadLeftAnimation);

			TextureRegion deadUp2 = getImage("assets/data/mage_images/mage_back_dying_02.png");
			TextureRegion deadUp3 = getImage("assets/data/mage_images/mage_back_dying_03.png");
			TextureRegion deadUp4 = getImage("assets/data/mage_images/mage_back_dead.png");
			Animation deadUpAnimation = new Animation(ANIMATION_FRAME_TIME, deadUp2, deadUp3, 
					deadUp4);
			animations.add(deadUpAnimation);

			TextureRegion deadDown2 = getImage("assets/data/mage_images/mage_front_dying_02.png");
			TextureRegion deadDown3 = getImage("assets/data/mage_images/mage_front_dying_03.png");
			TextureRegion deadDown4 = getImage("assets/data/mage_images/mage_front_dead.png");
			Animation deadDownAnimation = new Animation(ANIMATION_FRAME_TIME, deadDown2, 
					deadDown3, deadDown4);
			animations.add(deadDownAnimation);
		}
		else
		{
			TextureRegion deadRight1 = getImage("assets/data/rogue_images/rogue_right_dying_01.png");
			TextureRegion deadRight2 = getImage("assets/data/rogue_images/rogue_right_dying_02.png");
			TextureRegion deadRight3 = getImage("assets/data/rogue_images/rogue_right_dying_03.png");
			TextureRegion deadRight4 = getImage("assets/data/rogue_images/rogue_right_dead.png");
			Animation deadRightAnimation = new Animation(ANIMATION_FRAME_TIME, deadRight1, deadRight2, deadRight3, 
					deadRight4);
			animations.add(deadRightAnimation);

			TextureRegion deadLeft1 = getImage("assets/data/rogue_images/rogue_left_dying_01.png");
			TextureRegion deadLeft2 = getImage("assets/data/rogue_images/rogue_left_dying_02.png");
			TextureRegion deadLeft3 = getImage("assets/data/rogue_images/rogue_left_dying_03.png");
			TextureRegion deadLeft4 = getImage("assets/data/rogue_images/rogue_left_dead.png");
			Animation deadLeftAnimation = new Animation(ANIMATION_FRAME_TIME, deadLeft1, deadLeft2, 
					deadLeft3, deadLeft4);
			animations.add(deadLeftAnimation);

			TextureRegion deadUp1 = getImage("assets/data/rogue_images/rogue_back_dying_01.png");
			TextureRegion deadUp2 = getImage("assets/data/rogue_images/rogue_back_dying_02.png");
			TextureRegion deadUp3 = getImage("assets/data/rogue_images/rogue_back_dying_03.png");
			TextureRegion deadUp4 = getImage("assets/data/rogue_images/rogue_back_dead.png");
			Animation deadUpAnimation = new Animation(ANIMATION_FRAME_TIME, deadUp1, deadUp2, deadUp3, 
					deadUp4);
			animations.add(deadUpAnimation);

			TextureRegion deadDown1 = getImage("assets/data/rogue_images/rogue_front_dying_01.png");
			TextureRegion deadDown2 = getImage("assets/data/rogue_images/rogue_front_dying_02.png");
			TextureRegion deadDown3 = getImage("assets/data/rogue_images/rogue_front_dying_03.png");
			TextureRegion deadDown4 = getImage("assets/data/rogue_images/rogue_front_dead.png");
			Animation deadDownAnimation = new Animation(ANIMATION_FRAME_TIME, deadDown1, deadDown2, 
					deadDown3, deadDown4);
			animations.add(deadDownAnimation);
		}

		//special animations

		if(type == CharacterType.WARRIOR)
		{
			TextureRegion specialRight1 = getImage("assets/data/warrior_images/warrior_right_yell_01.png");
			TextureRegion specialRight2 = getImage("assets/data/warrior_images/warrior_right_yell_02.png");
			Animation specialRightAnimation = new Animation(ANIMATION_FRAME_TIME, specialRight1, specialRight2,
					specialRight1);
			animations.add(specialRightAnimation);

			TextureRegion specialLeft1 = getImage("assets/data/warrior_images/warrior_left_yell_01.png");
			TextureRegion specialLeft2 = getImage("assets/data/warrior_images/warrior_left_yell_02.png");
			Animation specialLeftAnimation = new Animation(ANIMATION_FRAME_TIME, specialLeft1, specialLeft2,
					specialLeft1);
			animations.add(specialLeftAnimation);

			TextureRegion specialUp1 = getImage("assets/data/warrior_images/warrior_back_yell_01.png");
			TextureRegion specialUp2 = getImage("assets/data/warrior_images/warrior_back_yell_02.png");
			Animation specialUpAnimation = new Animation(ANIMATION_FRAME_TIME, specialUp1, specialUp2,
					specialUp1);
			animations.add(specialUpAnimation);

			TextureRegion specialDown1 = getImage("assets/data/warrior_images/warrior_front_yell_01.png");
			TextureRegion specialDown2 = getImage("assets/data/warrior_images/warrior_front_yell_02.png");
			Animation specialDownAnimation = new Animation(ANIMATION_FRAME_TIME, specialDown1, specialDown2,
					specialDown1);
			animations.add(specialDownAnimation);
		}
		else if(type == CharacterType.MAGE)
		{
			TextureRegion specialRight1 = getImage("assets/data/mage_images/mage_right_heal_01.png");
			TextureRegion specialRight2 = getImage("assets/data/mage_images/mage_right_heal_02.png");
			TextureRegion specialRight3 = getImage("assets/data/mage_images/mage_right_heal_03.png");
			TextureRegion specialRight4 = getImage("assets/data/mage_images/mage_right_heal_04.png");
			TextureRegion specialRight5 = getImage("assets/data/mage_images/mage_right_heal_05.png");
			Animation specialRightAnimation = new Animation(ANIMATION_FRAME_TIME, specialRight1, specialRight2,
					specialRight3, specialRight4, specialRight5);
			animations.add(specialRightAnimation);

			TextureRegion specialLeft1 = getImage("assets/data/mage_images/mage_left_heal_01.png");
			TextureRegion specialLeft2 = getImage("assets/data/mage_images/mage_left_heal_02.png");
			TextureRegion specialLeft3 = getImage("assets/data/mage_images/mage_left_heal_03.png");
			TextureRegion specialLeft4 = getImage("assets/data/mage_images/mage_left_heal_04.png");
			TextureRegion specialLeft5 = getImage("assets/data/mage_images/mage_left_heal_05.png");
			Animation specialLeftAnimation = new Animation(ANIMATION_FRAME_TIME, specialLeft1, specialLeft2,
					specialLeft3, specialLeft4, specialLeft5);
			animations.add(specialLeftAnimation);

			TextureRegion specialUp1 = getImage("assets/data/mage_images/mage_back_heal_01.png");
			TextureRegion specialUp2 = getImage("assets/data/mage_images/mage_back_heal_02.png");
			TextureRegion specialUp3 = getImage("assets/data/mage_images/mage_back_heal_03.png");
			TextureRegion specialUp4 = getImage("assets/data/mage_images/mage_back_heal_04.png");
			TextureRegion specialUp5 = getImage("assets/data/mage_images/mage_back_heal_05.png");
			Animation specialUpAnimation = new Animation(ANIMATION_FRAME_TIME, specialUp1, specialUp2,
					specialUp3, specialUp4, specialUp5);
			animations.add(specialUpAnimation);

			TextureRegion specialDown1 = getImage("assets/data/mage_images/mage_front_heal_01.png");
			TextureRegion specialDown2 = getImage("assets/data/mage_images/mage_front_heal_02.png");
			TextureRegion specialDown3 = getImage("assets/data/mage_images/mage_front_heal_03.png");
			TextureRegion specialDown4 = getImage("assets/data/mage_images/mage_front_heal_04.png");
			TextureRegion specialDown5 = getImage("assets/data/mage_images/mage_front_heal_05.png");
			Animation specialDownAnimation = new Animation(ANIMATION_FRAME_TIME, specialDown1, specialDown2,
					specialDown3, specialDown4, specialDown5);
			animations.add(specialDownAnimation);
		}
		else
		{
			TextureRegion specialRight1 = getImage("assets/data/rogue_images/rogue_right_special_01.png");
			TextureRegion specialRight2 = getImage("assets/data/rogue_images/rogue_right_special_02.png");
			Animation specialRightAnimation = new Animation(ANIMATION_FRAME_TIME, specialRight1, specialRight2);
			animations.add(specialRightAnimation);

			TextureRegion specialLeft1 = getImage("assets/data/rogue_images/rogue_left_special_01.png");
			TextureRegion specialLeft2 = getImage("assets/data/rogue_images/rogue_left_special_02.png");
			Animation specialLeftAnimation = new Animation(ANIMATION_FRAME_TIME, specialLeft1, specialLeft2);
			animations.add(specialLeftAnimation);

			TextureRegion specialUp1 = getImage("assets/data/rogue_images/rogue_back_special_01.png");
			TextureRegion specialUp2 = getImage("assets/data/rogue_images/rogue_back_special_02.png");
			Animation specialUpAnimation = new Animation(ANIMATION_FRAME_TIME, specialUp1, specialUp2);
			animations.add(specialUpAnimation);

			TextureRegion specialDown1 = getImage("assets/data/rogue_images/rogue_front_special_01.png");
			TextureRegion specialDown2 = getImage("assets/data/rogue_images/rogue_front_special_02.png");
			Animation specialDownAnimation = new Animation(ANIMATION_FRAME_TIME, specialDown1, specialDown2);
			animations.add(specialDownAnimation);
		}

		// potion animations
		if(type == CharacterType.WARRIOR)
		{
			TextureRegion potionRight1 = getImage("assets/data/warrior_images/warrior_right_potion_01.png");
			TextureRegion potionRight2 = getImage("assets/data/warrior_images/warrior_right_potion_02.png");
			TextureRegion potionRight3 = getImage("assets/data/warrior_images/warrior_right_potion_03.png");
			Animation potionRightAnimation = new Animation(ANIMATION_FRAME_TIME, potionRight1, potionRight2,
					potionRight3);
			animations.add(potionRightAnimation);

			TextureRegion potionLeft1 = getImage("assets/data/warrior_images/warrior_left_potion_01.png");
			TextureRegion potionLeft2 = getImage("assets/data/warrior_images/warrior_left_potion_02.png");
			TextureRegion potionLeft3 = getImage("assets/data/warrior_images/warrior_left_potion_03.png");
			Animation potionLeftAnimation = new Animation(ANIMATION_FRAME_TIME, potionLeft1, potionLeft2,
					potionLeft3);
			animations.add(potionLeftAnimation);

			TextureRegion potionUp1 = getImage("assets/data/warrior_images/warrior_back_potion_01.png");
			TextureRegion potionUp2 = getImage("assets/data/warrior_images/warrior_back_potion_02.png");
			TextureRegion potionUp3 = getImage("assets/data/warrior_images/warrior_back_potion_03.png");
			Animation potionUpAnimation = new Animation(ANIMATION_FRAME_TIME, potionUp1, potionUp2,
					potionUp3);
			animations.add(potionUpAnimation);

			TextureRegion potionDown1 = getImage("assets/data/warrior_images/warrior_front_potion_01.png");
			TextureRegion potionDown2 = getImage("assets/data/warrior_images/warrior_front_potion_02.png");
			TextureRegion potionDown3 = getImage("assets/data/warrior_images/warrior_front_potion_03.png");
			Animation potionDownAnimation = new Animation(ANIMATION_FRAME_TIME, potionDown1, potionDown2,
					potionDown3);
			animations.add(potionDownAnimation);
		}
		else if(type == CharacterType.MAGE)
		{
			TextureRegion potionRight1 = getImage("assets/data/mage_images/mage_right_potion_01.png");
			TextureRegion potionRight2 = getImage("assets/data/mage_images/mage_right_potion_02.png");
			TextureRegion potionRight3 = getImage("assets/data/mage_images/mage_right_potion_03.png");
			Animation potionRightAnimation = new Animation(ANIMATION_FRAME_TIME, potionRight1, potionRight2,
					potionRight3);
			animations.add(potionRightAnimation);

			TextureRegion potionLeft1 = getImage("assets/data/mage_images/mage_left_potion_01.png");
			TextureRegion potionLeft2 = getImage("assets/data/mage_images/mage_left_potion_02.png");
			TextureRegion potionLeft3 = getImage("assets/data/mage_images/mage_left_potion_03.png");
			Animation potionLeftAnimation = new Animation(ANIMATION_FRAME_TIME, potionLeft1, potionLeft2,
					potionLeft3);
			animations.add(potionLeftAnimation);

			TextureRegion potionUp1 = getImage("assets/data/mage_images/mage_back_potion_01.png");
			TextureRegion potionUp2 = getImage("assets/data/mage_images/mage_back_potion_02.png");
			TextureRegion potionUp3 = getImage("assets/data/mage_images/mage_back_potion_03.png");
			Animation potionUpAnimation = new Animation(ANIMATION_FRAME_TIME, potionUp1, potionUp2,
					potionUp3);
			animations.add(potionUpAnimation);

			TextureRegion potionDown1 = getImage("assets/data/mage_images/mage_front_potion_01.png");
			TextureRegion potionDown2 = getImage("assets/data/mage_images/mage_front_potion_02.png");
			TextureRegion potionDown3 = getImage("assets/data/mage_images/mage_front_potion_03.png");
			Animation potionDownAnimation = new Animation(ANIMATION_FRAME_TIME, potionDown1, potionDown2,
					potionDown3);
			animations.add(potionDownAnimation);
		}
		else
		{
			TextureRegion potionRight1 = getImage("assets/data/rogue_images/rogue_right_potion_01.png");
			TextureRegion potionRight2 = getImage("assets/data/rogue_images/rogue_right_potion_02.png");
			TextureRegion potionRight3 = getImage("assets/data/rogue_images/rogue_right_potion_01.png");
			Animation potionRightAnimation = new Animation(ANIMATION_FRAME_TIME, potionRight1, potionRight2, 
					potionRight3);
			animations.add(potionRightAnimation);

			TextureRegion potionLeft1 = getImage("assets/data/rogue_images/rogue_left_potion_01.png");
			TextureRegion potionLeft2 = getImage("assets/data/rogue_images/rogue_left_potion_02.png");
			TextureRegion potionLeft3 = getImage("assets/data/rogue_images/rogue_left_potion_01.png");
			Animation potionLeftAnimation = new Animation(ANIMATION_FRAME_TIME, potionLeft1, potionLeft2, 
					potionLeft3);
			animations.add(potionLeftAnimation);

			TextureRegion potionUp1 = getImage("assets/data/rogue_images/rogue_back_potion_01.png");
			TextureRegion potionUp2 = getImage("assets/data/rogue_images/rogue_back_potion_02.png");
			TextureRegion potionUp3 = getImage("assets/data/rogue_images/rogue_back_potion_01.png");
			Animation potionUpAnimation = new Animation(ANIMATION_FRAME_TIME, potionUp1, potionUp2, 
					potionUp3);
			animations.add(potionUpAnimation);

			TextureRegion potionDown1 = getImage("assets/data/rogue_images/rogue_front_potion_01.png");
			TextureRegion potionDown2 = getImage("assets/data/rogue_images/rogue_front_potion_02.png");
			TextureRegion potionDown3 = getImage("assets/data/rogue_images/rogue_front_potion_01.png");
			Animation potionDownAnimation = new Animation(ANIMATION_FRAME_TIME, potionDown1, potionDown2, 
					potionDown3);
			animations.add(potionDownAnimation);
		}

		//revive animations

		if(type == CharacterType.WARRIOR)
		{
			TextureRegion reviveRight1 = getImage("assets/data/warrior_images/warrior_right_revive_01.png");
			TextureRegion reviveRight2 = getImage("assets/data/warrior_images/warrior_right_revive_02.png");
			TextureRegion reviveRight3 = getImage("assets/data/warrior_images/warrior_right_revive_03.png");
			Animation reviveRightAnimation = new Animation(ANIMATION_FRAME_TIME, reviveRight1, reviveRight2, 
					reviveRight3);
			animations.add(reviveRightAnimation);

			TextureRegion reviveLeft1 = getImage("assets/data/warrior_images/warrior_left_revive_01.png");
			TextureRegion reviveLeft2 = getImage("assets/data/warrior_images/warrior_left_revive_02.png");
			TextureRegion reviveLeft3 = getImage("assets/data/warrior_images/warrior_left_revive_03.png");
			Animation reviveLeftAnimation = new Animation(ANIMATION_FRAME_TIME, reviveLeft1, reviveLeft2, 
					reviveLeft3);
			animations.add(reviveLeftAnimation);

			TextureRegion reviveUp1 = getImage("assets/data/warrior_images/warrior_back_revive_01.png");
			TextureRegion reviveUp2 = getImage("assets/data/warrior_images/warrior_back_revive_02.png");
			TextureRegion reviveUp3 = getImage("assets/data/warrior_images/warrior_back_revive_03.png");
			Animation reviveUpAnimation = new Animation(ANIMATION_FRAME_TIME, reviveUp1, reviveUp2, reviveUp3);
			animations.add(reviveUpAnimation);

			TextureRegion reviveDown1 = getImage("assets/data/warrior_images/warrior_front_revive_01.png");
			TextureRegion reviveDown2 = getImage("assets/data/warrior_images/warrior_front_revive_02.png");
			TextureRegion reviveDown3 = getImage("assets/data/warrior_images/warrior_front_revive_03.png");
			Animation reviveDownAnimation = new Animation(ANIMATION_FRAME_TIME, reviveDown1, reviveDown2, 
					reviveDown3);
			animations.add(reviveDownAnimation);
		}
		else if(type == CharacterType.MAGE)
		{
			TextureRegion reviveRight1 = getImage("assets/data/mage_images/mage_right_revive_01.png");
			TextureRegion reviveRight2 = getImage("assets/data/mage_images/mage_right_revive_02.png");
			TextureRegion reviveRight3 = getImage("assets/data/mage_images/mage_right_revive_03.png");
			Animation reviveRightAnimation = new Animation(ANIMATION_FRAME_TIME, reviveRight1, reviveRight2, 
					reviveRight3);
			animations.add(reviveRightAnimation);

			TextureRegion reviveLeft1 = getImage("assets/data/mage_images/mage_left_revive_01.png");
			TextureRegion reviveLeft2 = getImage("assets/data/mage_images/mage_left_revive_02.png");
			TextureRegion reviveLeft3 = getImage("assets/data/mage_images/mage_left_revive_03.png");
			Animation reviveLeftAnimation = new Animation(ANIMATION_FRAME_TIME, reviveLeft1, reviveLeft2, 
					reviveLeft3);
			animations.add(reviveLeftAnimation);

			TextureRegion reviveUp1 = getImage("assets/data/mage_images/mage_back_revive_01.png");
			TextureRegion reviveUp2 = getImage("assets/data/mage_images/mage_back_revive_02.png");
			Animation reviveUpAnimation = new Animation(ANIMATION_FRAME_TIME, reviveUp1, reviveUp2);
			animations.add(reviveUpAnimation);

			TextureRegion reviveDown1 = getImage("assets/data/mage_images/mage_front_revive_01.png");
			TextureRegion reviveDown2 = getImage("assets/data/mage_images/mage_front_revive_02.png");
			Animation reviveDownAnimation = new Animation(ANIMATION_FRAME_TIME, reviveDown1, reviveDown2);
			animations.add(reviveDownAnimation);
		}
		else
		{
			TextureRegion reviveRight1 = getImage("assets/data/rogue_images/rogue_right_revive_01.png");
			TextureRegion reviveRight2 = getImage("assets/data/rogue_images/rogue_right_revive_02.png");
			TextureRegion reviveRight3 = getImage("assets/data/rogue_images/rogue_right_revive_03.png");
			Animation reviveRightAnimation = new Animation(ANIMATION_FRAME_TIME, reviveRight1, reviveRight2, 
					reviveRight3);
			animations.add(reviveRightAnimation);

			TextureRegion reviveLeft1 = getImage("assets/data/rogue_images/rogue_left_revive_01.png");
			TextureRegion reviveLeft2 = getImage("assets/data/rogue_images/rogue_left_revive_02.png");
			TextureRegion reviveLeft3 = getImage("assets/data/rogue_images/rogue_left_revive_03.png");
			Animation reviveLeftAnimation = new Animation(ANIMATION_FRAME_TIME, reviveLeft1, reviveLeft2, 
					reviveLeft3);
			animations.add(reviveLeftAnimation);

			TextureRegion reviveUp1 = getImage("assets/data/rogue_images/rogue_back_revive_01.png");
			TextureRegion reviveUp2 = getImage("assets/data/rogue_images/rogue_back_revive_02.png");
			TextureRegion reviveUp3 = getImage("assets/data/rogue_images/rogue_back_revive_03.png");
			Animation reviveUpAnimation = new Animation(ANIMATION_FRAME_TIME, reviveUp1, reviveUp2, 
					reviveUp3);
			animations.add(reviveUpAnimation);

			TextureRegion reviveDown1 = getImage("assets/data/rogue_images/rogue_front_revive_01.png");
			TextureRegion reviveDown2 = getImage("assets/data/rogue_images/rogue_front_revive_02.png");
			TextureRegion reviveDown3 = getImage("assets/data/rogue_images/rogue_front_revive_03.png");
			Animation reviveDownAnimation = new Animation(ANIMATION_FRAME_TIME, reviveDown1, reviveDown2, 
					reviveDown3);
			animations.add(reviveDownAnimation);
		}

		// end defending animations
		if(type == CharacterType.WARRIOR)
		{
			TextureRegion defendRight1 = getImage("assets/data/warrior_images/warrior_right_block_01.png");
			TextureRegion defendRight2 = getImage("assets/data/warrior_images/warrior_right_block_02.png");
			//TextureRegion defendRight3 = getCharacterImage("assets/data/warrior_images/warrior_right_block_03.png");
			Animation endDefendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendRight2,
					defendRight1);
			animations.add(endDefendingRightAnimation);

			TextureRegion defendLeft1 = getImage("assets/data/warrior_images/warrior_left_block_01.png");
			TextureRegion defendLeft2 = getImage("assets/data/warrior_images/warrior_left_block_02.png");
			//TextureRegion defendLeft3 = getCharacterImage("assets/data/warrior_images/warrior_left_block_03.png");
			Animation endDefendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendLeft2,
					defendLeft1);
			animations.add(endDefendingLeftAnimation);

			TextureRegion defendUp1 = getImage("assets/data/warrior_images/warrior_back_block_01.png");
			TextureRegion defendUp2 = getImage("assets/data/warrior_images/warrior_back_block_02.png");
			//TextureRegion defendUp3 = getCharacterImage("assets/data/warrior_images/warrior_back_block_03.png");
			Animation endDefendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendUp2,
					defendUp1);
			animations.add(endDefendingUpAnimation);

			TextureRegion defendDown1 = getImage("assets/data/warrior_images/warrior_front_block_01.png");
			TextureRegion defendDown2 = getImage("assets/data/warrior_images/warrior_front_block_02.png");
			//TextureRegion defendDown3 = getCharacterImage("assets/data/warrior_images/warrior_front_block_03.png");
			Animation endDefendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendDown2,
					defendDown1);
			animations.add(endDefendingDownAnimation);
		}
		else if(type == CharacterType.MAGE)
		{
			TextureRegion defendRight1 = getImage("assets/data/mage_images/mage_right_shield_01.png");
			TextureRegion defendRight2 = getImage("assets/data/mage_images/mage_right_shield_02.png");
			TextureRegion defendRight3 = getImage("assets/data/mage_images/mage_right_shield_03.png");
			TextureRegion defendRight4 = getImage("assets/data/mage_images/mage_right_shield_04.png");
			Animation endDefendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendRight4, defendRight3,
					defendRight2, defendRight1);
			animations.add(endDefendingRightAnimation);

			TextureRegion defendLeft1 = getImage("assets/data/mage_images/mage_left_shield_01.png");
			TextureRegion defendLeft2 = getImage("assets/data/mage_images/mage_left_shield_02.png");
			TextureRegion defendLeft3 = getImage("assets/data/mage_images/mage_left_shield_03.png");
			TextureRegion defendLeft4 = getImage("assets/data/mage_images/mage_left_shield_04.png");
			Animation endDefendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendLeft4, defendLeft3,
					defendLeft2, defendLeft1);
			animations.add(endDefendingLeftAnimation);

			TextureRegion defendUp1 = getImage("assets/data/mage_images/mage_back_shield_01.png");
			TextureRegion defendUp2 = getImage("assets/data/mage_images/mage_back_shield_02.png");
			TextureRegion defendUp3 = getImage("assets/data/mage_images/mage_back_shield_03.png");
			TextureRegion defendUp4 = getImage("assets/data/mage_images/mage_back_shield_04.png");
			Animation endDefendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendUp4, defendUp3,
					defendUp2, defendUp1);
			animations.add(endDefendingUpAnimation);

			TextureRegion defendDown1 = getImage("assets/data/mage_images/mage_front_shield_01.png");
			TextureRegion defendDown2 = getImage("assets/data/mage_images/mage_front_shield_02.png");
			TextureRegion defendDown3 = getImage("assets/data/mage_images/mage_front_shield_03.png");
			TextureRegion defendDown4 = getImage("assets/data/mage_images/mage_front_shield_04.png");
			Animation endDefendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendDown4, defendDown3,
					defendDown2, defendDown1);
			animations.add(endDefendingDownAnimation);
		}
		else
		{
			TextureRegion defendRight1 = getImage("assets/data/rogue_images/rogue_right_cloaking_01.png");
			TextureRegion defendRight2 = getImage("assets/data/rogue_images/rogue_right_cloaking_02.png");
			TextureRegion defendRight3 = getImage("assets/data/rogue_images/rogue_right_cloaking_03.png");
			TextureRegion defendRight4 = getImage("assets/data/rogue_images/rogue_right_cloaking_04.png");
			TextureRegion defendRight5 = getImage("assets/data/rogue_images/rogue_right_cloaked.png");
			Animation endDefendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendRight5, defendRight4,
					defendRight3, defendRight2, defendRight1);
			animations.add(endDefendingRightAnimation);

			TextureRegion defendLeft1 = getImage("assets/data/rogue_images/rogue_left_cloaking_01.png");
			TextureRegion defendLeft2 = getImage("assets/data/rogue_images/rogue_left_cloaking_02.png");
			TextureRegion defendLeft3 = getImage("assets/data/rogue_images/rogue_left_cloaking_03.png");
			TextureRegion defendLeft4 = getImage("assets/data/rogue_images/rogue_left_cloaking_04.png");
			TextureRegion defendLeft5 = getImage("assets/data/rogue_images/rogue_left_cloaked.png");
			Animation endDefendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendLeft5, defendLeft4,
					defendLeft3, defendLeft2, defendLeft1);
			animations.add(endDefendingLeftAnimation);

			TextureRegion defendUp1 = getImage("assets/data/rogue_images/rogue_back_cloaking_01.png");
			TextureRegion defendUp2 = getImage("assets/data/rogue_images/rogue_back_cloaking_02.png");
			TextureRegion defendUp3 = getImage("assets/data/rogue_images/rogue_back_cloaking_03.png");
			TextureRegion defendUp4 = getImage("assets/data/rogue_images/rogue_back_cloaking_04.png");
			TextureRegion defendUp5 = getImage("assets/data/rogue_images/rogue_back_cloaked.png");
			Animation endDefendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendUp5, defendUp4,
					defendUp3, defendUp2, defendUp1);
			animations.add(endDefendingUpAnimation);

			TextureRegion defendDown1 = getImage("assets/data/rogue_images/rogue_front_cloaking_01.png");
			TextureRegion defendDown2 = getImage("assets/data/rogue_images/rogue_front_cloaking_02.png");
			TextureRegion defendDown3 = getImage("assets/data/rogue_images/rogue_front_cloaking_03.png");
			TextureRegion defendDown4 = getImage("assets/data/rogue_images/rogue_front_cloaking_04.png");
			TextureRegion defendDown5 = getImage("assets/data/rogue_images/rogue_front_cloaked.png");
			Animation endDefendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendDown5, defendDown4,
					defendDown3, defendDown2, defendDown1);
			animations.add(endDefendingDownAnimation);
		}

		// pain animations
		if(type == CharacterType.WARRIOR)
		{
			TextureRegion rightPain = getImage("assets/data/warrior_images/warrior_right_pain.png");
			Animation rightPainAnimation = new Animation(ANIMATION_FRAME_TIME, rightPain);
			animations.add(rightPainAnimation);
			TextureRegion leftPain = getImage("assets/data/warrior_images/warrior_left_pain.png");
			Animation leftPainAnimation = new Animation(ANIMATION_FRAME_TIME, leftPain);
			animations.add(leftPainAnimation);
			TextureRegion backPain = getImage("assets/data/warrior_images/warrior_back_pain.png");
			Animation backPainAnimation = new Animation(ANIMATION_FRAME_TIME, backPain);
			animations.add(backPainAnimation);
			TextureRegion frontPain = getImage("assets/data/warrior_images/warrior_front_pain.png");
			Animation frontPainAnimation = new Animation(ANIMATION_FRAME_TIME, frontPain);
			animations.add(frontPainAnimation);
		}
		else if(type== CharacterType.MAGE)
		{
			TextureRegion rightPain = getImage("assets/data/mage_images/mage_right_dying_02.png");
			Animation rightPainAnimation = new Animation(ANIMATION_FRAME_TIME, rightPain);
			animations.add(rightPainAnimation);
			TextureRegion leftPain = getImage("assets/data/mage_images/mage_left_dying_02.png");
			Animation leftPainAnimation = new Animation(ANIMATION_FRAME_TIME, leftPain);
			animations.add(leftPainAnimation);
			TextureRegion backPain = getImage("assets/data/mage_images/mage_back_pain.png");
			Animation backPainAnimation = new Animation(ANIMATION_FRAME_TIME, backPain);
			animations.add(backPainAnimation);
			TextureRegion frontPain = getImage("assets/data/mage_images/mage_front_pain.png");
			Animation frontPainAnimation = new Animation(ANIMATION_FRAME_TIME, frontPain);
			animations.add(frontPainAnimation);
		}
		else
		{
			TextureRegion rightPain = getImage("assets/data/rogue_images/rogue_right_pain.png");
			Animation rightPainAnimation = new Animation(ANIMATION_FRAME_TIME, rightPain);
			animations.add(rightPainAnimation);
			TextureRegion leftPain = getImage("assets/data/rogue_images/rogue_left_pain.png");
			Animation leftPainAnimation = new Animation(ANIMATION_FRAME_TIME, leftPain);
			animations.add(leftPainAnimation);
			TextureRegion backPain = getImage("assets/data/rogue_images/rogue_back_pain.png");
			Animation backPainAnimation = new Animation(ANIMATION_FRAME_TIME, backPain);
			animations.add(backPainAnimation);
			TextureRegion frontPain = getImage("assets/data/rogue_images/rogue_front_pain.png");
			Animation frontPainAnimation = new Animation(ANIMATION_FRAME_TIME, frontPain);
			animations.add(frontPainAnimation);
		}

		if(type == CharacterType.WARRIOR)
		{
			warriorAnimations = animations;
		}
		else if(type == CharacterType.MAGE)
		{
			mageAnimations = animations;
		}
		else
		{
			rogueAnimations = animations;
		}
	}

	private void makeWolfAnimations()
	{
		float idleOffset = 4.0f;

		ArrayList<Animation> animations = new ArrayList<Animation> ();

		TextureRegion moveRight1 = getImage("assets/data/direwolf_images/dire_wolf_right_01.png");
		TextureRegion moveRight2 = getImage("assets/data/direwolf_images/dire_wolf_right_02.png");
		TextureRegion moveRight3 = getImage("assets/data/direwolf_images/dire_wolf_right_03.png");
		TextureRegion moveRight4 = getImage("assets/data/direwolf_images/dire_wolf_right_04.png");
		TextureRegion moveRight5 = getImage("assets/data/direwolf_images/dire_wolf_right_05.png");
		TextureRegion moveRight6 = getImage("assets/data/direwolf_images/dire_wolf_right_06.png");
		TextureRegion moveRight7 = getImage("assets/data/direwolf_images/dire_wolf_right_07.png");
		TextureRegion moveRight8 = getImage("assets/data/direwolf_images/dire_wolf_right_08.png");
		TextureRegion moveRight9 = getImage("assets/data/direwolf_images/dire_wolf_right_09.png");
		TextureRegion moveRight10 = getImage("assets/data/direwolf_images/dire_wolf_right_10.png");
		TextureRegion idleRight1 = getImage("assets/data/direwolf_images/dire_wolf_right_idle_01.png");
		TextureRegion idleRight2 = getImage("assets/data/direwolf_images/dire_wolf_right_idle_02.png");
		Animation movingRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveRight1, moveRight2, moveRight3,
				moveRight4, moveRight5, moveRight6, moveRight7, moveRight8, moveRight9, moveRight10);
		animations.add(movingRightAnimation);
		Animation facingRightAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleRight1, idleRight2);
		animations.add(facingRightAnimation);

		TextureRegion moveLeft1 = getImage("assets/data/direwolf_images/dire_wolf_left_01.png");
		TextureRegion moveLeft2 = getImage("assets/data/direwolf_images/dire_wolf_left_02.png");
		TextureRegion moveLeft3 = getImage("assets/data/direwolf_images/dire_wolf_left_03.png");
		TextureRegion moveLeft4 = getImage("assets/data/direwolf_images/dire_wolf_left_04.png");
		TextureRegion moveLeft5 = getImage("assets/data/direwolf_images/dire_wolf_left_05.png");
		TextureRegion moveLeft6 = getImage("assets/data/direwolf_images/dire_wolf_left_06.png");
		TextureRegion moveLeft7 = getImage("assets/data/direwolf_images/dire_wolf_left_07.png");
		TextureRegion moveLeft8 = getImage("assets/data/direwolf_images/dire_wolf_left_08.png");
		TextureRegion moveLeft9 = getImage("assets/data/direwolf_images/dire_wolf_left_09.png");
		TextureRegion moveLeft10 = getImage("assets/data/direwolf_images/dire_wolf_left_10.png");
		TextureRegion idleLeft1 = getImage("assets/data/direwolf_images/dire_wolf_left_idle_01.png");
		TextureRegion idleLeft2 = getImage("assets/data/direwolf_images/dire_wolf_left_idle_02.png");
		Animation movingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveLeft1, moveLeft2, moveLeft3,
				moveLeft4, moveLeft5, moveLeft6, moveLeft7, moveLeft8, moveLeft9, moveLeft10);
		animations.add(movingLeftAnimation);
		Animation facingLeftAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleLeft1, idleLeft2);
		animations.add(facingLeftAnimation);

		TextureRegion moveUp1 = getImage("assets/data/direwolf_images/dire_wolf_back_01.png");
		TextureRegion moveUp2 = getImage("assets/data/direwolf_images/dire_wolf_back_02.png");
		TextureRegion moveUp3 = getImage("assets/data/direwolf_images/dire_wolf_back_03.png");
		TextureRegion moveUp4 = getImage("assets/data/direwolf_images/dire_wolf_back_04.png");
		TextureRegion moveUp5 = getImage("assets/data/direwolf_images/dire_wolf_back_05.png");
		TextureRegion moveUp6 = getImage("assets/data/direwolf_images/dire_wolf_back_06.png");
		TextureRegion moveUp7 = getImage("assets/data/direwolf_images/dire_wolf_back_07.png");
		TextureRegion moveUp8 = getImage("assets/data/direwolf_images/dire_wolf_back_08.png");
		TextureRegion moveUp9 = getImage("assets/data/direwolf_images/dire_wolf_back_09.png");
		TextureRegion moveUp10 = getImage("assets/data/direwolf_images/dire_wolf_back_10.png");
		TextureRegion idleUp1 = getImage("assets/data/direwolf_images/dire_wolf_back_idle_01.png");
		TextureRegion idleUp2 = getImage("assets/data/direwolf_images/dire_wolf_back_idle_02.png");
		Animation movingUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveUp1, moveUp2, moveUp3,
				moveUp4, moveUp5, moveUp6, moveUp7, moveUp8, moveUp9, moveUp10);
		animations.add(movingUpAnimation);
		Animation facingUpAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleUp1, idleUp2);
		animations.add(facingUpAnimation);

		TextureRegion moveDown1 = getImage("assets/data/direwolf_images/dire_wolf_front_01.png");
		TextureRegion moveDown2 = getImage("assets/data/direwolf_images/dire_wolf_front_02.png");
		TextureRegion moveDown3 = getImage("assets/data/direwolf_images/dire_wolf_front_03.png");
		TextureRegion moveDown4 = getImage("assets/data/direwolf_images/dire_wolf_front_04.png");
		TextureRegion moveDown5 = getImage("assets/data/direwolf_images/dire_wolf_front_05.png");
		TextureRegion moveDown6 = getImage("assets/data/direwolf_images/dire_wolf_front_06.png");
		TextureRegion moveDown7 = getImage("assets/data/direwolf_images/dire_wolf_front_07.png");
		TextureRegion moveDown8 = getImage("assets/data/direwolf_images/dire_wolf_front_08.png");
		TextureRegion moveDown9 = getImage("assets/data/direwolf_images/dire_wolf_front_09.png");
		TextureRegion moveDown10 = getImage("assets/data/direwolf_images/dire_wolf_front_10.png");
		TextureRegion idleDown1 = getImage("assets/data/direwolf_images/dire_wolf_front_idle_01.png");
		TextureRegion idleDown2 = getImage("assets/data/direwolf_images/dire_wolf_front_idle_02.png");
		Animation movingDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1, moveDown2, moveDown3,
				moveDown4, moveDown5, moveDown6, moveDown7, moveDown8, moveDown9, moveDown10);
		animations.add(movingDownAnimation);
		Animation facingDownAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleDown1, idleDown2);
		animations.add(facingDownAnimation);

		TextureRegion attackingRight1 = getImage("assets/data/direwolf_images/dire_wolf_right_attack_01.png");
		TextureRegion attackingRight2 = getImage("assets/data/direwolf_images/dire_wolf_right_attack_02.png");
		Animation attackingRightAnimation = new Animation(ANIMATION_FRAME_TIME, attackingRight1, attackingRight2);
		animations.add(attackingRightAnimation);

		TextureRegion attackingLeft1 = getImage("assets/data/direwolf_images/dire_wolf_left_attack_01.png");
		TextureRegion attackingLeft2 = getImage("assets/data/direwolf_images/dire_wolf_left_attack_02.png");
		Animation attackingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, attackingLeft1, attackingLeft2);
		animations.add(attackingLeftAnimation);

		TextureRegion attackingBack1 = getImage("assets/data/direwolf_images/dire_wolf_back_attack_01.png");
		TextureRegion attackingBack2 = getImage("assets/data/direwolf_images/dire_wolf_back_attack_02.png");
		Animation attackingUpAnimation = new Animation(ANIMATION_FRAME_TIME, attackingBack1, attackingBack2);
		animations.add(attackingUpAnimation);

		TextureRegion attackingFront1 = getImage("assets/data/direwolf_images/dire_wolf_front_attack_01.png");
		TextureRegion attackingFront2 = getImage("assets/data/direwolf_images/dire_wolf_front_attack_02.png");
		Animation attackingDownAnimation = new Animation(ANIMATION_FRAME_TIME, attackingFront1, attackingFront2);
		animations.add(attackingDownAnimation);

		Animation defendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveRight1);
		animations.add(defendingRightAnimation);

		Animation defendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveLeft1);
		animations.add(defendingLeftAnimation);

		Animation defendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveUp1);
		animations.add(defendingUpAnimation);

		Animation defendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(defendingDownAnimation);

		TextureRegion deadRight1 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_01.png");
		TextureRegion deadRight2 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_02.png");
		TextureRegion deadRight3 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_03.png");
		TextureRegion deadRight4 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_04.png");
		TextureRegion deadRight5 = getImage("assets/data/direwolf_images/dire_wolf_right_dead.png");
		Animation deadRightAnimation = new Animation(ANIMATION_FRAME_TIME, deadRight1, deadRight2, 
				deadRight3, deadRight4, deadRight5);
		animations.add(deadRightAnimation);

		TextureRegion deadLeft1 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_01.png");
		TextureRegion deadLeft2 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_02.png");
		TextureRegion deadLeft3 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_03.png");
		TextureRegion deadLeft4 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_04.png");
		TextureRegion deadLeft5 = getImage("assets/data/direwolf_images/dire_wolf_left_dead.png");
		Animation deadLeftAnimation = new Animation(ANIMATION_FRAME_TIME, deadLeft1, deadLeft2, 
				deadLeft3, deadLeft4, deadLeft5);
		animations.add(deadLeftAnimation);

		TextureRegion deadUp1 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_01.png");
		TextureRegion deadUp2 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_02.png");
		TextureRegion deadUp3 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_03.png");
		TextureRegion deadUp4 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_04.png");
		TextureRegion deadUp5 = getImage("assets/data/direwolf_images/dire_wolf_back_dead.png");
		Animation deadUpAnimation = new Animation(ANIMATION_FRAME_TIME, deadUp1, deadUp2, deadUp3, 
				deadUp4, deadUp5);
		animations.add(deadUpAnimation);

		TextureRegion deadDown1 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_01.png");
		TextureRegion deadDown2 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_02.png");
		TextureRegion deadDown3 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_03.png");
		TextureRegion deadDown4 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_04.png");
		TextureRegion deadDown5 = getImage("assets/data/direwolf_images/dire_wolf_front_dead.png");
		Animation deadDownAnimation = new Animation(ANIMATION_FRAME_TIME, deadDown1, deadDown2, 
				deadDown3, deadDown4, deadDown5);
		animations.add(deadDownAnimation);

		Animation specialRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(specialRightAnimation);

		Animation specialLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(specialLeftAnimation);

		Animation specialUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(specialUpAnimation);

		Animation specialDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(specialDownAnimation);

		Animation potionRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(potionRightAnimation);

		Animation potionLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(potionLeftAnimation);

		Animation potionUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(potionUpAnimation);

		Animation potionDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(potionDownAnimation);

		TextureRegion reviveRight1 = getImage("assets/data/direwolf_images/dire_wolf_right_dead.png");
		TextureRegion reviveRight2 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_04.png");
		TextureRegion reviveRight3 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_03.png");
		TextureRegion reviveRight4 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_02.png");
		TextureRegion reviveRight5 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_01.png");
		Animation reviveRightAnimation = new Animation(ANIMATION_FRAME_TIME, reviveRight1, reviveRight2, 
				reviveRight3, reviveRight4, reviveRight5);
		animations.add(reviveRightAnimation);

		TextureRegion reviveLeft1 = getImage("assets/data/direwolf_images/dire_wolf_left_dead.png");
		TextureRegion reviveLeft2 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_04.png");
		TextureRegion reviveLeft3 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_03.png");
		TextureRegion reviveLeft4 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_02.png");
		TextureRegion reviveLeft5 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_01.png");
		Animation reviveLeftAnimation = new Animation(ANIMATION_FRAME_TIME, reviveLeft1, reviveLeft2, 
				reviveLeft3, reviveLeft4, reviveLeft5);
		animations.add(reviveLeftAnimation);

		TextureRegion reviveUp1 = getImage("assets/data/direwolf_images/dire_wolf_back_dead.png");
		TextureRegion reviveUp2 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_04.png");
		TextureRegion reviveUp3 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_03.png");
		TextureRegion reviveUp4 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_02.png");
		TextureRegion reviveUp5 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_01.png");
		Animation reviveUpAnimation = new Animation(ANIMATION_FRAME_TIME, reviveUp1, reviveUp2, reviveUp3, 
				reviveUp4, reviveUp5);
		animations.add(reviveUpAnimation);

		TextureRegion reviveDown1 = getImage("assets/data/direwolf_images/dire_wolf_front_dead.png");
		TextureRegion reviveDown2 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_04.png");
		TextureRegion reviveDown3 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_03.png");
		TextureRegion reviveDown4 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_02.png");
		TextureRegion reviveDown5 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_01.png");
		Animation reviveDownAnimation = new Animation(ANIMATION_FRAME_TIME, reviveDown1, reviveDown2, 
				reviveDown3, reviveDown4, reviveDown5);
		animations.add(reviveDownAnimation);

		Animation endDefendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveRight1);
		animations.add(endDefendingRightAnimation);

		Animation endDefendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveLeft1);
		animations.add(endDefendingLeftAnimation);

		Animation endDefendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveUp1);
		animations.add(endDefendingUpAnimation);

		Animation endDefendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(endDefendingDownAnimation);

		TextureRegion rightPain = getImage("assets/data/direwolf_images/dire_wolf_right_pain.png");
		Animation rightPainAnimation = new Animation(ANIMATION_FRAME_TIME, rightPain);
		animations.add(rightPainAnimation);

		TextureRegion leftPain = getImage("assets/data/direwolf_images/dire_wolf_left_pain.png");
		Animation leftPainAnimation = new Animation(ANIMATION_FRAME_TIME, leftPain);
		animations.add(leftPainAnimation);

		TextureRegion backPain = getImage("assets/data/direwolf_images/dire_wolf_back_pain.png");
		Animation backPainAnimation = new Animation(ANIMATION_FRAME_TIME, backPain);
		animations.add(backPainAnimation);

		TextureRegion frontPain = getImage("assets/data/direwolf_images/dire_wolf_front_pain.png");
		Animation frontPainAnimation = new Animation(ANIMATION_FRAME_TIME, frontPain);
		animations.add(frontPainAnimation);

		wolfAnimations = animations;
	}

	private void makeBossAnimations()
	{
		float idleOffset = 4.0f;

		ArrayList<Animation> animations = new ArrayList<Animation> ();

		TextureRegion moveRight1 = getImage("assets/data/big_boss_images/necro_right_01.png");
		TextureRegion moveRight2 = getImage("assets/data/big_boss_images/necro_right_02.png");
		TextureRegion moveRight3 = getImage("assets/data/big_boss_images/necro_right_03.png");
		TextureRegion moveRight4 = getImage("assets/data/big_boss_images/necro_right_04.png");
		TextureRegion moveRight5 = getImage("assets/data/big_boss_images/necro_right_05.png");
		TextureRegion moveRight6 = getImage("assets/data/big_boss_images/necro_right_06.png");
		TextureRegion moveRight7 = getImage("assets/data/big_boss_images/necro_right_07.png");
		TextureRegion moveRight8 = getImage("assets/data/big_boss_images/necro_right_08.png");
		TextureRegion idleRight1 = getImage("assets/data/big_boss_images/necro_right_idle_01.png");
		TextureRegion idleRight2 = getImage("assets/data/big_boss_images/necro_right_idle_02.png");
		Animation movingRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveRight1, moveRight2, moveRight3,
				moveRight4, moveRight5, moveRight6, moveRight7, moveRight8);
		animations.add(movingRightAnimation);
		Animation facingRightAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleRight1, idleRight2);
		animations.add(facingRightAnimation);

		TextureRegion moveLeft1 = getImage("assets/data/big_boss_images/necro_left_01.png");
		TextureRegion moveLeft2 = getImage("assets/data/big_boss_images/necro_left_02.png");
		TextureRegion moveLeft3 = getImage("assets/data/big_boss_images/necro_left_03.png");
		TextureRegion moveLeft4 = getImage("assets/data/big_boss_images/necro_left_04.png");
		TextureRegion moveLeft5 = getImage("assets/data/big_boss_images/necro_left_05.png");
		TextureRegion moveLeft6 = getImage("assets/data/big_boss_images/necro_left_06.png");
		TextureRegion moveLeft7 = getImage("assets/data/big_boss_images/necro_left_07.png");
		TextureRegion moveLeft8 = getImage("assets/data/big_boss_images/necro_left_08.png");
		TextureRegion idleLeft1 = getImage("assets/data/big_boss_images/necro_left_idle_01.png");
		TextureRegion idleLeft2 = getImage("assets/data/big_boss_images/necro_left_idle_01.png");
		Animation movingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveLeft1, moveLeft2, moveLeft3,
				moveLeft4, moveLeft5, moveLeft6, moveLeft7, moveLeft8);
		animations.add(movingLeftAnimation);
		Animation facingLeftAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleLeft1, idleLeft2);
		animations.add(facingLeftAnimation);

		TextureRegion moveUp1 = getImage("assets/data/big_boss_images/necro_back_01.png");
		TextureRegion moveUp2 = getImage("assets/data/big_boss_images/necro_back_02.png");
		TextureRegion moveUp3 = getImage("assets/data/big_boss_images/necro_back_03.png");
		TextureRegion moveUp4 = getImage("assets/data/big_boss_images/necro_back_04.png");
		TextureRegion moveUp5 = getImage("assets/data/big_boss_images/necro_back_05.png");
		TextureRegion moveUp6 = getImage("assets/data/big_boss_images/necro_back_06.png");
		TextureRegion moveUp7 = getImage("assets/data/big_boss_images/necro_back_07.png");
		TextureRegion moveUp8 = getImage("assets/data/big_boss_images/necro_back_08.png");
		TextureRegion idleUp1 = getImage("assets/data/big_boss_images/necro_back_idle_01.png");
		TextureRegion idleUp2 = getImage("assets/data/big_boss_images/necro_back_idle_02.png");
		Animation movingUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveUp1, moveUp2, moveUp3,
				moveUp4, moveUp5, moveUp6, moveUp7, moveUp8);
		animations.add(movingUpAnimation);
		Animation facingUpAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleUp1, idleUp2);
		animations.add(facingUpAnimation);

		TextureRegion moveDown1 = getImage("assets/data/big_boss_images/necro_front_01.png");
		TextureRegion moveDown2 = getImage("assets/data/big_boss_images/necro_front_02.png");
		TextureRegion moveDown3 = getImage("assets/data/big_boss_images/necro_front_03.png");
		TextureRegion moveDown4 = getImage("assets/data/big_boss_images/necro_front_04.png");
		TextureRegion moveDown5 = getImage("assets/data/big_boss_images/necro_front_05.png");
		TextureRegion moveDown6 = getImage("assets/data/big_boss_images/necro_front_06.png");
		TextureRegion moveDown7 = getImage("assets/data/big_boss_images/necro_front_07.png");
		TextureRegion moveDown8 = getImage("assets/data/big_boss_images/necro_front_08.png");
		TextureRegion idleDown1 = getImage("assets/data/big_boss_images/necro_front_idle_01.png");
		TextureRegion idleDown2 = getImage("assets/data/big_boss_images/necro_front_idle_02.png");
		Animation movingDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1, moveDown2, moveDown3,
				moveDown4, moveDown5, moveDown6, moveDown7, moveDown8);
		animations.add(movingDownAnimation);
		Animation facingDownAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleDown1, idleDown2);
		animations.add(facingDownAnimation);

		TextureRegion attackingRight1 = getImage("assets/data/big_boss_images/necro_right_attack_01.png");
		TextureRegion attackingRight2 = getImage("assets/data/big_boss_images/necro_right_attack_02.png");
		TextureRegion attackingRight3 = getImage("assets/data/big_boss_images/necro_right_attack_03.png");
		Animation attackingRightAnimation = new Animation(ANIMATION_FRAME_TIME, attackingRight1, attackingRight2,
				attackingRight3);
		animations.add(attackingRightAnimation);

		TextureRegion attackingLeft1 = getImage("assets/data/big_boss_images/necro_left_attack_01.png");
		TextureRegion attackingLeft2 = getImage("assets/data/big_boss_images/necro_left_attack_02.png");
		TextureRegion attackingLeft3 = getImage("assets/data/big_boss_images/necro_left_attack_03.png");
		Animation attackingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, attackingLeft1, attackingLeft2,
				attackingLeft3);
		animations.add(attackingLeftAnimation);

		TextureRegion attackingBack1 = getImage("assets/data/big_boss_images/necro_back_attack_01.png");
		TextureRegion attackingBack2 = getImage("assets/data/big_boss_images/necro_back_attack_02.png");
		TextureRegion attackingBack3 = getImage("assets/data/big_boss_images/necro_back_attack_03.png");
		Animation attackingUpAnimation = new Animation(ANIMATION_FRAME_TIME, attackingBack1, attackingBack2,
				attackingBack3);
		animations.add(attackingUpAnimation);

		TextureRegion attackingFront1 = getImage("assets/data/big_boss_images/necro_front_attack_01.png");
		TextureRegion attackingFront2 = getImage("assets/data/big_boss_images/necro_front_attack_02.png");
		TextureRegion attackingFront3 = getImage("assets/data/big_boss_images/necro_front_attack_03.png");
		Animation attackingDownAnimation = new Animation(ANIMATION_FRAME_TIME, attackingFront1, attackingFront2,
				attackingFront3);
		animations.add(attackingDownAnimation);

		TextureRegion defendingRight1 = getImage("assets/data/big_boss_images/necro_right_shield_01.png");
		TextureRegion defendingRight2 = getImage("assets/data/big_boss_images/necro_right_shield_02.png");
		TextureRegion defendingRight3 = getImage("assets/data/big_boss_images/necro_right_shield_03.png");
		Animation defendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendingRight1, defendingRight2,
				defendingRight3);
		animations.add(defendingRightAnimation);

		TextureRegion defendingLeft1 = getImage("assets/data/big_boss_images/necro_left_shield_01.png");
		TextureRegion defendingLeft2 = getImage("assets/data/big_boss_images/necro_left_shield_02.png");
		TextureRegion defendingLeft3 = getImage("assets/data/big_boss_images/necro_left_shield_03.png");
		Animation defendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendingLeft1, defendingLeft2,
				defendingLeft3);
		animations.add(defendingLeftAnimation);

		TextureRegion defendingUp1 = getImage("assets/data/big_boss_images/necro_back_shield_01.png");
		TextureRegion defendingUp2 = getImage("assets/data/big_boss_images/necro_back_shield_02.png");
		TextureRegion defendingUp3 = getImage("assets/data/big_boss_images/necro_right_shield_03.png");
		Animation defendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendingUp1, defendingUp2,
				defendingUp3);
		animations.add(defendingUpAnimation);

		TextureRegion defendingDown1 = getImage("assets/data/big_boss_images/necro_front_shield_01.png");
		TextureRegion defendingDown2 = getImage("assets/data/big_boss_images/necro_front_shield_02.png");
		TextureRegion defendingDown3 = getImage("assets/data/big_boss_images/necro_front_shield_03.png");
		Animation defendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendingDown1, defendingDown2,
				defendingDown3);
		animations.add(defendingDownAnimation);

		TextureRegion deadRight1 = getImage("assets/data/big_boss_images/necro_right_dying_01.png");
		TextureRegion deadRight2 = getImage("assets/data/big_boss_images/necro_right_dying_02.png");
		TextureRegion deadRight3 = getImage("assets/data/big_boss_images/necro_right_dying_03.png");
		TextureRegion deadRight4 = getImage("assets/data/big_boss_images/necro_right_dead.png");
		Animation deadRightAnimation = new Animation(ANIMATION_FRAME_TIME, deadRight1, deadRight2, 
				deadRight3, deadRight4);
		animations.add(deadRightAnimation);

		TextureRegion deadLeft1 = getImage("assets/data/big_boss_images/necro_left_dying_01.png");
		TextureRegion deadLeft2 = getImage("assets/data/big_boss_images/necro_left_dying_02.png");
		TextureRegion deadLeft3 = getImage("assets/data/big_boss_images/necro_left_dying_03.png");
		TextureRegion deadLeft4 = getImage("assets/data/big_boss_images/necro_left_dead.png");
		Animation deadLeftAnimation = new Animation(ANIMATION_FRAME_TIME, deadLeft1, deadLeft2, 
				deadLeft3, deadLeft4);
		animations.add(deadLeftAnimation);

		TextureRegion deadUp1 = getImage("assets/data/big_boss_images/necro_back_dying_01.png");
		TextureRegion deadUp2 = getImage("assets/data/big_boss_images/necro_back_dying_02.png");
		TextureRegion deadUp3 = getImage("assets/data/big_boss_images/necro_back_dying_03.png");
		TextureRegion deadUp4 = getImage("assets/data/big_boss_images/necro_back_dead.png");
		Animation deadUpAnimation = new Animation(ANIMATION_FRAME_TIME, deadUp1, deadUp2, 
				deadUp3, deadUp4);
		animations.add(deadUpAnimation);

		TextureRegion deadDown1 = getImage("assets/data/big_boss_images/necro_front_dying_01.png");
		TextureRegion deadDown2 = getImage("assets/data/big_boss_images/necro_front_dying_02.png");
		TextureRegion deadDown3 = getImage("assets/data/big_boss_images/necro_front_dying_03.png");
		TextureRegion deadDown4 = getImage("assets/data/big_boss_images/necro_front_dead.png");
		Animation deadDownAnimation = new Animation(ANIMATION_FRAME_TIME, deadDown1, deadDown2, 
				deadDown3, deadDown4);
		animations.add(deadDownAnimation);

		TextureRegion specialRight1 = getImage("assets/data/big_boss_images/necro_right_attack_01.png");
		TextureRegion specialRight2 = getImage("assets/data/big_boss_images/necro_right_attack_02.png");
		TextureRegion specialRight3 = getImage("assets/data/big_boss_images/necro_right_attack_03.png");
		Animation specialRightAnimation = new Animation(ANIMATION_FRAME_TIME, specialRight1, specialRight2, 
				specialRight3);
		animations.add(specialRightAnimation);

		TextureRegion specialLeft1 = getImage("assets/data/big_boss_images/necro_left_attack_01.png");
		TextureRegion specialLeft2 = getImage("assets/data/big_boss_images/necro_left_attack_02.png");
		TextureRegion specialLeft3 = getImage("assets/data/big_boss_images/necro_left_attack_03.png");
		Animation specialLeftAnimation = new Animation(ANIMATION_FRAME_TIME, specialLeft1, specialLeft2, 
				specialLeft3);
		animations.add(specialLeftAnimation);

		TextureRegion specialUp1 = getImage("assets/data/big_boss_images/necro_back_attack_01.png");
		TextureRegion specialUp2 = getImage("assets/data/big_boss_images/necro_back_attack_02.png");
		TextureRegion specialUp3 = getImage("assets/data/big_boss_images/necro_back_attack_03.png");
		Animation specialUpAnimation = new Animation(ANIMATION_FRAME_TIME, specialUp1, specialUp2, 
				specialUp3);
		animations.add(specialUpAnimation);

		TextureRegion specialDown1 = getImage("assets/data/big_boss_images/necro_front_attack_01.png");
		TextureRegion specialDown2 = getImage("assets/data/big_boss_images/necro_front_attack_02.png");
		TextureRegion specialDown3 = getImage("assets/data/big_boss_images/necro_front_attack_03.png");
		Animation specialDownAnimation = new Animation(ANIMATION_FRAME_TIME, specialDown1, specialDown2, 
				specialDown3);
		animations.add(specialDownAnimation);

		Animation potionRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(potionRightAnimation);

		Animation potionLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(potionLeftAnimation);

		Animation potionUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(potionUpAnimation);

		Animation potionDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(potionDownAnimation);

		TextureRegion reviveRight1 = getImage("assets/data/big_boss_images/necro_right_revive_01.png");
		TextureRegion reviveRight2 = getImage("assets/data/big_boss_images/necro_right_revive_02.png");
		TextureRegion reviveRight3 = getImage("assets/data/big_boss_images/necro_right_revive_03.png");
		Animation reviveRightAnimation = new Animation(ANIMATION_FRAME_TIME, reviveRight1, reviveRight2, 
				reviveRight3);
		animations.add(reviveRightAnimation);

		TextureRegion reviveLeft1 = getImage("assets/data/big_boss_images/necro_left_revive_01.png");
		TextureRegion reviveLeft2 = getImage("assets/data/big_boss_images/necro_left_revive_02.png");
		TextureRegion reviveLeft3 = getImage("assets/data/big_boss_images/necro_left_revive_03.png");
		Animation reviveLeftAnimation = new Animation(ANIMATION_FRAME_TIME, reviveLeft1, reviveLeft2, 
				reviveLeft3);
		animations.add(reviveLeftAnimation);

		TextureRegion reviveUp1 = getImage("assets/data/big_boss_images/necro_back_revive_01.png");
		TextureRegion reviveUp2 = getImage("assets/data/big_boss_images/necro_back_revive_02.png");
		TextureRegion reviveUp3 = getImage("assets/data/big_boss_images/necro_back_revive_03.png");
		Animation reviveUpAnimation = new Animation(ANIMATION_FRAME_TIME, reviveUp1, reviveUp2, reviveUp3);
		animations.add(reviveUpAnimation);

		TextureRegion reviveDown1 = getImage("assets/data/big_boss_images/necro_front_revive_01.png");
		TextureRegion reviveDown2 = getImage("assets/data/big_boss_images/necro_front_revive_02.png");
		TextureRegion reviveDown3 = getImage("assets/data/big_boss_images/necro_front_revive_03.png");
		Animation reviveDownAnimation = new Animation(ANIMATION_FRAME_TIME, reviveDown1, reviveDown2, 
				reviveDown3);
		animations.add(reviveDownAnimation);

		Animation endDefendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, defendingRight3,
				defendingRight2, defendingRight1);
		animations.add(endDefendingRightAnimation);

		Animation endDefendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, defendingLeft3,
				defendingLeft2, defendingLeft1);
		animations.add(endDefendingLeftAnimation);

		Animation endDefendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, defendingUp3,
				defendingUp2, defendingUp1);
		animations.add(endDefendingUpAnimation);

		Animation endDefendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, defendingDown3,
				defendingDown2, defendingDown1);
		animations.add(endDefendingDownAnimation);

		TextureRegion rightPain = getImage("assets/data/big_boss_images/necro_right_pain.png");
		Animation rightPainAnimation = new Animation(ANIMATION_FRAME_TIME, rightPain);
		animations.add(rightPainAnimation);

		TextureRegion leftPain = getImage("assets/data/big_boss_images/necro_left_pain.png");
		Animation leftPainAnimation = new Animation(ANIMATION_FRAME_TIME, leftPain);
		animations.add(leftPainAnimation);

		TextureRegion backPain = getImage("assets/data/big_boss_images/necro_back_pain.png");
		Animation backPainAnimation = new Animation(ANIMATION_FRAME_TIME, backPain);
		animations.add(backPainAnimation);

		TextureRegion frontPain = getImage("assets/data/big_boss_images/necro_front_pain.png");
		Animation frontPainAnimation = new Animation(ANIMATION_FRAME_TIME, frontPain);
		animations.add(frontPainAnimation);

		bossAnimations = animations;
	}

	private void makeNPCAnimations(String type)
	{
		float idleOffset = 4.0f;
		String imagePath;
		boolean adult = false;

		ArrayList<Animation> animations = new ArrayList<Animation> ();

		System.out.println(type);
		if(type.equals("male"))
		{
			imagePath = "assets/data/npc_male_images/npc_male";
			adult = true;
		}
		else if(type.equals("female"))
		{
			imagePath = "assets/data/npc_female_images/npc_female";
			adult = true;
		}
		else
		{
			imagePath = "assets/data/npc_kid_male_images/npc_kid_male";
		}

		TextureRegion moveRight1 = getNPCImage(imagePath + "_right_01.png", adult);
		TextureRegion moveRight2 = getNPCImage(imagePath + "_right_02.png", adult);
		TextureRegion moveRight3 = getNPCImage(imagePath + "_right_03.png", adult);
		TextureRegion moveRight4 = getNPCImage(imagePath + "_right_04.png", adult);
		TextureRegion moveRight5 = getNPCImage(imagePath + "_right_05.png", adult);
		TextureRegion moveRight6 = getNPCImage(imagePath + "_right_06.png", adult);
		TextureRegion moveRight7 = getNPCImage(imagePath + "_right_07.png", adult);
		TextureRegion moveRight8 = getNPCImage(imagePath + "_right_08.png", adult);
		TextureRegion idleRight1 = getNPCImage(imagePath + "_right_idle_01.png", adult);
		TextureRegion idleRight2 = getNPCImage(imagePath + "_right_idle_02.png", adult);
		Animation movingRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveRight1, moveRight2, moveRight3,
				moveRight4, moveRight5, moveRight6, moveRight7, moveRight8);
		animations.add(movingRightAnimation);
		Animation facingRightAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleRight1, idleRight2);
		animations.add(facingRightAnimation);

		TextureRegion moveLeft1 = getNPCImage(imagePath + "_left_01.png", adult);
		TextureRegion moveLeft2 = getNPCImage(imagePath + "_left_02.png", adult);
		TextureRegion moveLeft3 = getNPCImage(imagePath + "_left_03.png", adult);
		TextureRegion moveLeft4 = getNPCImage(imagePath + "_left_04.png", adult);
		TextureRegion moveLeft5 = getNPCImage(imagePath + "_left_05.png", adult);
		TextureRegion moveLeft6 = getNPCImage(imagePath + "_left_06.png", adult);
		TextureRegion moveLeft7 = getNPCImage(imagePath + "_left_07.png", adult);
		TextureRegion moveLeft8 = getNPCImage(imagePath + "_left_08.png", adult);
		TextureRegion idleLeft1 = getNPCImage(imagePath + "_left_idle_01.png", adult);
		TextureRegion idleLeft2 = getNPCImage(imagePath + "_left_idle_02.png", adult);
		Animation movingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveLeft1, moveLeft2, moveLeft3,
				moveLeft4, moveLeft5, moveLeft6, moveLeft7, moveLeft8);
		animations.add(movingLeftAnimation);
		Animation facingLeftAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleLeft1, idleLeft2);
		animations.add(facingLeftAnimation);

		TextureRegion moveUp1 = getNPCImage(imagePath + "_back_01.png", adult);
		TextureRegion moveUp2 = getNPCImage(imagePath + "_back_02.png", adult);
		TextureRegion moveUp3 = getNPCImage(imagePath + "_back_03.png", adult);
		TextureRegion moveUp4 = getNPCImage(imagePath + "_back_04.png", adult);
		TextureRegion moveUp5 = getNPCImage(imagePath + "_back_05.png", adult);
		TextureRegion moveUp6 = getNPCImage(imagePath + "_back_06.png", adult);
		TextureRegion moveUp7 = getNPCImage(imagePath + "_back_07.png", adult);
		TextureRegion moveUp8 = getNPCImage(imagePath + "_back_08.png", adult);
		TextureRegion idleUp1 = getNPCImage(imagePath + "_back_idle_01.png", adult);
		TextureRegion idleUp2 = getNPCImage(imagePath + "_back_idle_02.png", adult);
		Animation movingUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveUp1, moveUp2, moveUp3,
				moveUp4, moveUp5, moveUp6, moveUp7, moveUp8);
		animations.add(movingUpAnimation);
		Animation facingUpAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleUp1, idleUp2);
		animations.add(facingUpAnimation);

		TextureRegion moveDown1 = getNPCImage(imagePath + "_front_01.png", adult);
		TextureRegion moveDown2 = getNPCImage(imagePath + "_front_02.png", adult);
		TextureRegion moveDown3 = getNPCImage(imagePath + "_front_03.png", adult);
		TextureRegion moveDown4 = getNPCImage(imagePath + "_front_04.png", adult);
		TextureRegion moveDown5 = getNPCImage(imagePath + "_front_05.png", adult);
		TextureRegion moveDown6 = getNPCImage(imagePath + "_front_06.png", adult);
		TextureRegion moveDown7 = getNPCImage(imagePath + "_front_07.png", adult);
		TextureRegion moveDown8 = getNPCImage(imagePath + "_front_08.png", adult);
		TextureRegion idleDown1 = getNPCImage(imagePath + "_front_idle_01.png", adult);
		TextureRegion idleDown2 = getNPCImage(imagePath + "_front_idle_02.png", adult);
		Animation movingDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1, moveDown2, moveDown3,
				moveDown4, moveDown5, moveDown6, moveDown7, moveDown8);
		animations.add(movingDownAnimation);
		Animation facingDownAnimation = new Animation(ANIMATION_FRAME_TIME*idleOffset, idleDown1, idleDown2);
		animations.add(facingDownAnimation);

		TextureRegion attackingRight1 = getImage("assets/data/direwolf_images/dire_wolf_right_attack_01.png");
		TextureRegion attackingRight2 = getImage("assets/data/direwolf_images/dire_wolf_right_attack_02.png");
		Animation attackingRightAnimation = new Animation(ANIMATION_FRAME_TIME, attackingRight1, attackingRight2);
		animations.add(attackingRightAnimation);

		TextureRegion attackingLeft1 = getImage("assets/data/direwolf_images/dire_wolf_left_attack_01.png");
		TextureRegion attackingLeft2 = getImage("assets/data/direwolf_images/dire_wolf_left_attack_02.png");
		Animation attackingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, attackingLeft1, attackingLeft2);
		animations.add(attackingLeftAnimation);

		TextureRegion attackingBack1 = getImage("assets/data/direwolf_images/dire_wolf_back_attack_01.png");
		TextureRegion attackingBack2 = getImage("assets/data/direwolf_images/dire_wolf_back_attack_02.png");
		Animation attackingUpAnimation = new Animation(ANIMATION_FRAME_TIME, attackingBack1, attackingBack2);
		animations.add(attackingUpAnimation);

		TextureRegion attackingFront1 = getImage("assets/data/direwolf_images/dire_wolf_front_attack_01.png");
		TextureRegion attackingFront2 = getImage("assets/data/direwolf_images/dire_wolf_front_attack_02.png");
		Animation attackingDownAnimation = new Animation(ANIMATION_FRAME_TIME, attackingFront1, attackingFront2);
		animations.add(attackingDownAnimation);

		Animation defendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveRight1);
		animations.add(defendingRightAnimation);

		Animation defendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveLeft1);
		animations.add(defendingLeftAnimation);

		Animation defendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveUp1);
		animations.add(defendingUpAnimation);

		Animation defendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(defendingDownAnimation);

		TextureRegion deadRight1 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_01.png");
		TextureRegion deadRight2 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_02.png");
		TextureRegion deadRight3 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_03.png");
		TextureRegion deadRight4 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_04.png");
		TextureRegion deadRight5 = getImage("assets/data/direwolf_images/dire_wolf_right_dead.png");
		Animation deadRightAnimation = new Animation(ANIMATION_FRAME_TIME, deadRight1, deadRight2, 
				deadRight3, deadRight4, deadRight5);
		animations.add(deadRightAnimation);

		TextureRegion deadLeft1 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_01.png");
		TextureRegion deadLeft2 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_02.png");
		TextureRegion deadLeft3 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_03.png");
		TextureRegion deadLeft4 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_04.png");
		TextureRegion deadLeft5 = getImage("assets/data/direwolf_images/dire_wolf_left_dead.png");
		Animation deadLeftAnimation = new Animation(ANIMATION_FRAME_TIME, deadLeft1, deadLeft2, 
				deadLeft3, deadLeft4, deadLeft5);
		animations.add(deadLeftAnimation);

		TextureRegion deadUp1 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_01.png");
		TextureRegion deadUp2 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_02.png");
		TextureRegion deadUp3 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_03.png");
		TextureRegion deadUp4 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_04.png");
		TextureRegion deadUp5 = getImage("assets/data/direwolf_images/dire_wolf_back_dead.png");
		Animation deadUpAnimation = new Animation(ANIMATION_FRAME_TIME, deadUp1, deadUp2, deadUp3, 
				deadUp4, deadUp5);
		animations.add(deadUpAnimation);

		TextureRegion deadDown1 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_01.png");
		TextureRegion deadDown2 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_02.png");
		TextureRegion deadDown3 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_03.png");
		TextureRegion deadDown4 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_04.png");
		TextureRegion deadDown5 = getImage("assets/data/direwolf_images/dire_wolf_front_dead.png");
		Animation deadDownAnimation = new Animation(ANIMATION_FRAME_TIME, deadDown1, deadDown2, 
				deadDown3, deadDown4, deadDown5);
		animations.add(deadDownAnimation);

		Animation specialRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(specialRightAnimation);

		Animation specialLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(specialLeftAnimation);

		Animation specialUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(specialUpAnimation);

		Animation specialDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(specialDownAnimation);

		Animation potionRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(potionRightAnimation);

		Animation potionLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(potionLeftAnimation);

		Animation potionUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(potionUpAnimation);

		Animation potionDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(potionDownAnimation);

		TextureRegion reviveRight1 = getImage("assets/data/direwolf_images/dire_wolf_right_dead.png");
		TextureRegion reviveRight2 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_04.png");
		TextureRegion reviveRight3 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_03.png");
		TextureRegion reviveRight4 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_02.png");
		TextureRegion reviveRight5 = getImage("assets/data/direwolf_images/dire_wolf_right_dying_01.png");
		Animation reviveRightAnimation = new Animation(ANIMATION_FRAME_TIME, reviveRight1, reviveRight2, 
				reviveRight3, reviveRight4, reviveRight5);
		animations.add(reviveRightAnimation);

		TextureRegion reviveLeft1 = getImage("assets/data/direwolf_images/dire_wolf_left_dead.png");
		TextureRegion reviveLeft2 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_04.png");
		TextureRegion reviveLeft3 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_03.png");
		TextureRegion reviveLeft4 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_02.png");
		TextureRegion reviveLeft5 = getImage("assets/data/direwolf_images/dire_wolf_left_dying_01.png");
		Animation reviveLeftAnimation = new Animation(ANIMATION_FRAME_TIME, reviveLeft1, reviveLeft2, 
				reviveLeft3, reviveLeft4, reviveLeft5);
		animations.add(reviveLeftAnimation);

		TextureRegion reviveUp1 = getImage("assets/data/direwolf_images/dire_wolf_back_dead.png");
		TextureRegion reviveUp2 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_04.png");
		TextureRegion reviveUp3 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_03.png");
		TextureRegion reviveUp4 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_02.png");
		TextureRegion reviveUp5 = getImage("assets/data/direwolf_images/dire_wolf_back_dying_01.png");
		Animation reviveUpAnimation = new Animation(ANIMATION_FRAME_TIME, reviveUp1, reviveUp2, reviveUp3, 
				reviveUp4, reviveUp5);
		animations.add(reviveUpAnimation);

		TextureRegion reviveDown1 = getImage("assets/data/direwolf_images/dire_wolf_front_dead.png");
		TextureRegion reviveDown2 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_04.png");
		TextureRegion reviveDown3 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_03.png");
		TextureRegion reviveDown4 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_02.png");
		TextureRegion reviveDown5 = getImage("assets/data/direwolf_images/dire_wolf_front_dying_01.png");
		Animation reviveDownAnimation = new Animation(ANIMATION_FRAME_TIME, reviveDown1, reviveDown2, 
				reviveDown3, reviveDown4, reviveDown5);
		animations.add(reviveDownAnimation);

		Animation endDefendingRightAnimation = new Animation(ANIMATION_FRAME_TIME, moveRight1);
		animations.add(endDefendingRightAnimation);

		Animation endDefendingLeftAnimation = new Animation(ANIMATION_FRAME_TIME, moveLeft1);
		animations.add(endDefendingLeftAnimation);

		Animation endDefendingUpAnimation = new Animation(ANIMATION_FRAME_TIME, moveUp1);
		animations.add(endDefendingUpAnimation);

		Animation endDefendingDownAnimation = new Animation(ANIMATION_FRAME_TIME, moveDown1);
		animations.add(endDefendingDownAnimation);

		TextureRegion rightPain = getImage("assets/data/direwolf_images/dire_wolf_right_pain.png");
		Animation rightPainAnimation = new Animation(ANIMATION_FRAME_TIME, rightPain);
		animations.add(rightPainAnimation);

		TextureRegion leftPain = getImage("assets/data/direwolf_images/dire_wolf_left_pain.png");
		Animation leftPainAnimation = new Animation(ANIMATION_FRAME_TIME, leftPain);
		animations.add(leftPainAnimation);

		TextureRegion backPain = getImage("assets/data/direwolf_images/dire_wolf_back_pain.png");
		Animation backPainAnimation = new Animation(ANIMATION_FRAME_TIME, backPain);
		animations.add(backPainAnimation);

		TextureRegion frontPain = getImage("assets/data/direwolf_images/dire_wolf_front_pain.png");
		Animation frontPainAnimation = new Animation(ANIMATION_FRAME_TIME, frontPain);
		animations.add(frontPainAnimation);

		if(type.equals("male"))
		{
			npcMaleAnimations = animations;
		}
		else if(type.equals("female"))
		{
			npcFemaleAnimations = animations;
		}
		else
		{
			npcKidAnimations = animations;
		}
	}
	
	/**
	 * Make the animation
	 */
	private void makeExplosionAnimation(boolean boss)
	{
		String path;
		if(!boss)
		{
			path = "assets/data/mage_images/";
		}
		else
		{
			path = "assets/data/big_boss_images/necro_";
		}
		
		TextureRegion explosion1 = getLargeImage(path + "explosion_01.png");
		TextureRegion explosion2 = getLargeImage(path + "explosion_02.png");
		TextureRegion explosion3 = getLargeImage(path + "explosion_03.png");
		TextureRegion explosion4 = getLargeImage(path + "explosion_04.png");
		TextureRegion explosion5 = getLargeImage(path + "explosion_05.png");
		TextureRegion explosion6 = getLargeImage(path + "explosion_06.png");
		TextureRegion explosion7 = getLargeImage(path + "explosion_07.png");
		Animation explosionAnimation = new Animation(ANIMATION_FRAME_TIME, explosion1,
				explosion2, explosion3, explosion4, explosion5, explosion6, explosion7);
		
		if(boss)
		{
			bossExplosion = explosionAnimation;
		}
		else
		{
			mageExplosion = explosionAnimation;
		}
	}
	
	/**
	 * Make the animation
	 */
	private void makeSparkleAnimation(boolean boss)
	{
		String path;
		if(!boss)
		{
			path = "assets/data/mage_images/heal";
		}
		else
		{
			path = "assets/data/big_boss_images/necro_life_drain";
		}
		
		TextureRegion sparkles1 = getLargeImage(path + "_01.png");
		TextureRegion sparkles2 = getLargeImage(path + "_02.png");
		TextureRegion sparkles3 = getLargeImage(path + "_03.png");
		TextureRegion sparkles4 = getLargeImage(path + "_04.png");
		TextureRegion sparkles5 = getLargeImage(path + "_05.png");
		Animation sparklesAnimation = new Animation(ANIMATION_FRAME_TIME, sparkles1,
				sparkles2, sparkles3, sparkles4, sparkles5);
		
		if(boss)
		{
			bossSparkles = sparklesAnimation;
		}
		else
		{
			mageSparkles = sparklesAnimation;
		}
	}

	/**
	 * Make animation for the gate
	 */
	private void makeGateAnimation()
	{
		//Make the animation
		TextureRegion gateOpen1 = getLongImage("assets/data/terrain/gate_open_1.png");
		TextureRegion gateOpen2 = getLongImage("assets/data/terrain/gate_open_2.png");
		TextureRegion gateOpen3 = getLongImage("assets/data/terrain/gate_open_3.png");
		Animation openAnimation = new Animation(ANIMATION_FRAME_TIME, gateOpen1, gateOpen2, gateOpen3);
		
		gateAnimation = openAnimation;
	}
	
	/**
	 * Make the animation
	 */
	private void makePoisonCloudAnimation()
	{
		TextureRegion poison1 = getLargeImage("assets/data/rogue_images/poison_cloud_01.png");
		TextureRegion poison2 = getLargeImage("assets/data/rogue_images/poison_cloud_02.png");
		TextureRegion poison3 = getLargeImage("assets/data/rogue_images/poison_cloud_03.png");
		TextureRegion poison4 = getLargeImage("assets/data/rogue_images/poison_cloud_04.png");
		TextureRegion poison5 = getLargeImage("assets/data/rogue_images/poison_cloud_05.png");
		TextureRegion poison6 = getLargeImage("assets/data/rogue_images/poison_cloud_06.png");
		poisonCloudAnimation = new Animation(ANIMATION_FRAME_TIME, poison1,
				poison2, poison3, poison4, poison5, poison6);
	}
	
	/**
	 * Make arrow image
	 */
	private void makeArrowImage()
	{
		Texture texture = new Texture(Gdx.files.internal("assets/data/rogue_images/Arrow.png"));
		textures.add(texture);
		arrowImage = new TextureRegion(texture, 0, 0, 64, 64);
	}

	public static ArrayList<Animation> getWarriorAnimations() {
		return warriorAnimations;
	}

	public static ArrayList<Animation> getMageAnimations() {
		return mageAnimations;
	}

	public static ArrayList<Animation> getRogueAnimations() {
		return rogueAnimations;
	}

	public static ArrayList<Animation> getFighterAnimations() {
		return fighterAnimations;
	}

	public static ArrayList<Animation> getWizardAnimations() {
		return wizardAnimations;
	}

	public static ArrayList<Animation> getThiefAnimations() {
		return thiefAnimations;
	}

	public static ArrayList<Animation> getWolfAnimations() {
		return wolfAnimations;
	}

	public static ArrayList<Animation> getBossAnimations() {
		return bossAnimations;
	}

	public static ArrayList<Animation> getNpcMaleAnimations() {
		return npcMaleAnimations;
	}

	public static ArrayList<Animation> getNpcFemaleAnimations() {
		return npcFemaleAnimations;
	}

	public static ArrayList<Animation> getNpcKidAnimations() {
		return npcKidAnimations;
	}
	
	public static Animation getMageExplosion()
	{
		return mageExplosion;
	}
	
	public static Animation getBossExplosion()
	{
		return bossExplosion;
	}
	
	public static Animation getMageSparkles()
	{
		return mageSparkles;
	}
	
	public static Animation getBossSparkles()
	{
		return bossSparkles;
	}
	
	public static Animation getGateAnimation()
	{
		return gateAnimation;
	}
	
	public static Animation getPoisonCloudAnimation()
	{
		return poisonCloudAnimation;
	}
	
	public static TextureRegion getArrowImage()
	{
		return arrowImage;
	}
	
	public void dispose()
	{
		for(Texture texture : textures)
		{
			texture.dispose();
		}
	}

}
