/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package game;

import java.util.ArrayList;
import java.util.List;

import pathfinding.Graph;
import manager.AIManager;
import manager.CollisionManager;
import manager.LineOfSightManager;
import menu.ProgressJournal;
import gameobject.character.Enemy;
import gameobject.character.PartyMember;
import gameobject.GameObject;
import gameobject.GateDoor;
import gameobject.ItemDrop;
import gameobject.NPC;
import gameobject.ObjectType;
import gameobject.Projectile;
import gameobject.SolidObject;
import gameobject.TransitionObject;
import gameobject.TextBox;
import gameobject.Dialogue;
import gameobject.character.AnimationState;
import gameobject.character.BigBoss;
import gameobject.character.Character;
import gameobject.character.CharacterType;
import gameobject.character.DeadTree;
import gameobject.character.EnemyHealthBar;
import hud.DigitIcon;
import hud.HUD;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.Texture.TextureFilter;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Rectangle;

/**
 * Class representing the screen to be shown
 * @author jordan, Kevin, Eyleen
 * @version Oct 4 2013
 */
public class GameScreen implements Screen {

	private final QuestMain game;

	private final String levelPath;
	private final int level;
	private DigitIcon levelIconOne;
	private DigitIcon levelIconTen;

	// constants

	private final int IMAGE_WIDTH = 64;
	private final int IMAGE_HEIGHT = 64;

	private final int BACKGROUND_WIDTH = 1000;
	private final int BACKGROUND_HEIGHT = 1000;

	// 0.05 = 64
	private final float OBJECT_WIDTH = 0.05f;
	private final float OBJECT_HEIGHT = 0.05f;

	//0.1 = 128
	private final float OBJECT_WIDTH_LONG = 0.1f;
	private final float OBJECT_HEIGHT_TALL = 0.1f;

	private Rectangle screenDimensions;

	private BitmapFont font;

	// fields
	private OrthographicCamera camera;
	private SpriteBatch spriteBatch;
	private TextureRegion background;

	private List<GameObject> gameObjects;
	private List<Character> allies;
	private List<Character> enemies;
	private List<NPC> npcs;
	private List<DeadTree> deadtrees;
	private TextureRegion primaryHalo;
	private TextureRegion secondaryHalo;
	private TextureRegion tertiaryHalo;
	private TextureRegion damageHalo;
	private TextureRegion tauntedHalo;

	private List<GameObject> newObjects;
	private List<GameObject> deadObjects;
	private List<GameObject> softObjects;
	
	private boolean dialogueIsFinished;

	//private PartyMember player;
	private TransitionObject transitionObject;

	public NPC test = null;

	// managers
	private CollisionManager collisionManager;
	private LineOfSightManager lineOfSightManager;
	private AIManager aiManager;

	//HUD
	private HUD hud;
	private Player player;
	private Clock clock;
	private DialogueHandler dialogueHandler;

	private Boolean characterSwitch = false;

	private int numEnemies;
	private long time;

	public long delta;
	
	private Graph graph;
	
	private ArrayList<Texture> textures;

	/**
	 * Constructor
	 * @param game the QuestMain object representing the game
	 * @param levelpath the path to the text file representing the level
	 * representing the level 
	 */
	public GameScreen(QuestMain game, String levelpath, int level) {
		this.game = game;
		this.levelPath = levelpath;
		this.level = level;
		
		dialogueIsFinished = false;
	}

	/**
	 * Make other objects
	 * @param levelpath the path to the text file representing the level
	 * @throws Exception
	 */
	private void makeObjects(String levelpath) throws Exception {
		// grab textures

		// bush
		Texture bush = new Texture(Gdx.files.internal("assets/data/terrain/bush1.png"));
		textures.add(bush);
		Texture bush2 = new Texture(Gdx.files.internal("assets/data/terrain/bush2.png"));
		textures.add(bush2);

		// rock
		Texture rock = new Texture(Gdx.files.internal("assets/data/terrain/rock1.png"));
		textures.add(rock);
		Texture rock2 = new Texture(Gdx.files.internal("assets/data/terrain/rock2.png"));
		textures.add(rock2);
		Texture rockBig = new Texture(Gdx.files.internal("assets/data/terrain/rockBig.png"));
		textures.add(rockBig);

		// tree
		Texture tree = new Texture(Gdx.files.internal("assets/data/terrain/tree1.png"));
		textures.add(tree);
		Texture tree2 = new Texture(Gdx.files.internal("assets/data/terrain/tree2.png"));
		textures.add(tree2);
		Texture tree3 = new Texture(Gdx.files.internal("assets/data/terrain/tree3.png"));
		textures.add(tree3);
		Texture treeStump = new Texture(Gdx.files.internal("assets/data/terrain/tree_stump.png"));
		textures.add(treeStump);

		//pathway
		Texture straightPath = new Texture(Gdx.files.internal("assets/data/terrain/dirt_path_straight.png"));
		textures.add(straightPath);
		Texture straightPathH = new Texture(Gdx.files.internal("assets/data/terrain/dirt_path_straight_h.png"));
		textures.add(straightPathH);
		Texture tPath = new Texture(Gdx.files.internal("assets/data/terrain/dirt_path_T.png"));
		textures.add(tPath);
		Texture tPath2 = new Texture(Gdx.files.internal("assets/data/terrain/dirt_path_T2.png"));
		textures.add(tPath2);
		Texture tPath3 = new Texture(Gdx.files.internal("assets/data/terrain/dirt_path_T3.png"));
		textures.add(tPath3);
		Texture tPath4 = new Texture(Gdx.files.internal("assets/data/terrain/dirt_path_T4.png"));
		textures.add(tPath4);
		Texture turnPath = new Texture(Gdx.files.internal("assets/data/terrain/dirt_path_turn.png"));
		textures.add(turnPath);
		Texture turnPath2 = new Texture(Gdx.files.internal("assets/data/terrain/dirt_path_turn_2.png"));
		textures.add(turnPath2);
		Texture turnPath3 = new Texture(Gdx.files.internal("assets/data/terrain/dirt_path_turn_3.png"));
		textures.add(turnPath3);
		Texture turnPath4 = new Texture(Gdx.files.internal("assets/data/terrain/dirt_path_turn_4.png"));
		textures.add(turnPath4);
		Texture xPath = new Texture(Gdx.files.internal("assets/data/terrain/dirt_path_X.png"));
		textures.add(xPath);
		Texture empty = new Texture(Gdx.files.internal("assets/data/terrain/empty.png"));
		textures.add(empty);

		// NPC
		Texture npcTexture = new Texture(Gdx.files.internal("assets/data/warrior_images/warrior_front_02.png"));
		textures.add(npcTexture);

		// fence
		Texture fenceCorner = new Texture(Gdx.files.internal("assets/data/terrain/fence_corner.png"));
		textures.add(fenceCorner);
		Texture fenceCorner2 = new Texture(Gdx.files.internal("assets/data/terrain/fence_corner2.png"));
		textures.add(fenceCorner2);
		Texture fenceCorner3 = new Texture(Gdx.files.internal("assets/data/terrain/fence_corner3.png"));
		textures.add(fenceCorner3);
		Texture fenceCorner4 = new Texture(Gdx.files.internal("assets/data/terrain/fence_corner4.png"));
		textures.add(fenceCorner4);
		Texture fenceHorizontal = new Texture(Gdx.files.internal("assets/data/terrain/fence_horizontal.png"));
		textures.add(fenceHorizontal);
		Texture fenceVertical = new Texture(Gdx.files.internal("assets/data/terrain/fence_vertical.png"));
		textures.add(fenceVertical);

		// deco
		Texture flowerPatch = new Texture(Gdx.files.internal("assets/data/terrain/flower_patch.png"));
		textures.add(flowerPatch);
		Texture grassTufts = new Texture(Gdx.files.internal("assets/data/terrain/grass_tufts.png"));
		textures.add(grassTufts);
		Texture log = new Texture(Gdx.files.internal("assets/data/terrain/log.png"));
		textures.add(log);
		
		// gate
		Texture gateLength = new Texture(Gdx.files.internal("assets/data/terrain/iron_fence.png"));
		textures.add(gateLength);
		Texture gateDoor = new Texture(Gdx.files.internal("assets/data/terrain/iron_fence_door.png"));
		textures.add(gateDoor);
		Texture gateOpen1 = new Texture(Gdx.files.internal("assets/data/terrain/gate_open_1.png"));
		textures.add(gateOpen1);
		Texture gateOpen2 = new Texture(Gdx.files.internal("assets/data/terrain/gate_open_2.png"));
		textures.add(gateOpen2);
		Texture gateOpen3 = new Texture(Gdx.files.internal("assets/data/terrain/gate_open_3.png"));
		textures.add(gateOpen3);

		// get the level file
		FileHandle file = Gdx.files.internal(levelpath);
		String fileText = file.readString();
		System.out.println(fileText);
		String[] fileTokens = fileText.split(" |\n");
		int i = 0;


		// loop through the tokens
		while (!fileTokens[i].equals("EOF"))
		{
			// get the background
			if(fileTokens[i].equals("Background:"))
			{
				i++;
				String background = fileTokens[i];
				makeBackground(background);
			}

			// else there's an object
			else
			{
				String type = fileTokens[i];
				i++;
				float x = Float.parseFloat(fileTokens[i]);
				i++;
				float y = Float.parseFloat(fileTokens[i]);

				// object is a Bush
				if(type.equals("Bush:"))
				{
					makeClass1Object(bush, x, y);
				}
				// object is a Bush
				else if(type.equals("Bush2:"))
				{
					makeClass1Object(bush2, x, y);
				}
				// object is a rock
				else if(type.equals("Rock:"))
				{
					makeClass1Object(rock, x, y);
				}
				// object is a rock
				else if(type.equals("Rock2:"))
				{
					makeClass1Object(rock2, x, y);
				}
				// object is a rock
				else if(type.equals("RockBig:"))
				{
					makeClass2Object(rockBig, x, y);
				}
				// object is a tree
				else if(type.equals("Tree:"))
				{
					makeClass3Object(tree, x, y);
				}
				// object is a tree
				else if(type.equals("Tree2:"))
				{
					makeClass3Object(tree2, x, y);
				}
				// object is a tree
				else if(type.equals("Tree3:"))
				{
					makeClass3Object(tree3, x, y);
				}
				// object is a tree stump
				else if(type.equals("TreeStump:"))
				{
					makeClass1Object(treeStump, x, y);
				}
				// object is a dead tree
				else if(type.equals("DeadTree:"))
				{
					makeDeadTree(x, y);
				}
				// object is a wolf
				else if(type.equals("Wolf:"))
				{
					makeWolf(x, y);
				}
				// object is an enemy warrior
				else if(type.equals("EnemyWarrior:"))
				{
					makeEnemy(x, y, CharacterType.WARRIOR);
				}
				// object is an enemy mage
				else if(type.equals("EnemyMage:"))
				{
					makeEnemy(x, y, CharacterType.MAGE);
				}
				// object is an enemy rogue
				else if(type.equals("EnemyRogue:"))
				{
					makeEnemy(x, y, CharacterType.ROGUE);
				}
				else if(type.equals("Boss:"))
				{
					makeBoss(x, y);
				}
				// object is an NPC
				else if(type.equals("NPC:"))
				{
					i++;
					String name = fileTokens[i];
					i++;
					String type2 = fileTokens[i];
					makeNPC(npcTexture, x, y, name, type2);
				}
				// object is the transition object
				else if(type.equals("Transition:"))
				{
					makeTransition(straightPathH, x, y);
				}
				else if(type.equals("Transition2:"))
				{
					makeTransition(straightPath, x, y);
				}
				else if(type.equals("Transition3:"))
				{
					makeTransition(empty, x, y);
				}
				// object is the player
				else if(type.equals("Player:"))
				{
					makePartyMember(x, y, CharacterType.WARRIOR);
				}
				// object is the ally
				else if(type.equals("Mage:"))
				{
					makePartyMember(x, y, CharacterType.MAGE);
				}
				else if(type.equals("Rogue:"))
				{
					makePartyMember(x, y, CharacterType.ROGUE);
				}
				else if(type.equals("StraightPath:"))
				{
					makeSoftObject(straightPath, x, y);
				}
				else if(type.equals("StraightPathH:"))
				{
					makeSoftObject(straightPathH, x, y);
				}
				else if(type.equals("TPath:"))
				{
					makeSoftObject(tPath, x, y);
				}
				else if(type.equals("TPath2:"))
				{
					makeSoftObject(tPath2, x, y);
				}
				else if(type.equals("TPath3:"))
				{
					makeSoftObject(tPath3, x, y);
				}
				else if(type.equals("TPath4:"))
				{
					makeSoftObject(tPath4, x, y);
				}
				else if(type.equals("TurnPath:"))
				{
					makeSoftObject(turnPath, x, y);
				}
				else if(type.equals("TurnPath2:"))
				{
					makeSoftObject(turnPath2, x, y);
				}
				else if(type.equals("TurnPath3:"))
				{
					makeSoftObject(turnPath3, x, y);
				}
				else if(type.equals("TurnPath4:"))
				{
					makeSoftObject(turnPath4, x, y);
				}
				else if(type.equals("XPath:"))
				{
					makeSoftObject(xPath, x, y);
				}
				else if(type.equals("FenceCorner:"))
				{
					makeClass1Object(fenceCorner, x, y);
				}
				else if(type.equals("FenceCorner2:"))
				{
					makeClass1Object(fenceCorner2, x, y);
				}
				else if(type.equals("FenceCorner3:"))
				{
					makeClass1Object(fenceCorner3, x, y);
				}
				else if(type.equals("FenceCorner4:"))
				{
					makeClass1Object(fenceCorner4, x, y);
				}
				else if(type.equals("FenceHorizontal:"))
				{
					makeClass1Object(fenceHorizontal, x, y);
				}
				else if(type.equals("FenceVertical:"))
				{
					makeClass1Object(fenceVertical, x, y);
				}
				else if(type.equals("FlowerPatch:"))
				{
					makeSoftObject2(flowerPatch, x, y);
				}
				else if(type.equals("GrassTufts:"))
				{
					makeSoftObject2(grassTufts, x, y);
				}
				else if(type.equals("Log:"))
				{
					makeSoftObject(log, x, y);
				}
				else if(type.equals("Gate:"))
				{
					makeClass1Object(gateLength, x, y);
				}
				else if(type.equals("GateDoor:"))
				{
					TextureRegion gateDoorRegion = new TextureRegion(gateDoor, 0, 0, IMAGE_WIDTH*2, IMAGE_HEIGHT);
					makeGateDoor(gateDoorRegion, x, y);
				}
				else if (type.equals("Wall:"))
				{	
					makeClass1Object(empty, x, y);
				}
			}

			i++;
		}

		// make sure that a player, transition, and background were made
		if(background == null)
		{
			throw new Exception("Need to provide background in level file.");
		}
		if(transitionObject == null)
		{
			throw new Exception("Need to provide transition in level file.");
		}
		if(player.getWarrior() == null)
		{
			throw new Exception("Need to provide a player in level file.");
		}
		if(player.getMage() == null)
		{
			throw new Exception("Need to provide a mage in level file.");
		}
		if(player.getRogue() == null)
		{
			throw new Exception("Need to provide a rogue in level file.");
		}

	}

	private void makeGateDoor(TextureRegion gateDoor, float x, float y) throws Exception {
		
		//Make the animation
		Animation openAnimation = AssetLoader.getGateAnimation();
		
		GateDoor object = new GateDoor(x, y, gateDoor, OBJECT_WIDTH_LONG, OBJECT_HEIGHT, ObjectType.GATE, 
				openAnimation);
		collisionManager.subscribeObject(object);
		gameObjects.add(object);
		
	}

	/**
	 * Make the transition object
	 * @param image the image to be used by the object
	 * @param x the x position of the object
	 * @param y the y position of the object
	 * @throws Exception if the object cannot be made
	 */
	private void makeTransition(Texture image, float x, float y) throws Exception {
		// create the transition
		TransitionObject transitionObj = new TransitionObject(x, y, image, OBJECT_WIDTH, OBJECT_HEIGHT);
		collisionManager.subscribeObject(transitionObj);
		gameObjects.add(transitionObj);
		transitionObject = transitionObj;
	}

	/**
	 * Make an enemy object
	 * @param x the x position of the enemy
	 * @param y the y position of the enemy
	 * @throws Exception if the enemy cannot be made
	 */
	private void makeWolf(float x, float y) throws Exception
	{
		ArrayList<Animation> animations = AssetLoader.getWolfAnimations();

		// create the enemy
		Enemy e1 = new Enemy (x, y, AnimationState.FACING_LEFT, OBJECT_WIDTH, OBJECT_HEIGHT,
				ObjectType.ENEMY, CharacterType.WARRIOR, animations);
		//add him to the collision manager
		collisionManager.subscribeObject(e1);
		lineOfSightManager.subscribeCharacter(e1);
		gameObjects.add(e1);
		enemies.add(e1);
		numEnemies++;
	}

	/**
	 * Make a boss enemy
	 * @param x the starting x position
	 * @param y the starting y position
	 * @throws Exception if the boss cannot be made
	 */
	public void makeBoss(float x, float y) throws Exception
	{
		ArrayList<Animation> animations = AssetLoader.getBossAnimations();

		// create the enemy
		BigBoss boss = new BigBoss (x, y, AnimationState.FACING_LEFT, OBJECT_WIDTH, OBJECT_HEIGHT,
				ObjectType.ENEMY, CharacterType.MAGE, animations);
		//add him to the collision manager
		collisionManager.subscribeObject(boss);
		lineOfSightManager.subscribeCharacter(boss);
		gameObjects.add(boss);
		enemies.add(boss);
		numEnemies++;
	}

	/**
	 * Make an NPC (non-player character)
	 * @param picture the picture to be used by the NPC
	 * @param x the x position of the NPC
	 * @param y the y position of the NPC
	 * @throws Exception if the NPC cannot be made
	 */
	private void makeNPC(Texture picture, float x, float y, String name, String type) throws Exception
	{

		ArrayList<Animation> animations;

		if(type.equals("male"))
		{
			animations = AssetLoader.getNpcMaleAnimations();
		}
		else if(type.equals("female"))
		{
			animations = AssetLoader.getNpcFemaleAnimations();
		}
		else
		{
			animations = AssetLoader.getNpcKidAnimations();
		}

		// create the NPC
		NPC npc1;
		if(type.equals("male") || type.equals("female"))
		{
			npc1 = new NPC(x, y, picture, OBJECT_WIDTH, OBJECT_HEIGHT, name, animations);
		}
		else
		{
			npc1 = new NPC(x, y, picture, OBJECT_WIDTH, OBJECT_HEIGHT, name, animations);
		}
		//add him to the collision manager
		collisionManager.subscribeObject(npc1);
		gameObjects.add(npc1);
		npcs.add(npc1);
	}

	/**
	 * Make a solid object, 64x64
	 * @param image the image for the object
	 * @param x the x position of the object
	 * @param y the y position of the object
	 * @throws Exception
	 */
	private void makeClass1Object(Texture image, float x, float y) throws Exception
	{
		SolidObject object = new SolidObject(x, y, image, OBJECT_WIDTH, OBJECT_HEIGHT, ObjectType.SHORT_OBSTACLE);
		collisionManager.subscribeObject(object);
		gameObjects.add(object);
	}
	/**
	 * Make a solid object, 128x64
	 * @param image the image for the object
	 * @param x the x position of the object
	 * @param y the y position of the object
	 * @throws Exception
	 */
	private void makeClass2Object(Texture image, float x, float y) throws Exception
	{
		SolidObject object = new SolidObject(x, y, image, OBJECT_WIDTH_LONG, OBJECT_HEIGHT, ObjectType.SHORT_OBSTACLE);
		collisionManager.subscribeObject(object);
		gameObjects.add(object);
	}
	/**
	 * Make a solid object, 64x128
	 * @param image the image for the object
	 * @param x the x position of the object
	 * @param y the y position of the object
	 * @throws Exception
	 */
	private void makeClass3Object(Texture image, float x, float y) throws Exception
	{
		SolidObject object = new SolidObject(x, y, image, OBJECT_WIDTH, OBJECT_HEIGHT_TALL, ObjectType.TALL_OBSTACLE);
		collisionManager.subscribeObject(object);
		gameObjects.add(object);
	}

	/**
	 * Make a soft object
	 * @param image the image for the object
	 * @param x the x position of the object
	 * @param y the y position of the object
	 * @throws Exception
	 */
	private void makeSoftObject(Texture image, float x, float y) throws Exception
	{
		SolidObject object = new SolidObject(x, y, image, OBJECT_WIDTH, OBJECT_HEIGHT, ObjectType.SOFT);
		softObjects.add(object);

		//TODO
	}
	/**
	 * Make a large soft object
	 * @param image the image for the object
	 * @param x the x position of the object
	 * @param y the y position of the object
	 * @throws Exception
	 */
	private void makeSoftObject2(Texture image, float x, float y) throws Exception
	{
		SolidObject object = new SolidObject(x, y, image, OBJECT_WIDTH_LONG, OBJECT_HEIGHT_TALL, ObjectType.SOFT);
		softObjects.add(object);

		//TODO
	}

	/**
	 * Make a dead tree
	 * @throws Exception 
	 * 
	 */
	private void makeDeadTree(float x, float y) throws Exception {
		DeadTree object = new DeadTree(x, y);

		collisionManager.subscribeObject(object);
		gameObjects.add(object);
		deadtrees.add(object);

	}

	/**
	 * Make a camera to view the screen
	 * @param w the width of the camera view
	 * @param h the height of the camera view
	 */
	private void makeCamera(float w, float h)
	{
		camera = new OrthographicCamera(1, h/w);
	}

	/**
	 * Get the camera
	 * @return
	 */
	public OrthographicCamera getCamera()
	{
		return camera;
	}

	/**
	 * Make the background texture
	 * @param backgroundWidth the width of the background
	 * @param backgroundHeight the height of the background
	 */
	private void makeBackground(String backgroundFilePath)
	{
		Texture backgroundTexture = new Texture(Gdx.files.internal(backgroundFilePath));
		textures.add(backgroundTexture);

		background = new TextureRegion(backgroundTexture, 0, 0, 1024, 1024);

	}

	/**
	 * Make a player object at the given coordinate
	 * @param startingX the x coordinate
	 * @param startingY the y coordinate
	 * @throws Exception 
	 */
	private void makeEnemy(float startingX, float startingY, CharacterType type) throws Exception
	{
		// animations should match the order in Character.makeAnimations
		ArrayList<Animation> animations;

		if(type == CharacterType.WARRIOR)
		{
			animations = AssetLoader.getFighterAnimations();
		}
		else if(type == CharacterType.MAGE)
		{
			animations = AssetLoader.getWizardAnimations();
		}
		else
		{
			animations = AssetLoader.getThiefAnimations();
		}

		Enemy enemy = new Enemy(startingX, startingY, AnimationState.FACING_RIGHT, OBJECT_WIDTH,
				OBJECT_HEIGHT, ObjectType.ENEMY, type, animations);

		collisionManager.subscribeObject(enemy);
		lineOfSightManager.subscribeCharacter(enemy);
		gameObjects.add(enemy);
		enemies.add(enemy);
		numEnemies++;
	}

	/**
	 * Make a player object at the given coordinate
	 * @param startingX the x coordinate
	 * @param startingY the y coordinate
	 * @throws Exception 
	 */
	private void makePartyMember(float startingX, float startingY, CharacterType type) throws Exception
	{
		ArrayList<Animation> animations;

		if(type == CharacterType.WARRIOR)
		{
			animations = AssetLoader.getWarriorAnimations();
		}
		else if(type == CharacterType.MAGE)
		{
			animations = AssetLoader.getMageAnimations();
		}
		else
		{
			animations = AssetLoader.getRogueAnimations();
		}

		ObjectType objectType;
		if(player.getCurrentPlayer() == type)
		{
			objectType = ObjectType.PRIMARY;
		}
		else if(player.getCommanded() == type)
		{
			objectType = ObjectType.SECONDARY;
		}
		else
		{
			objectType = ObjectType.TERTIARY;
		}

		PartyMember partyMember = new PartyMember(startingX, startingY, AnimationState.FACING_RIGHT, OBJECT_WIDTH,
				OBJECT_HEIGHT, objectType, type, animations);

		collisionManager.subscribeObject(partyMember);
		lineOfSightManager.subscribeCharacter(partyMember);
		gameObjects.add(partyMember);
		allies.add(partyMember);
		if(type == CharacterType.WARRIOR)
		{
			player.setWarrior(partyMember);
		}
		else if(type == CharacterType.MAGE)
		{
			player.setMage(partyMember);
		}
		else
		{
			player.setRogue(partyMember);
		}
	}

	/*
	 * @see com.badlogic.gdx.Screen#dispose()
	 */
	@Override
	public void dispose() {
		for(Texture texture : textures)
		{
			texture.dispose();
		}
		gameObjects.clear();
		spriteBatch.dispose();
	}

	/*
	 * @see com.badlogic.gdx.Screen#render(float)
	 */
	@Override
	public void render(float delta) {
		// while game isn't over, run the game loop
		if(!transitionObject.switchScreen())
		{
			try {
				update(delta);
			} catch (Exception e) {
				e.printStackTrace();
			}
			clearScreen();

			camera.update();

			draw();
		}
		// otherwise the game is over
		else
		{
			clearScreen();
			camera.update();
			game.setScreen(level + 1);
		}

	}

	/**
	 * Update the objects in the scene
	 * @param delta
	 * @throws Exception 
	 */
	private void update(float delta) throws Exception
	{
		// update the graph
		graph.update();

		//update dialogue
		dialogueHandler.update();

		if(clock.isPaused()){
			return;
		}else{
			clock.tick();
		}
		// check for character switch
		if((Gdx.input.isKeyPressed(Keys.TAB) || Gdx.input.isKeyPressed(Keys.SHIFT_LEFT))
				&& !characterSwitch)
		{
			player.switchCharacter();
			characterSwitch = true;
		}
		else if(!(Gdx.input.isKeyPressed(Keys.TAB) || Gdx.input.isKeyPressed(Keys.SHIFT_LEFT)) && characterSwitch)
		{
			characterSwitch = false;
		}

		// add new objects
		for(GameObject object : newObjects)
		{
			if(!gameObjects.contains(object))
			{
				gameObjects.add(object);
			}
		}
		newObjects.clear();

		// update objects
		for(GameObject object : gameObjects)
		{
			object.update(delta);
		}

		// check for collisions
		collisionManager.checkCollisions();

		// update what the character's can see
		lineOfSightManager.checkLineOfSight();

		//check for deaths
		checkForDeath();

		// revive party members if enemies are all dead
		if(numEnemies <= 0)
		{
			player.reviveParty();
		}

		// update the HUD
		hud.update();

		//update dialogue
		dialogueHandler.update();

		//clock
		delta = (System.currentTimeMillis() - time)%2000;
		//System.out.println(delta);
		if (delta >= 2001)
		{
			System.out.println(delta);
			delta = 0;
			System.out.println("refersh");
		}

	}

	/**
	 * Clear the screen
	 */
	private void clearScreen()
	{
		Gdx.gl.glClearColor(1, 1, 1, 1);
		Gdx.gl.glClear(GL10.GL_COLOR_BUFFER_BIT);
	}

	/**
	 * Draw the objects on the screen
	 */
	private void draw()
	{
		spriteBatch.setProjectionMatrix(camera.combined);
		spriteBatch.begin();

		// draw background
		spriteBatch.draw(background, screenDimensions.getX(), screenDimensions.getY(), screenDimensions.getWidth(), screenDimensions.getHeight());

		// draw soft objects
		for(GameObject object : softObjects)
		{
			if (object.getImageToDraw() != null)
			{
				spriteBatch.draw(object.getImageToDraw(), object.getX(), object.getY(), object.getWidth(), object.getHeight());
			}
		}

		// draw objects
		ArrayList<EnemyHealthBar> healthBars = new ArrayList<EnemyHealthBar>();
		for(GameObject object : gameObjects)
		{
			if (object.getImageToDraw() != null)
			{
				// if the object is a projectile
				if(object.getObjectType() == ObjectType.ALLYPROJECTILE ||
						object.getObjectType() == ObjectType.ENEMYPROJECTILE)
				{
					// rotate the projectile to the correct angle
					Projectile projectile = (Projectile) object;
					spriteBatch.draw(object.getImageToDraw(), object.getX(), object.getY(),
							object.getWidth()/2.0f, object.getHeight()/2.0f,
							object.getWidth(), object.getHeight(), 1, 1, projectile.getAngle());
				}
				// otherwise draw it as normal
				else
				{
					// draw halo for player controlled character
					if(object.getObjectType() == ObjectType.PRIMARY)
					{
						spriteBatch.draw(primaryHalo, object.getX(), object.getY() - 0.011f, object.getWidth(), object.getHeight());
					}
					// draw halos for allies
					if(object.getObjectType() == ObjectType.SECONDARY)
					{
						// only draw the halo if the character is alive
						Character character = (Character) object;
						if(!character.isDead())
						{
							spriteBatch.draw(secondaryHalo, object.getX(), object.getY() - 0.011f, object.getWidth(), object.getHeight());
						}
					}
					if(object.getObjectType() == ObjectType.TERTIARY)
					{
						// only draw the halo if the character is alive
						Character character = (Character) object;
						if(!character.isDead())
						{
							spriteBatch.draw(tertiaryHalo, object.getX(), object.getY() - 0.011f, object.getWidth(), object.getHeight());
						}
					}
					if(object.getObjectType() == ObjectType.ENEMY)
					{
						Enemy enemy = (Enemy) object;
						EnemyHealthBar healthBar = enemy.getHealthBar();
						healthBars.add(healthBar);
					}
					if(object.getObjectType() == ObjectType.PRIMARY || object.getObjectType() == ObjectType.SECONDARY ||
							object.getObjectType() == ObjectType.TERTIARY || object.getObjectType() == ObjectType.ENEMY)
					{
						Character character = (Character) object;
						// draw halo for being taunted
						if(character.isTaunted())
						{
							spriteBatch.draw(tauntedHalo, object.getX(), object.getY() - 0.011f, object.getWidth(), object.getHeight());
						}
						// draw halo for being attacked
						if(character.wasAttacked())
						{
							spriteBatch.draw(damageHalo, object.getX(), object.getY() - 0.011f, object.getWidth(), object.getHeight());
						}
					}
					spriteBatch.draw(object.getImageToDraw(), object.getX(), object.getY(), object.getWidth(), object.getHeight());
				}

				// if the object is an NPC
				if (object.getObjectType() == ObjectType.NPC)
				{
					test = (NPC) object;
				}
			}

		}
		for(EnemyHealthBar healthBar : healthBars)
		{
			healthBar.draw(spriteBatch, healthBar.getOwner().getHealth(), healthBar.getOwner().getMaxHealth());
		}

		dialogueHandler.draw(spriteBatch);

		//draw the HUD
		hud.draw(spriteBatch);
		
		levelIconOne.draw(spriteBatch);
		if(level + 1 > 9)
		{
			levelIconTen.draw(spriteBatch);
		}

		spriteBatch.end();
	}


	/*
	 * @see com.badlogic.gdx.Screen#resize(int, int)
	 */
	@Override
	public void resize(int width, int height) {
	}

	/*
	 * @see com.badlogic.gdx.Screen#pause()
	 */
	@Override
	public void pause(){
	}

	/*
	 * @see com.badlogic.gdx.Screen#resume()
	 */
	@Override
	public void resume() {
	}

	/*
	 * @see com.badlogic.gdx.Screen#show()
	 */
	@Override
	public void show() {

		// create the player class if necessary
		if(Player.getInstance() == null)
		{
			Player.createInstance(game);
		}
		// create the clock if necessary
		if(Clock.getInstance() == null)
		{
			Clock.createInstance();
		}
		
		HUD.createInstance();
		
		textures = new ArrayList<Texture>();

		player = Player.getInstance();
		hud = HUD.getInstance();
		dialogueHandler = DialogueHandler.getInstance();
		clock = Clock.getInstance();
		collisionManager = CollisionManager.makeNewInstance();
		lineOfSightManager = new LineOfSightManager();
		aiManager = AIManager.getNewInstance();

		screenDimensions = new Rectangle(-0.5f, -0.3f, 1.0f, 0.8f);

		Gdx.graphics.setDisplayMode(BACKGROUND_WIDTH, BACKGROUND_HEIGHT, false);
		float w = Gdx.graphics.getWidth();
		float h = Gdx.graphics.getHeight();

		spriteBatch = new SpriteBatch();

		newObjects = new ArrayList<GameObject>();
		deadObjects = new ArrayList<GameObject>();
		gameObjects = new ArrayList<GameObject>();
		softObjects = new ArrayList<GameObject>();
		enemies = new ArrayList<Character>();
		allies = new ArrayList<Character>();
		npcs = new ArrayList<NPC>();
		deadtrees = new ArrayList<DeadTree>();

		//time = System.currentTimeMillis();

		makeCamera(w, h);
		
		numEnemies = 0;

		try {
			makeObjects(levelPath);
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		try {
			hud.initHUD(game);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		dialogueHandler.setInitTrue(level);

		font = new BitmapFont();
		font.setColor(new Color(0.0f, 0.0f, 0.0f, 1.0f));
		font.setScale(0.2f, 0.03f);

		Texture pHaloTexture = new Texture(Gdx.files.internal("assets/data/player_halo.png"));
		primaryHalo = new TextureRegion(pHaloTexture, 0, 0, IMAGE_WIDTH, IMAGE_HEIGHT);
		textures.add(pHaloTexture);
		Texture aHaloTexture = new Texture(Gdx.files.internal("assets/data/ally_halo.png"));
		secondaryHalo = new TextureRegion(aHaloTexture, 0, 0, IMAGE_WIDTH, IMAGE_HEIGHT);
		textures.add(aHaloTexture);
		tertiaryHalo = new TextureRegion(aHaloTexture, 0, 0, IMAGE_WIDTH, IMAGE_HEIGHT);
		Texture dHaloTexture = new Texture(Gdx.files.internal("assets/data/damage_halo.png"));
		damageHalo = new TextureRegion(dHaloTexture, 0, 0, IMAGE_WIDTH, IMAGE_HEIGHT);
		textures.add(dHaloTexture);
		Texture tHaloTexture = new Texture(Gdx.files.internal("assets/data/taunt_halo.png"));
		tauntedHalo = new TextureRegion(tHaloTexture, 0, 0, IMAGE_WIDTH, IMAGE_HEIGHT);	
		textures.add(tHaloTexture);

		DialogueHandler.getInstance().onEntry();

		ProgressJournal.makeNewInstance();
		ProgressJournal.onExit(level - 1);
		// make new graph
		this.graph = new Graph();
		
		int levelOne;
		int levelTen;
		
		if(level + 1 > 9)
		{
			levelTen = 1;
			levelOne = level + 1 - 10;
		}
		else
		{
			levelTen = 0;
			levelOne = level + 1;
		}
		
		levelIconOne = new DigitIcon(0.44f, -0.365f, levelOne, 1, "white");
		levelIconTen = new DigitIcon(0.41f, -0.365f, levelTen, 1, "white");
	}

	/*
	 * @see com.badlogic.gdx.Screen#hide()
	 */
	@Override
	public void hide() {
		clearScreen();
	}

	/**
	 * Checks if any objects are dead, and removes them if they are
	 * @throws Exception if an object cannot be removed
	 */
	private void checkForDeath() throws Exception {

		// iterates through game objects
		for (GameObject obj : gameObjects)
		{
			if (obj.isMarkedForDeath())
			{
				if(obj.getObjectType() != ObjectType.EXPLOSION && 
						obj.getObjectType() != ObjectType.POISON_CLOUD &&
						obj.getObjectType() != ObjectType.HEALING_SPARKLES)
				{
					collisionManager.unsubscribeObject(obj);
				}
				deadObjects.add(obj);
			}
		}

		// remove dead objects
		for(GameObject obj : deadObjects)
		{
			if(obj.getObjectType() == ObjectType.ENEMY){
				addNewObject(new ItemDrop(obj.getX(), obj.getY()));
			}
			gameObjects.remove(obj);

			// keep track of enemy deaths
			if(obj.getObjectType() == ObjectType.ENEMY)
			{
				numEnemies--;
				if(numEnemies <= 0){
					ProgressJournal.onEnemyDeath(level);
					DialogueHandler.getInstance().onEnemyDeath();
				}
			}
		}
		deadObjects.clear();
	}

	/**
	 * Remove an object explicitly
	 * @param object the object to be removed
	 */
	public void removeObject(GameObject object)
	{
		if(gameObjects.contains(object))
		{
			deadObjects.add(object);
		}
	}

	/**
	 * Adds a new object to the screen
	 * @param newObject the new object
	 */
	public void addNewObject(GameObject newObject)
	{
		if (!newObjects.contains(newObject))
		{
			newObjects.add(newObject);
		}
	}

	/**
	 * Get the list of enemies
	 * @return enemies
	 */
	public List<Character> getEnemyList()
	{
		return enemies;
	}

	/**
	 * Get the list of dead trees
	 * @return enemies
	 */
	public List<DeadTree> getDeadTreeList()
	{
		return deadtrees;
	}

	/**
	 * Get the list of player allies
	 * @return allies
	 */
	public List<Character> getAllyList()
	{
		return allies;
	}

	/**
	 * Get the path to the level file for this screen
	 * @return levelPath
	 */
	public String getLevelPath()
	{
		return levelPath;
	}

	/**
	 * Get list of all gameObjects
	 * @return gameObjects
	 */
	public List<GameObject> getAllObjects()
	{
		return gameObjects;
	}

	/**
	 * Cast a ray between two points and return the objects it collided with
	 * @param startX the x coordinate of the ray's start
	 * @param startY the y coordinate of the ray's start
	 * @param endX the x coordinate of the ray's end
	 * @param endY the y coordinate of the ray's end
	 * @return the collection of objects the ray collided with
	 */
	public ArrayList<GameObject> castRay(float startX, float startY, float endX, float endY)
	{
		// figure out the angle of the ray
		float diffX = endX - startX;
		float diffY = endY - startY;
		double angle =  Math.atan2(diffY, diffX);

		// get the length of the ray
		float len = Math.abs(diffX*diffX + diffY*diffY);

		// the interval for checking collisions along the line
		float interval = 0.05f;

		// get the number of intervals
		float numIntervals = len/interval;

		// get the intervals along the x and y
		float xInterval = interval*(float)Math.cos(angle);
		float yInterval = interval*(float)Math.sin(angle);

		float checkPointX = startX;
		float checkPointY = startY;

		ArrayList<GameObject> collisions = new ArrayList<GameObject>();

		// iterate over the ray and check for collisions
		for(int i = 0 ; i < numIntervals ; i ++)
		{
			checkPointX += xInterval;
			checkPointY += yInterval;

			// check if the ray has collided with an object
			for(GameObject object : gameObjects)
			{
				if(object.getCollider().contains(checkPointX, checkPointY))
				{
					collisions.add(object);
				}
			}
		}

		return collisions;
	}

	/**
	 * Get the number of enemies left
	 * @return numEnemies
	 */
	public int getNumEnemiesLeft()
	{
		return numEnemies;
	}

	public NPC getNPC(String name) {

		for(int i = 0; i < npcs.size(); i++){
			if(name.equals(npcs.get(i).getName())){
				return npcs.get(i);
			}
		}
		return null;

	}

	/**
	 * Get the level number
	 * @return level
	 */
	public int getLevel()
	{
		return level;
	}

	/**
	 * Get the dimensions of the screen as a rectangle
	 * @return screenDimensions
	 */
	public Rectangle getScreenDimensions()
	{
		return screenDimensions;
	}

	public boolean isDialogueFinished() {
		return dialogueHandler.isFinished();
	}
	
	public Graph getGraph()
	{
		return graph;
	}
	
	/**
	 * Add a texture to the list so it can be disposed of
	 * @param texture the texture to add
	 */
	public void addTexture(Texture texture)
	{
		textures.add(texture);
	}
}

