/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package game;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.audio.Sound;

import gameobject.ObjectType;
import gameobject.character.AnimationState;
import gameobject.character.CharacterType;
import gameobject.character.PartyMember;

/**
 * The class embodies the [physical] player including the three player characters,
 * and the inventory
 * @author Eyleen
 */
public class Player {
	private PartyMember warrior;
	private PartyMember mage;
	private PartyMember rogue;
	private CharacterType currentPlayer;
	private CharacterType commanded;
	
	private int numHealthPotion = 5;
	private int numEnergyPotion = 3;
	
	private static Player instance = null;
	private final QuestMain game;
	
	private Player(QuestMain g){
		game = g;
		currentPlayer = CharacterType.WARRIOR;
		commanded = CharacterType.MAGE;
	}
	
	//initialize the audio file
	Sound switchSound = Gdx.audio.newSound(Gdx.files.internal("assets/data/sound/characterswitch.wav"));
	Sound potionUseSound = Gdx.audio.newSound(Gdx.files.internal("assets/data/sound/potionuse.wav"));
		
	
	public static void createInstance(QuestMain g){
		instance = new Player(g);
	}
	
	public void setHealthPotionNumber(int hp){
		numHealthPotion = hp;
	}
	
	public void setEnergyPotionNumber(int sp){
		numEnergyPotion = sp;
	}
	
	public static Player getInstance(){
		return instance;
	}
	
	public void switchCharacter()
	{
		PartyMember playerCharacter = warrior;
		PartyMember nextCharacter = mage;
		PartyMember thirdCharacter = rogue;
		if(warrior.getObjectType() == ObjectType.PRIMARY)
		{
			playerCharacter = warrior;
			nextCharacter = mage;
			thirdCharacter = rogue;
		}
		else if(mage.getObjectType() == ObjectType.PRIMARY)
		{
			playerCharacter = mage;
			nextCharacter = rogue;
			thirdCharacter = warrior;
		}
		else if(rogue.getObjectType() == ObjectType.PRIMARY)
		{
			playerCharacter = rogue;
			nextCharacter = warrior;
			thirdCharacter = mage;
		}
		
		//play sound
		switchSound.play(1.0f);
		
		// refresh the ai in preparation for the switch
		playerCharacter.refreshAI();
		
		// try to switch
		if(!nextCharacter.isDead())
		{
			if(nextCharacter.getObjectType() == ObjectType.SECONDARY)
			{
				playerCharacter.setObjectType(ObjectType.SECONDARY);
			}
			else
			{
				playerCharacter.setObjectType(ObjectType.TERTIARY);
			}
			nextCharacter.setObjectType(ObjectType.PRIMARY);
		}
		else if(!thirdCharacter.isDead())
		{
			playerCharacter.setObjectType(ObjectType.SECONDARY);
			thirdCharacter.setObjectType(ObjectType.PRIMARY);
		}
		else if(!playerCharacter.isDead())
		{
			// do nothing, no one alive to switch to
		}
		else
		{
			// everyone is dead
			game.gameOver();
		}
		
		// set the player type
		if(warrior.getObjectType() == ObjectType.PRIMARY)
		{
			currentPlayer = CharacterType.WARRIOR;
		}
		else if(mage.getObjectType() == ObjectType.PRIMARY)
		{
			currentPlayer = CharacterType.MAGE;
		}
		else if(rogue.getObjectType() == ObjectType.PRIMARY)
		{
			currentPlayer = CharacterType.ROGUE;
		}
		
		// set the commanded type
		if(warrior.getObjectType() == ObjectType.SECONDARY)
		{
			commanded = CharacterType.WARRIOR;
		}
		else if(mage.getObjectType() == ObjectType.SECONDARY)
		{
			commanded = CharacterType.MAGE;
		}
		else if(rogue.getObjectType() == ObjectType.SECONDARY)
		{
			commanded = CharacterType.ROGUE;
		}
	}
	
	/**
	 * Switch whoever's commanded
	 * @param newCommanded
	 */
	public void switchCommanded(CharacterType newCommanded)
	{
		if(currentPlayer != newCommanded)
		{
			if(newCommanded == CharacterType.WARRIOR)
			{
				if(commanded == CharacterType.MAGE)
				{
					mage.setObjectType(ObjectType.TERTIARY);
					warrior.setObjectType(ObjectType.SECONDARY);
				}
				else if(commanded == CharacterType.ROGUE)
				{
					rogue.setObjectType(ObjectType.TERTIARY);
					warrior.setObjectType(ObjectType.SECONDARY);
				}
			}
			else if(newCommanded == CharacterType.MAGE)
			{
				if(commanded == CharacterType.WARRIOR)
				{
					warrior.setObjectType(ObjectType.TERTIARY);
					mage.setObjectType(ObjectType.SECONDARY);
				}
				else if(commanded == CharacterType.ROGUE)
				{
					rogue.setObjectType(ObjectType.TERTIARY);
					mage.setObjectType(ObjectType.SECONDARY);
				}
			}
			else
			{
				if(commanded == CharacterType.WARRIOR)
				{
					warrior.setObjectType(ObjectType.TERTIARY);
					rogue.setObjectType(ObjectType.SECONDARY);
				}
				else if(commanded == CharacterType.MAGE)
				{
					mage.setObjectType(ObjectType.TERTIARY);
					rogue.setObjectType(ObjectType.SECONDARY);
				}
			}
			
			commanded = newCommanded;
		}
	}
	
	public void reviveParty()
	{
		warrior.revive();
		warrior.resetBooleans();
		mage.revive();
		mage.resetBooleans();
		rogue.revive();
		rogue.resetBooleans();
	}
	
	public void setWarrior(PartyMember w){
		warrior = w;
	}
	
	public PartyMember getWarrior(){
		return warrior;
	}
	
	public void setMage(PartyMember m){
		mage = m;
	}
	
	public PartyMember getMage(){
		return mage;
	}
	
	public void setRogue(PartyMember r){
		rogue = r;
	}
	
	public PartyMember getRogue(){
		return rogue;
	}
	
	public void addHealthPotion(){
		if(numHealthPotion < 9)
			numHealthPotion++;
	}
	
	public void useHealthPotion(){
		if(numHealthPotion <= 0){
			return;
		}
		
		//get object type player
				CharacterType controlledType = getCurrentPlayer();
				PartyMember controlled = null;
				
				switch(controlledType)
				{
					case WARRIOR:
						controlled = warrior;
						break;
					case MAGE:
						controlled = mage;
						break;
					case ROGUE:
						controlled = rogue;
						break;
				}
				
				switch(controlled.getDirection())
				{
					case RIGHT:
						controlled.setCharacterState(AnimationState.POTION_RIGHT);
						break;
					case LEFT:
						controlled.setCharacterState(AnimationState.POTION_LEFT);
						break;
					case UP:
						controlled.setCharacterState(AnimationState.POTION_UP);
						break;
					case DOWN:
						controlled.setCharacterState(AnimationState.POTION_DOWN);
						break;
				}		
		
		numHealthPotion--;
		//play sound
		potionUseSound.play(1.0f);
		
		if(warrior.isActivePartyMember()){
			warrior.increaseHealth(50);
		}else if(mage.getObjectType() == ObjectType.PRIMARY){
			mage.increaseHealth(50);
		}else{
			rogue.increaseHealth(50);
		}
	}
	
	public int getHealthPotionNumber(){
		return numHealthPotion;
	}
	
	public void addEnergyPotion(){
		if(numEnergyPotion < 9)
			numEnergyPotion++;
	}
	
	public void useEnergyPotion(){
		if(numEnergyPotion <= 0){
			return;
		}
		
		//play sound
		potionUseSound.play(1.0f);
		numEnergyPotion--;
		
		//get object type player
		CharacterType controlledType = getCurrentPlayer();
		PartyMember controlled = null;
		
		switch(controlledType)
		{
			case WARRIOR:
				controlled = warrior;
				break;
			case MAGE:
				controlled = mage;
				break;
			case ROGUE:
				controlled = rogue;
				break;
		}
		
		switch(controlled.getDirection())
		{
			case RIGHT:
				controlled.setCharacterState(AnimationState.POTION_RIGHT);
				break;
			case LEFT:
				controlled.setCharacterState(AnimationState.POTION_LEFT);
				break;
			case UP:
				controlled.setCharacterState(AnimationState.POTION_UP);
				break;
			case DOWN:
				controlled.setCharacterState(AnimationState.POTION_DOWN);
				break;
		}
		
		if(warrior.isActivePartyMember()){
			warrior.increaseEnergy(50);
		}else if(mage.getObjectType() == ObjectType.PRIMARY){
			mage.increaseEnergy(50);
		}else{
			rogue.increaseEnergy(50);
		}
	}
	
	public int getEnergyPotionNumber(){
		return numEnergyPotion;
	}
	
	public CharacterType getCurrentPlayer()
	{
		return currentPlayer;
	}
	
	public CharacterType getCommanded()
	{
		return commanded;
	}
}
