/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package gameobject;

import java.util.ArrayList;

import pathfinding.GraphNode;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Circle;
import com.badlogic.gdx.math.Rectangle;

import event.CollisionEvent;

/**
 * Abstract game object with a position, size, and type
 * @author jordan
 * @version Oct 4 2013
 */
public abstract class AbstractGameObject implements GameObject {
	
	// type
	protected ObjectType objectType;
	
	// tag
	protected String tag;
	
	// fields
	protected float x;
	protected float y;
	protected float width;
	protected float height;
	
	private ArrayList<GraphNode> occupiedNodes;
	
	protected boolean markedForDeath = false;
	
	protected String name = "Default";
	
	/**
	 * Constructor
	 * @param x the x position of the object
	 * @param y the y position of the object
	 * @param width the width of the object
	 * @param height the height of the object
	 * @param type the type of the object
	 */
	public AbstractGameObject(float x, float y, float width, float height, ObjectType type)
	{
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		
		objectType = type;
		
		occupiedNodes = new ArrayList<GraphNode>();
		
		tag = "Default";
	}

	/*
	 * @see gameobject.GameObject#getType()
	 */
	@Override
	public ObjectType getObjectType() {
		return objectType;
	}

	/*
	 * @see gameobject.GameObject#update(float)
	 */
	@Override
	public void update(float delta) {
		// will be overridden by subclasses
	}

	/*
	 * @see gameobject.GameObject#getImageToDraw()
	 */
	@Override
	public TextureRegion getImageToDraw() {
		// will be overridden by subclasses
		return null;
	}

	/*
	 * @see gameobject.GameObject#getX()
	 */
	@Override
	public float getX() {
		return x;
	}

	/*
	 * @see gameobject.GameObject#getY()
	 */
	@Override
	public float getY() {
		return y;
	}
	
	/*
	 * @see gameobject.GameObject#getCenterX()
	 */
	@Override
	public float getCenterX() {
		return x + width/2.00f;
	}
	
	/*
	 * @see gameobject.GameObject#getCenterY()
	 */
	@Override
	public float getCenterY() {
		return y + height/2.00f;
	}

	/*
	 * @see gameobject.GameObject#getWidth()
	 */
	@Override
	public float getWidth() {
		return width;
	}

	/*
	 * @see gameobject.GameObject#getHeight()
	 */
	@Override
	public float getHeight() {
		return height;
	}

	/*
	 * @see gameobject.GameObject#getCollider()
	 */
	@Override
	public Rectangle getCollider() {
		return new Rectangle(x, y, width, height);
	}

	/*
	 * @see gameobject.GameObject#handleCollision(event.CollisionEvent)
	 */
	@Override
	public void handleCollision(CollisionEvent cevent) {
		// overridden by subclasses
	}
	
	/*
	 * @see gameobject.GameObject#isMarkedForDeath()
	 */
	@Override
	public boolean isMarkedForDeath()
	{
		return markedForDeath;
	}

	/*
	 * @see gameobject.GameObject#getTag()
	 */
	public String getTag()
	{
		return tag;
	}
	
	/*
	 * @see gameobject.GameObject#setTag(java.lang.String)
	 */
	public void setTag(String newTag)
	{
		tag = newTag;
	}
	
	/*
	 * @see gameobject.GameObject#getOccupiedNodes()
	 */
	public ArrayList<GraphNode> getOccupiedNodes()
	{
		return occupiedNodes;
	}
	
	/*
	 * @see gameobject.GameObject#addOccupiedNode(pathfinding.GraphNode)
	 */
	public void addOccupiedNode(GraphNode node)
	{
		if(!occupiedNodes.contains(node))
		{
			occupiedNodes.add(node);
		}
	}
	
	/*
	 * @see gameobject.GameObject#removeOccupiedNode(pathfinding.GraphNode)
	 */
	public void removeOccupiedNode(GraphNode node)
	{
		if(occupiedNodes.contains(node))
		{
			occupiedNodes.remove(node);
		}
	}
	
	/*
	 * @see gameobject.GameObject#getAttackZone()
	 */
	public Circle getAttackZone()
	{
		return new Circle(getCenterX(), getCenterY(), 0.05f);
	}
	
	/*
	 * @see gameobject.GameObject#getName()
	 */
	public String getName()
	{
		return name;
	}

}
