/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package gameobject;

import java.util.ArrayList;

import pathfinding.GraphNode;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Circle;
import com.badlogic.gdx.math.Rectangle;

import event.CollisionEvent;

/**
 * Interface representing a basic game object
 * @author jordan
 * @version Oct 3 2013
 */
public interface GameObject {


	/**
	 * Get the type of the object
	 * @return the type of the object
	 */
	ObjectType getObjectType();

	/**
	 * Update the object
	 * @param delta the time since the last update
	 */
	void update(float delta);
	
	/**
	 * Gets the image to be drawn
	 * @return the image as a TextureRegion
	 */
	TextureRegion getImageToDraw();
	
	/**
	 * Get the x position of the object
	 * @return the x position as a float
	 */
	float getX();
	
	/**
	 * Get the y position of the object
	 * @return the y position as a float
	 */
	float getY();
	
	/**
	 * Get the x position of the center of the object
	 * @return the center x position as a float
	 */
	float getCenterX();
	
	/**
	 * Get the y position of the center of the object
	 * @return the center y position as a float
	 */
	float getCenterY();
	
	/**
	 * Get the width of the object
	 * @return the width as a float
	 */
	float getWidth();
	
	/**
	 * Get the height of the object
	 * @return the height as a float
	 */
	float getHeight();
	
	/**
	 * Get the rectangle that represents the bounds of the object
	 * @return the rectangle
	 */
	Rectangle getCollider();
	
	/**
	 * Handle a collision event when received
	 * @param cevent the event object
	 */
	void handleCollision(CollisionEvent cevent);
	
	/**
	 * Is the object marked for death
	 * @return true if marked for death, false otherwise
	 */
	boolean isMarkedForDeath();

	/**
	 * Get the object's tag
	 * @return
	 */
	String getTag();
	
	/**
	 * Set the object's tag
	 */
	void setTag(String newTag);
	
	/**
	 * Get the nodes occupied by the game object
	 * @return
	 */
	ArrayList<GraphNode> getOccupiedNodes();
	
	/**
	 * Add a node that is occupied by the game object
	 * @param node the node occupied by the object
	 */
	void addOccupiedNode(GraphNode node);
	
	/**
	 * Remove a node that is no longer occupied by the game object
	 * @param node the node no longer occupied
	 */
	void removeOccupiedNode(GraphNode node);
	
	/**
	 * Get the zone within which this character can be attacked
	 * @return the attack zone as a Circle
	 */
	public Circle getAttackZone();
	
	/**
	 * Get the character's name as a string
	 * @return name
	 */
	public String getName();
	
}
