/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package gameobject;

import event.CollisionEvent;
import game.AssetLoader;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.TextureRegion;

/**
 * Class representing a projectile fired by a rogue
 * @author jordan
 * @version Jan 6 2014
 */
public class Projectile extends AbstractGameObject {
	
	private double speed;
	
	private float range;
	private int damage;
	private double angle;
	
	private float traveled;
	
	private TextureRegion image;
	
	/**
	 * Constructor
	 * @param x the starting x position of the projectile
	 * @param y the starting y position of the projectile
	 * @param width the width of the projectile
	 * @param height the height of the projectile
	 * @param type the type of object
	 * @param destX the destination x position
	 * @param destY the destination y position
	 * @param speed the speed of the projectile
	 * @param range the range of the projectile
	 * @param damage the damage caused by the projectile
	 */
	public Projectile(float x, float y, float width, float height,
			ObjectType type, double destX, double destY, float speed, float range, int damage) {
		super(x, y, width, height, type);

		this.speed = speed;
		this.range = range;
		this.damage = damage;
		
		traveled = 0;
		
		// get the angle of the projectile
		double diffX = destX - x - width/2.0f;
		double diffY = destY - y - height/2.0f;
		angle =  Math.atan2(diffY, diffX);
		
		image = AssetLoader.getArrowImage();
		
		if(type == ObjectType.ALLYPROJECTILE)
		{
			tag = "Ally Projectile";
		}
		else
		{
			tag = "Enemy Projectile";
		}
	}
	
	/*
	 * @see gameobject.AbstractGameObject#update(float)
	 */
	@Override
	public void update(float delta)
	{
		// move the projectile towards destination
		x += Math.cos(angle) * speed;
		y += Math.sin(angle) * speed;
		
		traveled += speed;
		
		// destroy projectile if destination is reached
		if(Float.compare(traveled, range) > 0)
		{
			markedForDeath = true;
		}
	}
	
	/*
	 * @see gameobject.AbstractGameObject#getImageToDraw()
	 */
	@Override
	public TextureRegion getImageToDraw()
	{
		return image;
	}
	
	/*
	 * @see gameobject.AbstractGameObject#handleCollision(event.CollisionEvent)
	 */
	@Override
	public void handleCollision(CollisionEvent cevent) {
		GameObject collidingObject = cevent.getCollidingObject();
		ObjectType otherType = collidingObject.getObjectType();
		if(objectType == ObjectType.ALLYPROJECTILE && otherType != ObjectType.PRIMARY &&
				otherType != ObjectType.SECONDARY && otherType != ObjectType.TERTIARY &&
				otherType != ObjectType.ALLYPROJECTILE)
		{
			if (otherType != ObjectType.POTION && otherType != ObjectType.SHORT_OBSTACLE)
			{
				markedForDeath = true;
			}
			
		}
		if(objectType == ObjectType.ENEMYPROJECTILE && otherType != ObjectType.ENEMY &&
				otherType != ObjectType.ENEMYPROJECTILE)
		{
			if (otherType != ObjectType.POTION && otherType != ObjectType.SHORT_OBSTACLE)
			{
				markedForDeath = true;
			}
		}
	}
	
	/**
	 * Get the damage caused by the projectile
	 * @return damage
	 */
	public int getDamage()
	{
		return damage;
	}
	
	/**
	 * Get the angle of the projectile
	 * @return the angle in degrees
	 */
	public float getAngle()
	{
		return (float) Math.toDegrees(angle);
	}
}
