/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package gameobject.character;

import java.util.ArrayList;

import manager.AIManager;
import pathfinding.Graph;
import pathfinding.GraphNode;
import pathfinding.PathFinder;
import strategy.Goal;
import strategy.GoalManager;
import strategy.Strategy;
import strategy.mage.MageStrategy;
import strategy.rogue.RogueStrategy;
import strategy.warrior.WarriorStrategy;
import ability.AttackAbility;
import ability.DefendAbility;
import ability.MageAttack;
import ability.MageDefend;
import ability.RogueAttack;
import ability.MageSpecial;
import ability.RogueDefend;
import ability.RogueSpecial;
import ability.SpecialAbility;
import ability.WarriorAttack;
import ability.WarriorDefend;
import ability.WarriorSpecial;

import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Circle;
import com.badlogic.gdx.math.Rectangle;

import event.AIEvent;
import event.AIEventType;
import event.CollisionEvent;
import event.MovementEvent;
import game.GameScreen;
import game.GameUtility;
import game.Player;
import game.QuestMain;
import gameobject.AbstractGameObject;
import gameobject.GameObject;
import gameobject.GateDoor;
import gameobject.ObjectType;

/**
 * Class representing a moving object with behavior
 * Can be either player controlled or ai controlled
 * @author jordan, kevin, eyleen
 * @version Oct 4 2013
 */
public abstract class Character extends AbstractGameObject{

	// character type
	protected CharacterType characterType;

	// character ai
	private ArrayList<Character> charactersInSight;
	private ArrayList<Property> debuffs;

	protected Strategy strategy;
	protected PathFinder pathFinder;
	protected GoalManager goalManager;

	// ability constants

	// warrior
	final private float WARRIOR_ATTACK_RANGE = 0.02f;
	final private int WARRIOR_ATTACK_DAMAGE = 40;
	final private int WARRIOR_ATTACK_ENERGY_USE = 10;
	final private int WARRIOR_ATTACK_COOLDOWN = 30;
	final private float WARRIOR_DEFEND_ENERGY_USE = 0.1f;
	final private float WARRIOR_SPECIAL_RANGE = 1.0f;
	final private int WARRIOR_SPECIAL_ENERGY_USE = 20;
	final private int WARRIOR_SPECIAL_DURATION = 500;
	final private int WARRIOR_SPECIAL_COOLDOWN = 100;
	final private int WARRIOR_MAX_HEALTH = 300;
	final private int WARRIOR_MAX_ENERGY = 100;
	final private float WARRIOR_ENERGY_REGEN = 0.2f;

	// mage
	final private float MAGE_ATTACK_RANGE = 0.3f;
	final private float MAGE_ATTACK_RADIUS = 0.07f;
	final private int MAGE_ATTACK_DAMAGE = 20;
	final private int MAGE_ATTACK_ENERGY_USE = 30;
	final private int MAGE_ATTACK_COOLDOWN = 50;
	final private float MAGE_DEFEND_ENERGY_USE = 0.5f;
	final private float MAGE_SPECIAL_RANGE = 0.2f;
	final private int MAGE_SPECIAL_ENERGY_USE = 50;
	final private int MAGE_SPECIAL_HEALING = 50;
	final private int MAGE_SPECIAL_COOLDOWN = 100;
	final private int MAGE_MAX_HEALTH = 100;
	final private int MAGE_MAX_ENERGY = 200;
	final private float MAGE_ENERGY_REGEN = 0.2f;

	// rogue
	final private float ROGUE_ATTACK_RANGE = 1.0f;
	final private int ROGUE_ATTACK_DAMAGE = 30;
	final private int ROGUE_ATTACK_ENERGY_USE = 10;
	final private int ROGUE_ATTACK_COOLDOWN = 30;
	final private float ROGUE_DEFEND_ENERGY_USE = 0.5f;
	final private float ROGUE_SPECIAL_RANGE = 0.15f;
	final private int ROGUE_SPECIAL_ENERGY_USE = 20;
	final private int ROGUE_SPECIAL_DAMAGE = 100;
	final private int ROGUE_SPECIAL_COOLDOWN = 100;
	final private int ROGUE_MAX_HEALTH = 150;
	final private int ROGUE_MAX_ENERGY = 150;
	final private float ROGUE_ENERGY_REGEN = 0.2f;


	// fields

	protected float speed = 0.005f;

	protected int maxHealth;
	protected int health;
	protected int maxEnergy;
	protected float energy;
	protected float energyRegen;

	private ArrayList<Character> attackers;

	protected AnimationState characterState;
	private float dx =  0;
	private float dy = 0;

	// character states
	protected boolean movingLeft = false;
	protected boolean movingRight = false;
	protected boolean movingUp = false;
	protected boolean movingDown = false;
	protected boolean attackingLeft = false;
	protected boolean attackingRight = false;
	protected boolean attackingUp = false;
	protected boolean attackingDown = false;
	protected boolean defending = false;
	protected boolean defendingRight = false;
	protected boolean defendingLeft = false;
	protected boolean defendingUp = false;
	protected boolean defendingDown = false;
	protected boolean specialRight = false;
	protected boolean specialLeft = false;
	protected boolean specialUp = false;
	protected boolean specialDown = false;
	protected boolean canDefend = true;

	private boolean attacked = false;
	private boolean taunted = false;
	private int maxAttackedTimer = 10;
	private int attackedTimer = 0;

	protected float cursorX;
	protected float cursorY;

	protected boolean dead = false;

	protected boolean attacking = false;
	protected boolean usingSpecial = false;
	private float stateTime = 0.0f;

	private boolean rogueHiding = false;

	// abilities

	protected AttackAbility attackAbility;
	protected DefendAbility defendAbility;
	protected SpecialAbility specialAbility;

	// animations
	private final int NUM_ANIMATIONS = 40;

	// movement animations
	//private Animation idleAnimation;
	private Animation movingRightAnimation;
	private Animation movingLeftAnimation;
	private Animation movingUpAnimation;
	private Animation movingDownAnimation;

	// attacking animations
	private Animation attackingRightAnimation;
	private Animation attackingLeftAnimation;
	private Animation attackingUpAnimation;
	private Animation attackingDownAnimation;

	// defending animations
	protected Animation defendingRightAnimation;
	private Animation defendingLeftAnimation;
	private Animation defendingUpAnimation;
	private Animation defendingDownAnimation;

	// idle animations
	private Animation facingRightAnimation;
	private Animation facingLeftAnimation;
	private Animation facingUpAnimation;
	private Animation facingDownAnimation;

	// dead animations
	private Animation deadRightAnimation;
	private Animation deadLeftAnimation;
	private Animation deadUpAnimation;
	private Animation deadDownAnimation;

	// special animations
	private Animation specialRightAnimation;
	private Animation specialLeftAnimation;
	private Animation specialUpAnimation;
	private Animation specialDownAnimation;

	// potion animations
	private Animation potionRightAnimation;
	private Animation potionLeftAnimation;
	private Animation potionUpAnimation;
	private Animation potionDownAnimation;

	// revive animations
	private Animation reviveRightAnimation;
	private Animation reviveLeftAnimation;
	private Animation reviveUpAnimation;
	private Animation reviveDownAnimation;

	// end defending animations
	protected Animation endDefendingRightAnimation;
	protected Animation endDefendingLeftAnimation;
	protected Animation endDefendingUpAnimation;
	protected Animation endDefendingDownAnimation;

	// pain animations
	protected Animation painRightAnimation;
	protected Animation painLeftAnimation;
	protected Animation painUpAnimation;
	protected Animation painDownAnimation;

	/**
	 * Constructor
	 * @param x
	 * @param y
	 * @param startingState
	 * @param width
	 * @param height
	 * @param objectType
	 * @param characterType
	 * @param animations
	 * @throws Exception
	 */
	public Character(float x, float y, AnimationState startingState, float width, float height,
			ObjectType objectType, CharacterType characterType, ArrayList<Animation> animations) throws Exception
			{
		super(x, y, width, height, objectType);
		characterState = startingState;
		this.characterType = characterType;

		// initialize ai
		charactersInSight = new ArrayList<Character>();
		debuffs = new ArrayList<Property>();
		attackers = new ArrayList<Character>();

		pathFinder = new PathFinder(this);
		goalManager = new GoalManager(this);

		if(animations.size() != NUM_ANIMATIONS)
		{
			System.out.println(animations.size() + "  " + NUM_ANIMATIONS);
			throw new Exception("Wrong number of animations.");
		}
		makeAnimations(animations);

		if(characterType == CharacterType.WARRIOR)
		{
			strategy = new WarriorStrategy(this);
			attackAbility = new WarriorAttack(WARRIOR_ATTACK_RANGE, WARRIOR_ATTACK_DAMAGE,
					WARRIOR_ATTACK_ENERGY_USE, WARRIOR_ATTACK_COOLDOWN, this);
			defendAbility = new WarriorDefend(WARRIOR_DEFEND_ENERGY_USE, this);
			specialAbility = new WarriorSpecial(WARRIOR_SPECIAL_RANGE,
					WARRIOR_SPECIAL_ENERGY_USE, WARRIOR_SPECIAL_DURATION, WARRIOR_SPECIAL_COOLDOWN, this);
			maxHealth = WARRIOR_MAX_HEALTH;
			maxEnergy = WARRIOR_MAX_ENERGY;
			energyRegen = WARRIOR_ENERGY_REGEN;
		}
		else if(characterType == CharacterType.MAGE)
		{
			strategy = new MageStrategy(this);
			attackAbility = new MageAttack(false, MAGE_ATTACK_RANGE, MAGE_ATTACK_RADIUS, MAGE_ATTACK_DAMAGE,
					MAGE_ATTACK_ENERGY_USE, MAGE_ATTACK_COOLDOWN, this);
			defendAbility = new MageDefend(MAGE_DEFEND_ENERGY_USE, this);
			specialAbility = new MageSpecial(false, MAGE_SPECIAL_RANGE,
					MAGE_SPECIAL_ENERGY_USE, MAGE_SPECIAL_HEALING, MAGE_SPECIAL_COOLDOWN, this);
			maxHealth = MAGE_MAX_HEALTH;
			maxEnergy = MAGE_MAX_ENERGY;
			energyRegen = MAGE_ENERGY_REGEN;
		}
		else
		{
			strategy = new RogueStrategy(this);
			attackAbility = new RogueAttack(ROGUE_ATTACK_RANGE, ROGUE_ATTACK_DAMAGE,
					ROGUE_ATTACK_ENERGY_USE, ROGUE_ATTACK_COOLDOWN, this);
			defendAbility = new RogueDefend(ROGUE_DEFEND_ENERGY_USE, this);
			specialAbility = new RogueSpecial(ROGUE_SPECIAL_RANGE,
					ROGUE_SPECIAL_ENERGY_USE, ROGUE_SPECIAL_DAMAGE, ROGUE_SPECIAL_COOLDOWN, this);
			maxHealth = ROGUE_MAX_HEALTH;
			maxEnergy = ROGUE_MAX_ENERGY;
			energyRegen = ROGUE_ENERGY_REGEN;
		}

		health = maxHealth;
		energy = maxEnergy;
			}

	/*
	 * @see gameobject.AbstractGameObject#update(float)
	 */
	@Override
	public void update(float delta)
	{
		checkForInput(delta);

		// update player velocity and state

		// moving straight
		if(movingRight) dx = speed;
		if(movingLeft) dx = -speed;
		if(movingUp) dy = speed;
		if(movingDown) dy = -speed;

		// moving diagonally
		if(movingRight && movingUp)
		{
			dx = (float) (speed*Math.cos(Math.PI/4));
			dy = (float) (speed*Math.sin(Math.PI/4));
		}
		if(movingRight && movingDown)
		{
			dx = (float) (speed*Math.cos(Math.PI/4));
			dy = (float) (-speed*Math.sin(Math.PI/4));
		}
		if(movingLeft && movingUp)
		{
			dx = (float) (-speed*Math.cos(Math.PI/4));
			dy = (float) (speed*Math.sin(Math.PI/4));
		}
		if(movingLeft && movingDown)
		{
			dx = (float) (-speed*Math.cos(Math.PI/4));
			dy = (float) (-speed*Math.sin(Math.PI/4));
		}

		// not moving
		if(!movingRight && !movingLeft) dx = 0.0f;
		if(!movingUp && !movingDown) dy = 0.0f;

		// attacking
		if ((attackingLeft || attackingRight || attackingUp || attackingDown) && !attacking)
		{			
			if(attackAbility.isReady())
			{
				// set attack ability to active and activate it
				attackAbility.activate(cursorX, cursorY);
				attacking = true;
			}
			else
			{
				attackingLeft = false;
				attackingRight = false;
				attackingUp = false;
				attackingDown = false;
			}
		}

		if (specialLeft || specialRight || specialUp || specialDown && !usingSpecial)
		{
			if(specialAbility.isReady())
			{
				specialAbility.activate();
				usingSpecial = true;
			}
			else
			{
				specialLeft = false;
				specialRight = false;
				specialUp = false;
				specialDown = false;
			}
		}

		updateCharacterState();

		// attack animation is over
		if(attackingLeftAnimation.isAnimationFinished(getStateTime()) ||
				attackingRightAnimation.isAnimationFinished(getStateTime()) ||
				attackingUpAnimation.isAnimationFinished(getStateTime()) ||
				attackingDownAnimation.isAnimationFinished(getStateTime()))
		{
			attacking = false;
		}
		// special animation is over
		if(specialLeftAnimation.isAnimationFinished(getStateTime()) ||
				specialRightAnimation.isAnimationFinished(getStateTime()) ||
				specialUpAnimation.isAnimationFinished(getStateTime()) ||
				specialDownAnimation.isAnimationFinished(getStateTime()))
		{
			usingSpecial = false;
		}

		// update position

		GameScreen gameScreen = QuestMain.getCurrentScreen();
		Rectangle screenDimensions = gameScreen.getScreenDimensions();
		// make sure character doesn't walk off screen
		if(x > screenDimensions.getX())
		{
			if((x + width) < (screenDimensions.getX() + screenDimensions.getWidth()))
			{
				x += dx;
			}
			else
			{
				x = screenDimensions.getX() + screenDimensions.getWidth() - width - 0.0001f;
			}
		}
		else
		{
			x = screenDimensions.getX() + 0.0001f;
		}
		if(y > screenDimensions.getY())
		{
			if((y + height) < (screenDimensions.getY() + screenDimensions.getHeight()))
			{
				y += dy;
			}
			else
			{
				y = screenDimensions.getY() + screenDimensions.getHeight() - height - 0.0001f;
			}
		}
		else
		{
			y = screenDimensions.getY() + 0.0001f;
		}

		setStateTime(getStateTime() + delta);

		if(energy >= defendAbility.getEnergyUse())
		{
			canDefend = true;
		}
		else
		{
			canDefend = false;
		}

		// code for having enemies avoid the rogue
		if (rogueHiding)
		{
		}

		//code for defending / rogue hiding
		if(defending && canDefend)
		{
			energy -= defendAbility.getEnergyUse();
		}
		else if(!defending)
		{
			regenEnergy();
		}

		// check to see if character has died
		if (health <= 0 && !dead)
		{
			health = 0;
			if(objectType == ObjectType.ENEMY)
			{
				//System.out.println("Killed enemy!");
				markedForDeath = true;
			}
			else
			{
				dead = true;
				if(objectType == ObjectType.PRIMARY)
				{
					Player.getInstance().switchCharacter();
				}
			}
		}

		// attacked timer
		if(attackedTimer > 0)
		{
			attackedTimer--;
		}
		if(attacked && attackedTimer <= 0)
		{
			attackedTimer = 0;
			attacked = false;
		}

		attackAbility.update(delta);
		specialAbility.update(delta);

	}

	/**
	 * Regenerate the character's energy
	 */
	private void regenEnergy() {
		// increment energy if not at max
		if (energy < maxEnergy)
		{
			energy += energyRegen;
		}
	}

	/**
	 * Gets input from the keyboard and updates the appropriate booleans. Overridden by subclass.
	 */
	protected void checkForInput(float delta)
	{

	}

	/**
	 * Updates the state of the player for deciding animations
	 */
	private void updateCharacterState()
	{
		// if the player just got revived
		if (characterState == AnimationState.REVIVE_RIGHT || 
				characterState == AnimationState.REVIVE_LEFT || 
				characterState == AnimationState.REVIVE_UP || 
				characterState == AnimationState.REVIVE_DOWN)
		{
			if(reviveLeftAnimation.isAnimationFinished(getStateTime()) ||
					reviveRightAnimation.isAnimationFinished(getStateTime()) ||
					reviveUpAnimation.isAnimationFinished(getStateTime()) ||
					reviveDownAnimation.isAnimationFinished(getStateTime()))
			{
				switch(characterState)
				{
				case REVIVE_UP:
					setCharacterState(AnimationState.FACING_UP);
					break;
				case REVIVE_DOWN:
					setCharacterState(AnimationState.FACING_DOWN);
					break;
				case REVIVE_RIGHT:
					setCharacterState(AnimationState.FACING_RIGHT);
					break;
				case REVIVE_LEFT:
					setCharacterState(AnimationState.FACING_LEFT);
					break;
				default:
					break;
				}
			}
		}

		// end drinking potions
		if (characterState == AnimationState.POTION_RIGHT || 
				characterState == AnimationState.POTION_LEFT || 
				characterState == AnimationState.POTION_UP || 
				characterState == AnimationState.POTION_DOWN)
		{
			if(potionLeftAnimation.isAnimationFinished(getStateTime()) ||
					potionRightAnimation.isAnimationFinished(getStateTime()) ||
					potionUpAnimation.isAnimationFinished(getStateTime()) ||
					potionDownAnimation.isAnimationFinished(getStateTime()))
			{
				switch(characterState)
				{
				case POTION_UP:
					setCharacterState(AnimationState.FACING_UP);
					break;
				case POTION_DOWN:
					setCharacterState(AnimationState.FACING_DOWN);
					break;
				case POTION_RIGHT:
					setCharacterState(AnimationState.FACING_RIGHT);
					break;
				case POTION_LEFT:
					setCharacterState(AnimationState.FACING_LEFT);
					break;
				default:
					break;
				}
			}
		}


		// if the player is moving, update state
		if(dx != 0.0f || dy != 0.0f)
		{	
			// if the character is ai controlled and has a target, calculate direction
			boolean movement = true;
			if(objectType == ObjectType.SECONDARY || objectType == ObjectType.TERTIARY || objectType == ObjectType.ENEMY)
			{
				Goal goal = goalManager.getCurrentGoal();
				GameObject target = goal.getTarget();

				if(target != null)
				{
					Direction moveDirection = GameUtility.getDirection(this, target.getX(), target.getY());
					if(moveDirection == Direction.RIGHT) setCharacterState(AnimationState.MOVING_RIGHT);
					else if(moveDirection == Direction.LEFT) setCharacterState(AnimationState.MOVING_LEFT);
					else if(moveDirection == Direction.UP) setCharacterState(AnimationState.MOVING_UP);
					else if(moveDirection == Direction.DOWN) setCharacterState(AnimationState.MOVING_DOWN);
					movement = false;
				}
				else
				{
					Graph graph = QuestMain.getCurrentScreen().getGraph();
					GraphNode destination = graph.findSafePlace(this);
					Direction moveDirection = GameUtility.getDirection(this, destination.getX(), destination.getY());
					if(moveDirection == Direction.RIGHT) setCharacterState(AnimationState.MOVING_RIGHT);
					else if(moveDirection == Direction.LEFT) setCharacterState(AnimationState.MOVING_LEFT);
					else if(moveDirection == Direction.UP) setCharacterState(AnimationState.MOVING_UP);
					else if(moveDirection == Direction.DOWN) setCharacterState(AnimationState.MOVING_DOWN);
					movement = false;
				}
			}
			// otherwise base direction off of movement
			if(movement)
			{
				if(dy > 0.0f && dx == 0.0f) setCharacterState(AnimationState.MOVING_UP);
				else if(dy < 0.0f && dx == 0.0f) setCharacterState(AnimationState.MOVING_DOWN);
				else if(dx > 0.0f) setCharacterState(AnimationState.MOVING_RIGHT);
				else if (dx < 0.0f) setCharacterState(AnimationState.MOVING_LEFT);
			}
		}
		// else leave player facing in right direction
		else
		{
			switch(characterState)
			{
			case MOVING_UP:
				setCharacterState(AnimationState.FACING_UP);
				break;
			case MOVING_DOWN:
				setCharacterState(AnimationState.FACING_DOWN);
				break;
			case MOVING_RIGHT:
				setCharacterState(AnimationState.FACING_RIGHT);
				break;
			case MOVING_LEFT:
				setCharacterState(AnimationState.FACING_LEFT);
				break;
			default:
				break;
			}
		}

		// set attacking state
		if (attackingLeft)
		{
			setCharacterState(AnimationState.ATTACKING_LEFT);
		}
		else if(attackingRight)
		{
			setCharacterState(AnimationState.ATTACKING_RIGHT);
		}
		else if (attackingUp)
		{
			setCharacterState(AnimationState.ATTACKING_UP);
		}
		else if(attackingDown)
		{
			setCharacterState(AnimationState.ATTACKING_DOWN);
		}


		// set special state
		if (specialLeft)
		{
			if (characterType == CharacterType.MAGE  && characterState != AnimationState.SPECIAL_LEFT)
			{
				((MageSpecial) specialAbility).activate();
			}
			setCharacterState(AnimationState.SPECIAL_LEFT);
		}
		else if(specialRight)
		{
			if (characterType == CharacterType.MAGE  && characterState != AnimationState.SPECIAL_RIGHT)
			{
				((MageSpecial) specialAbility).activate();
			}
			setCharacterState(AnimationState.SPECIAL_RIGHT);
		}
		else if (specialUp)
		{
			if (characterType == CharacterType.MAGE  && characterState != AnimationState.SPECIAL_UP)
			{
				((MageSpecial) specialAbility).activate();
			}
			setCharacterState(AnimationState.SPECIAL_UP);
		}
		else if(specialDown)
		{
			if (characterType == CharacterType.MAGE  && characterState != AnimationState.SPECIAL_DOWN)
			{
				((MageSpecial) specialAbility).activate();
			}
			setCharacterState(AnimationState.SPECIAL_DOWN);
		}

		if(defending && canDefend)
		{
			switch(characterState)
			{
			case MOVING_RIGHT:
			case FACING_RIGHT:
			case DEFENDING_RIGHT:
				setCharacterState(AnimationState.DEFENDING_RIGHT);
				defendingRight = true;
				break;
			case MOVING_LEFT:
			case FACING_LEFT:
			case DEFENDING_LEFT:
				setCharacterState(AnimationState.DEFENDING_LEFT);
				defendingLeft = true;
				break;
			case MOVING_UP:
			case FACING_UP:
			case DEFENDING_UP:
				setCharacterState(AnimationState.DEFENDING_UP);
				defendingUp = true;
				break;
			case MOVING_DOWN:
			case FACING_DOWN:
			case DEFENDING_DOWN:
				setCharacterState(AnimationState.DEFENDING_DOWN);
				defendingDown = true;
				break;
			default:
				break;
			}
		}

		//switch to end defending
		if (!defending &&
				(characterState == AnimationState.DEFENDING_RIGHT || 
				characterState == AnimationState.DEFENDING_LEFT || 
				characterState == AnimationState.DEFENDING_UP || 
				characterState == AnimationState.DEFENDING_DOWN))
		{
			if(defendingLeftAnimation.isAnimationFinished(getStateTime()) ||
					defendingRightAnimation.isAnimationFinished(getStateTime()) ||
					defendingUpAnimation.isAnimationFinished(getStateTime()) ||
					defendingDownAnimation.isAnimationFinished(getStateTime()))
			{
				switch(characterState)
				{
				case DEFENDING_RIGHT:
					setCharacterState(AnimationState.END_DEFEND_RIGHT);
					defendingRight = true;
					break;
				case DEFENDING_LEFT:
					setCharacterState(AnimationState.END_DEFEND_LEFT);
					defendingLeft = true;
					break;
				case DEFENDING_UP:
					setCharacterState(AnimationState.END_DEFEND_UP);
					defendingUp = true;
					break;
				case DEFENDING_DOWN:
					setCharacterState(AnimationState.END_DEFEND_DOWN);
					defendingDown = true;
					break;
				default:
					break;
				}
			}
		}

		// switch to normal
		if (characterState == AnimationState.END_DEFEND_RIGHT || 
				characterState == AnimationState.END_DEFEND_LEFT || 
				characterState == AnimationState.END_DEFEND_UP || 
				characterState == AnimationState.END_DEFEND_DOWN)
		{
			if(endDefendingRightAnimation.isAnimationFinished(getStateTime()) ||
					endDefendingLeftAnimation.isAnimationFinished(getStateTime()) ||
					endDefendingUpAnimation.isAnimationFinished(getStateTime()) ||
					endDefendingDownAnimation.isAnimationFinished(getStateTime()))
			{
				switch(characterState)
				{
				case END_DEFEND_RIGHT:
					setCharacterState(AnimationState.FACING_RIGHT);
					defendingRight = false;
					break;
				case END_DEFEND_LEFT:
					setCharacterState(AnimationState.FACING_LEFT);
					defendingLeft = false;
					break;
				case END_DEFEND_UP:
					setCharacterState(AnimationState.FACING_UP);
					defendingUp = false;
					break;
				case END_DEFEND_DOWN:
					setCharacterState(AnimationState.FACING_DOWN);
					defendingDown = false;
					break;
				default:
					break;
				}
			}
		}

		// if animations are over
		if (characterState == AnimationState.ATTACKING_LEFT && !attacking)
		{
			setCharacterState(AnimationState.FACING_LEFT);
		}
		else if (characterState == AnimationState.ATTACKING_RIGHT && !attacking)
		{
			setCharacterState(AnimationState.FACING_RIGHT);
		}
		else if (characterState == AnimationState.ATTACKING_UP && !attacking)
		{
			setCharacterState(AnimationState.FACING_UP);
		}
		else if (characterState == AnimationState.ATTACKING_DOWN && !attacking)
		{
			setCharacterState(AnimationState.FACING_DOWN);
		}

		if (characterState == AnimationState.SPECIAL_LEFT && !usingSpecial)
		{
			setCharacterState(AnimationState.FACING_LEFT);
		}
		else if (characterState == AnimationState.SPECIAL_RIGHT && !usingSpecial)
		{
			setCharacterState(AnimationState.FACING_RIGHT);
		}
		else if (characterState == AnimationState.SPECIAL_UP && !usingSpecial)
		{
			setCharacterState(AnimationState.FACING_UP);
		}
		else if (characterState == AnimationState.SPECIAL_DOWN && !usingSpecial)
		{
			setCharacterState(AnimationState.FACING_DOWN);
		}

		// dead
		if(dead)
		{
			switch(characterState)
			{
			case MOVING_RIGHT:
			case FACING_RIGHT:
			case ATTACKING_RIGHT:
			case SPECIAL_RIGHT:
				setCharacterState(AnimationState.DEAD_RIGHT);
				break;
			case MOVING_LEFT:
			case FACING_LEFT:
			case ATTACKING_LEFT:
			case SPECIAL_LEFT:
				setCharacterState(AnimationState.DEAD_LEFT);
				break;
			case MOVING_UP:
			case FACING_UP:
			case ATTACKING_UP:
			case SPECIAL_UP:
				setCharacterState(AnimationState.DEAD_UP);
				break;
			case MOVING_DOWN:
			case FACING_DOWN:
			case ATTACKING_DOWN:
			case SPECIAL_DOWN:
				setCharacterState(AnimationState.DEAD_DOWN);
				break;
			default:
				break;
			}
		}

	}

	/**
	 * Sets the state of the player to the given state
	 * @param newState the state to be switched to
	 */
	public void setCharacterState(AnimationState newState)
	{
		if(characterState != newState)
		{
			characterState = newState;
			setStateTime(0.0f);
		}
	}

	/**
	 * Makes the the animations
	 */
	private void makeAnimations(ArrayList<Animation>animations)
	{
		// should match the order of animations in GameScreen.makePartyMember
		// also make sure to update NUM_ANIMATIONS
		movingRightAnimation = animations.get(0);
		facingRightAnimation = animations.get(1);
		movingLeftAnimation = animations.get(2);
		facingLeftAnimation = animations.get(3);
		movingUpAnimation = animations.get(4);
		facingUpAnimation = animations.get(5);
		movingDownAnimation = animations.get(6);
		facingDownAnimation = animations.get(7);
		attackingRightAnimation = animations.get(8);
		attackingLeftAnimation = animations.get(9);
		attackingUpAnimation = animations.get(10);
		attackingDownAnimation = animations.get(11);
		defendingRightAnimation = animations.get(12);
		defendingLeftAnimation = animations.get(13);
		defendingUpAnimation = animations.get(14);
		defendingDownAnimation = animations.get(15);
		deadRightAnimation = animations.get(16);
		deadLeftAnimation = animations.get(17);
		deadUpAnimation = animations.get(18);
		deadDownAnimation = animations.get(19);
		specialRightAnimation = animations.get(20);
		specialLeftAnimation = animations.get(21);
		specialUpAnimation = animations.get(22);
		specialDownAnimation = animations.get(23);
		potionRightAnimation = animations.get(24);
		potionLeftAnimation = animations.get(25);
		potionUpAnimation = animations.get(26);
		potionDownAnimation = animations.get(27);
		reviveRightAnimation = animations.get(28);
		reviveLeftAnimation = animations.get(29);
		reviveUpAnimation = animations.get(30);
		reviveDownAnimation = animations.get(31);
		endDefendingRightAnimation = animations.get(32);
		endDefendingLeftAnimation = animations.get(33);
		endDefendingUpAnimation = animations.get(34);
		endDefendingDownAnimation = animations.get(35);	
		painRightAnimation = animations.get(36);
		painLeftAnimation = animations.get(37);
		painUpAnimation = animations.get(38);
		painDownAnimation = animations.get(39);
	}

	/*
	 * @see gameobject.AbstractGameObject#getImageToDraw()
	 */
	@Override
	public TextureRegion getImageToDraw()
	{
		TextureRegion image = facingRightAnimation.getKeyFrame(getStateTime(), true);

		switch(characterState)
		{
		case MOVING_RIGHT:
			image = movingRightAnimation.getKeyFrame(getStateTime(), true);
			break;
		case FACING_RIGHT:
			image = facingRightAnimation.getKeyFrame(getStateTime(), true);
			break;
		case MOVING_LEFT:
			image = movingLeftAnimation.getKeyFrame(getStateTime(), true);
			break;
		case FACING_LEFT:
			image = facingLeftAnimation.getKeyFrame(getStateTime(), true);
			break;
		case MOVING_UP:
			image = movingUpAnimation.getKeyFrame(getStateTime(), true);
			break;
		case FACING_UP:
			image = facingUpAnimation.getKeyFrame(getStateTime(), true);
			break;
		case MOVING_DOWN:
			image = movingDownAnimation.getKeyFrame(getStateTime(), true);
			break;
		case FACING_DOWN:
			image = facingDownAnimation.getKeyFrame(getStateTime(), true);
			break;
		case ATTACKING_LEFT:
			image = attackingLeftAnimation.getKeyFrame(getStateTime(), false);
			break;
		case ATTACKING_RIGHT:
			image = attackingRightAnimation.getKeyFrame(getStateTime(), false);
			break;
		case ATTACKING_UP:
			image = attackingUpAnimation.getKeyFrame(getStateTime(), false);
			break;
		case ATTACKING_DOWN:
			image = attackingDownAnimation.getKeyFrame(getStateTime(), false);
			break;
		case DEFENDING_RIGHT:
			image = defendingRightAnimation.getKeyFrame(getStateTime(), false);
			break;
		case DEFENDING_LEFT:
			image = defendingLeftAnimation.getKeyFrame(getStateTime(), false);
			break;
		case DEFENDING_UP:
			image = defendingUpAnimation.getKeyFrame(getStateTime(), false);
			break;
		case DEFENDING_DOWN:
			image = defendingDownAnimation.getKeyFrame(getStateTime(), false);
			break;
		case DEAD_RIGHT:
			image = deadRightAnimation.getKeyFrame(getStateTime(), false);
			break;
		case DEAD_LEFT:
			image = deadLeftAnimation.getKeyFrame(getStateTime(), false);
			break;
		case DEAD_UP:
			image = deadUpAnimation.getKeyFrame(getStateTime(), false);
			break;
		case DEAD_DOWN:
			image = deadDownAnimation.getKeyFrame(getStateTime(), false);
			break;
		case SPECIAL_RIGHT:
			image = specialRightAnimation.getKeyFrame(getStateTime(), false);
			break;
		case SPECIAL_LEFT:
			image = specialLeftAnimation.getKeyFrame(getStateTime(), false);
			break;
		case SPECIAL_UP:
			image = specialUpAnimation.getKeyFrame(getStateTime(), false);
			break;
		case SPECIAL_DOWN:
			image = specialDownAnimation.getKeyFrame(getStateTime(), false);
			break;
		case POTION_RIGHT:
			image = potionRightAnimation.getKeyFrame(getStateTime(), false);
			break;
		case POTION_LEFT:
			image = potionLeftAnimation.getKeyFrame(getStateTime(), false);
			break;
		case POTION_UP:
			image = potionUpAnimation.getKeyFrame(getStateTime(), false);
			break;
		case POTION_DOWN:
			image = potionDownAnimation.getKeyFrame(getStateTime(), false);
			break;
		case REVIVE_RIGHT:
			image = reviveRightAnimation.getKeyFrame(getStateTime(), false);
			break;
		case REVIVE_LEFT:
			image = reviveLeftAnimation.getKeyFrame(getStateTime(), false);
			break;
		case REVIVE_UP:
			image = reviveUpAnimation.getKeyFrame(getStateTime(), false);
			break;
		case REVIVE_DOWN:
			image = reviveDownAnimation.getKeyFrame(getStateTime(), false);
			break;
		case END_DEFEND_RIGHT:
			image = endDefendingRightAnimation.getKeyFrame(getStateTime(), false);
			break;
		case END_DEFEND_LEFT:
			image = endDefendingLeftAnimation.getKeyFrame(getStateTime(), false);
			break;
		case END_DEFEND_UP:
			image = endDefendingUpAnimation.getKeyFrame(getStateTime(), false);
			break;
		case END_DEFEND_DOWN:
			image = endDefendingDownAnimation.getKeyFrame(getStateTime(), false);
			break;
		}

		return image;
	}

	/*
	 * @see gameobject.AbstractGameObject#handleCollision(event.CollisionEvent)
	 */
	@Override
	public void handleCollision(CollisionEvent cevent) {
		GameObject otherObject = cevent.getCollidingObject();
		ObjectType type = otherObject.getObjectType();

		if(type != ObjectType.TRANSITION && type != ObjectType.ALLYPROJECTILE &&
				type != ObjectType.ENEMYPROJECTILE && type != ObjectType.POTION)
		{
			if (type == ObjectType.GATE)
			{
				GateDoor gate = (GateDoor)otherObject;
				
				if(!gate.isOpen())
				{
					collideWithObstacle(otherObject);
				}	
			}
			else
			{
				collideWithObstacle(otherObject);
			}
		}
		
		handleSpecialCollision(cevent);
	}

	/**
	 * Handle special collision. Overridden by subclass.
	 * @param cevent the collision event
	 */
	protected void handleSpecialCollision(CollisionEvent cevent)
	{

	}

	/**
	 * Handle a collision with an object of type obstacle
	 * @param otherObject the colliding object
	 */
	protected void collideWithObstacle(GameObject otherObject)
	{
		// center of player
		float centerX = x + width/2.0f;
		float centerY = y + height/2.0f;
		// center of other object
		float otherCenterX = otherObject.getX() + otherObject.getWidth()/2.0f;
		float otherCenterY = otherObject.getY() + otherObject.getHeight()/2.0f;
		// difference
		float diffX = otherCenterX - centerX;
		float diffY = otherCenterY - centerY;
		float ratio = diffX/diffY;

		// colliding into other object's left side
		if((ratio > 1.0f || ratio < -1.0f) && diffX > 0.0f)
		{
			x = otherObject.getX() - width - 0.001f;
			movingRight = false;
		}
		// colliding into other object's right side
		else if((ratio > 1.0f || ratio < -1.0f) && diffX <= 0.0f)
		{
			x = otherObject.getX() + otherObject.getWidth() + 0.001f;
			movingLeft = false;
		}
		// colliding into other object's bottom
		else if((ratio <= 1.0f && ratio >= -1.0f) && diffY > 0.0f)
		{
			y = otherObject.getY() - height - 0.001f;
			movingUp = false;
		}
		// colliding into other object's top
		else if((ratio <= 1.0f && ratio >= -1.0f) && diffY <= 0.0f)
		{
			y = otherObject.getY() + otherObject.getHeight() + 0.001f;
			movingDown = false;
		}
	}

	/**
	 * Get the line of sight of a character
	 * @return
	 */
	public Circle getLineOfSight()
	{
		return new Circle(x, y, 0.5f);
	}

	/**
	 * Get the characters within this character's line of sight
	 * @return list of characters
	 */
	public ArrayList<Character> getCharactersInSight()
	{
		return charactersInSight;
	}

	/**
	 * This character sees another character
	 * @param character the character that has been seen
	 */
	public void characterSeen(Character character)
	{
		// if the character isn't already in view, and it isn't this object
		if(!charactersInSight.contains(character) && !character.equals(this))
		{
			// add the character to the list of seen characters
			charactersInSight.add(character);
		}
	}

	/**
	 * This character can no longer see another character
	 * @param character the character no longer in view
	 */
	public void characterNoLongerSeen(Character character)
	{
		// if the character was in view, and it isn't this object
		if(charactersInSight.contains(character) && !character.equals(this))
		{
			// remove the character from the list of seen characters
			charactersInSight.remove(character);
		}
	}

	/**
	 * Check whether the character is attacking
	 * @return attacking
	 */
	public boolean isAttacking() {
		return attacking;
	}

	/**
	 * Get the maximum health
	 * @return maxHealth
	 */
	public int getMaxHealth() {
		// returns Max Health a player can have
		return maxHealth;
	}

	/**
	 * Get the maximum energy
	 * @return maxEnergy
	 */
	public int getMaxEnergy() {
		// returns max energy that the player can have
		return maxEnergy;
	}

	/**
	 * Get the CharacterType of this character
	 * @return characterType
	 */
	public CharacterType getCharacterType()
	{
		return characterType;
	}

	/**
	 * Get the state of the animation
	 * @return characterState
	 */
	public AnimationState getCharacterState()
	{
		return characterState;
	}

	/**
	 * Get the health of this character
	 * @return health
	 */
	public int getHealth()
	{
		return health;
	}

	/**
	 * Return the current energy
	 * @return energy
	 */
	public float getEnergy(){
		return energy;
	}

	/**
	 * change the current energy
	 * @return energy
	 */
	public void useEnergy(float energyUse){

		// take away damage from character
		energy -= energyUse;
		if (energy < 0)
		{
			energy = 0;
		}
	}

	/**
	 * Check whether the character is stunned
	 * @return true if the character is stunned, false otherwise
	 */
	public boolean isStunned()
	{
		// false for now, stunned isn't implemented yet
		return false;
	}

	/**
	 * Set whether the character is defening or not
	 * @param defend true if defending, false otherwise
	 */
	public void setDefending(boolean defend) {

		defending = defend;
	}

	/**
	 * Check whether the character is defending or not
	 * @return true if defending, false otherwise
	 */
	public boolean getDefending() {
		return defending;
	}

	/**
	 * @return character's attack ability
	 */
	public AttackAbility getAttackAbility(){
		return attackAbility;
	}

	/**
	 * @return character's defend ability
	 */
	public DefendAbility getDefendAbility(){
		return defendAbility;
	}

	/**
	 * @return character's special ability
	 */
	public SpecialAbility getSpecialAbility(){
		return specialAbility;
	}

	/**
	 * Add a debuff
	 * @param p the debuff to be added
	 */
	public void addDebuff(Property p){
		debuffs.add(p);
	}

	/**
	 * Remove a debuff
	 * @param p the debuff to be removed
	 */
	public void removeDebuff(Property p){
		debuffs.remove(p);
	}

	/**
	 * Checks if the character is berserk
	 * @return
	 */
	public boolean isBerserk()
	{
		return debuffs.contains(Property.BERSERK);
	}

	/**
	 * Take damage
	 * @param damage the damage to be taken
	 */
	public void takeDamage(int damage) {
		// take away damage from character
		health -= damage;
		if (health < 0)
		{
			health = 0;
		}
		//System.out.println(getObjectType() + " took " + damage + " damage and is at " + health + " health.");
	}

	/**
	 * Check if the character is dead
	 * @return true if dead, false otherwise
	 */
	public boolean isDead()
	{
		return dead;
	}

	/**
	 * Get the direction the character is facing
	 * @return the direction of the character
	 */
	public Direction getDirection()
	{
		Direction direction;
		switch(characterState)
		{
		case FACING_RIGHT:
		case MOVING_RIGHT:
		case ATTACKING_RIGHT:
		case DEFENDING_RIGHT:
		case SPECIAL_RIGHT:
		case DEAD_RIGHT:
		case REVIVE_RIGHT:
			direction = Direction.RIGHT;
			break;
		case FACING_LEFT:
		case MOVING_LEFT:
		case ATTACKING_LEFT:
		case DEFENDING_LEFT:
		case SPECIAL_LEFT:
		case DEAD_LEFT:
		case REVIVE_LEFT:
			direction = Direction.LEFT;
			break;
		case FACING_UP:
		case MOVING_UP:
		case ATTACKING_UP:
		case DEFENDING_UP:
		case SPECIAL_UP:
		case DEAD_UP:
		case REVIVE_UP:
			direction = Direction.UP;
			break;
		case FACING_DOWN:
		case MOVING_DOWN:
		case ATTACKING_DOWN:
		case DEFENDING_DOWN:
		case SPECIAL_DOWN:
		case DEAD_DOWN:
		case REVIVE_DOWN:
			direction = Direction.DOWN;
			break;
		default:
			direction = Direction.RIGHT;
			break;
		}
		return direction;
	}

	/**
	 * Heal the character
	 * @param amountHealed the amount of health added
	 */
	public void heal(int amountHealed) {

		if (health + amountHealed > maxHealth)
		{
			health = maxHealth;
		}
		else
		{
			health = health + amountHealed;
		}
	}

	/**
	 * Set the rogue to hide
	 * @param b true if hiding, false otherwise
	 */
	public void setRogueHiding(boolean b) {
		// started hiding
		if(b && !rogueHiding)
		{
			// send the event to the ai
			AIManager aiManager = AIManager.getInstance();
			AIEvent aievent = new AIEvent(this, AIEventType.HIDE);
			aiManager.handleEvent(aievent);
		}
		// stopped hiding
		else if(!b && rogueHiding)
		{
			// send the event to the ai
			AIManager aiManager = AIManager.getInstance();
			AIEvent aievent = new AIEvent(this, AIEventType.END_HIDE);
			aiManager.handleEvent(aievent);
		}
		rogueHiding = b;
	}

	/**
	 * Check whether the rogue is hiding
	 * @return true if hiding, false otherwise
	 */
	public boolean getRogueHiding() {
		return rogueHiding;
	}

	/**
	 * The character is attacked
	 * @param damage the damage from the attack
	 * @param attacker the source of the attack
	 */
	public void attacked(int damage, GameObject attacker)
	{
		if(!dead)
		{
			float diffX = attacker.getX() - x;
			float diffY = attacker.getY() - y;

			float absDiffX = Math.abs(diffX);
			float absDiffY = Math.abs(diffY);

			boolean fromLeft = false;
			boolean fromRight = false;
			boolean fromAbove = false;
			boolean fromBelow = false;

			// figure out what direction the attack came from
			if(absDiffX >= absDiffY)
			{
				if(diffX >= 0)
				{
					fromRight = true;
				}
				else
				{
					fromLeft = true;
				}
			}
			else
			{
				if(diffY >= 0)
				{
					fromAbove = true;
				}
				else
				{
					fromBelow = true;
				}
			}

			// check if the character successfully defended
			if(canDefend && ((defendingRight && fromRight) || (defendingLeft && fromLeft) ||
					(defendingUp && fromAbove) || (defendingDown && fromBelow)))
			{
				return;
			}

			// otherwise take damage
			takeDamage(damage);
			attacked = true;
			attackedTimer = maxAttackedTimer;
		}
	}

	/**
	 * Get the current animation
	 * @return the current animation
	 */
	public Animation getAnimation()
	{
		switch(characterState)
		{
		case MOVING_RIGHT:
			return movingRightAnimation;
		case FACING_RIGHT:
			return facingRightAnimation;
		case MOVING_LEFT:
			return movingLeftAnimation;
		case FACING_LEFT:
			return facingLeftAnimation;
		case MOVING_UP:
			return movingUpAnimation;
		case FACING_UP:
			return facingUpAnimation;
		case MOVING_DOWN:
			return movingDownAnimation;
		case FACING_DOWN:
			return facingDownAnimation;
		case ATTACKING_LEFT:
			return attackingLeftAnimation;
		case ATTACKING_RIGHT:
			return attackingRightAnimation;
		case ATTACKING_UP:
			return attackingUpAnimation;
		case ATTACKING_DOWN:
			return attackingDownAnimation;
		case DEFENDING_RIGHT:
			return defendingRightAnimation;
		case DEFENDING_LEFT:
			return defendingLeftAnimation;
		case DEFENDING_UP:
			return defendingUpAnimation;
		case DEFENDING_DOWN:
			return defendingDownAnimation;
		case DEAD_RIGHT:
			return deadRightAnimation;
		case DEAD_LEFT:
			return deadLeftAnimation;
		case DEAD_UP:
			return deadUpAnimation;
		case DEAD_DOWN:
			return deadDownAnimation;
		case SPECIAL_RIGHT:
			return specialRightAnimation;
		case SPECIAL_LEFT:
			return specialLeftAnimation;
		case SPECIAL_UP:
			return specialUpAnimation;
		case SPECIAL_DOWN:
			return specialDownAnimation;
		case POTION_RIGHT:
			return potionRightAnimation;
		case POTION_LEFT:
			return potionLeftAnimation;
		case POTION_UP:
			return potionUpAnimation;
		case POTION_DOWN:
			return potionDownAnimation;
		case REVIVE_RIGHT:
			return reviveRightAnimation;
		case REVIVE_LEFT:
			return reviveLeftAnimation;
		case REVIVE_UP:
			return reviveUpAnimation;
		case REVIVE_DOWN:
			return reviveDownAnimation;
		default:
			return movingRightAnimation;
		}
	}

	/**
	 * Get the state time
	 * @return stateTime
	 */
	public float getStateTime() {
		return stateTime;
	}

	/**
	 * Set the state time
	 * @param stateTime the new state time
	 */
	public void setStateTime(float stateTime) {
		this.stateTime = stateTime;
	}

	public void setMovingLeft(boolean movingLeft) {
		this.movingLeft = movingLeft;
	}

	public boolean isMovingLeft()
	{
		return movingLeft;
	}

	public void setMovingRight(boolean movingRight) {
		this.movingRight = movingRight;
	}

	public boolean isMovingRight()
	{
		return movingRight;
	}

	public void setMovingUp(boolean movingUp) {
		this.movingUp = movingUp;
	}

	public boolean isMovingUp()
	{
		return movingUp;
	}

	public void setMovingDown(boolean movingDown) {
		this.movingDown = movingDown;
	}

	public boolean isMovingDown()
	{
		return movingDown;
	}

	public void setAttackingLeft(boolean attackingLeft) {
		this.attackingLeft = attackingLeft;
	}

	public void setAttackingRight(boolean attackingRight) {
		this.attackingRight = attackingRight;
	}

	public void setAttackingUp(boolean attackingUp) {
		this.attackingUp = attackingUp;
	}

	public void setAttackingDown(boolean attackingDown) {
		this.attackingDown = attackingDown;
	}

	public void setDefendingRight(boolean defendingRight) {
		this.defendingRight = defendingRight;
	}

	public void setDefendingLeft(boolean defendingLeft) {
		this.defendingLeft = defendingLeft;
	}

	public void setDefendingUp(boolean defendingUp) {
		this.defendingUp = defendingUp;
	}

	public void setDefendingDown(boolean defendingDown) {
		this.defendingDown = defendingDown;
	}

	public void setSpecialRight(boolean specialRight) {
		this.specialRight = specialRight;
	}

	public void setSpecialLeft(boolean specialLeft) {
		this.specialLeft = specialLeft;
	}

	public void setSpecialUp(boolean specialUp) {
		this.specialUp = specialUp;
	}

	public void setSpecialDown(boolean specialDown) {
		this.specialDown = specialDown;
	}

	/**
	 * Get the pathfinder for this character
	 * @return pathFinder
	 */
	public PathFinder getPathfinder()
	{
		return pathFinder;
	}

	/**
	 * Get the goal manager for this character
	 * @return goalManager
	 */
	public GoalManager getGoalManager()
	{
		return goalManager;
	}

	/**
	 * Refresh the ai
	 */
	public void refreshAI()
	{
		pathFinder = new PathFinder(this);
		goalManager = new GoalManager(this);
	}

	/**
	 * Get the attack range of the character.
	 * @return the attack range as a Circle
	 */
	public Circle getAttackRange()
	{
		return new Circle(getCenterX(), getCenterY(), attackAbility.getRange());
	}

	/**
	 * Get the special range of the character.
	 * @return the special range as a Circle
	 */
	public Circle getSpecialRange()
	{
		if (characterType == CharacterType.WARRIOR)
		{
			return new Circle(getCenterX(), getCenterY(), WARRIOR_SPECIAL_RANGE);
		}
		else if(characterType == CharacterType.MAGE)
		{
			return new Circle(getCenterX(), getCenterY(), MAGE_SPECIAL_RANGE);
		}
		else
		{
			return new Circle(getCenterX(), getCenterY(), ROGUE_SPECIAL_RANGE);
		}

	}

	/**
	 * Change the value of where the character is attacking
	 */
	public void setCursorX(float targetX)
	{
		cursorX = targetX;
	}
	/**
	 * Change the value of where the character is attacking
	 */
	public void setCursorY(float targetY)
	{
		cursorY = targetY;
	}

	/**
	 * Gets the mage's attack radius 
	 * @return the float radius of the mage's attack
	 */
	public float getAttackRadius() {
		return MAGE_ATTACK_RADIUS;
	}

	/**
	 * Reset all the booleans
	 */
	public void resetBooleans()
	{
		movingLeft = false;
		movingRight = false;
		movingUp = false;
		movingDown = false;
		attackingLeft = false;
		attackingRight = false;
		attackingUp = false;
		attackingDown = false;
		defending = false;
		defendingRight = false;
		defendingLeft = false;
		defendingUp = false;
		defendingDown = false;
		specialRight = false;
		specialLeft = false;
		specialUp = false;
		specialDown = false;
	}

	public float getSpeed()
	{
		return speed;
	}

	/**
	 * Get the attack point for the direction from which the character would be attacked
	 * @param direction the direction (from the character's point of view) of the attack point
	 * @return the point
	 */
	public float[] getAttackPoint(Direction direction)
	{
		float[] point = new float[2];

		// find the character's direction
		Direction charDirection = getDirection();

		// figure out the absolue direction
		Direction actualDirection;
		if((charDirection == Direction.LEFT && direction == Direction.LEFT) ||
				(charDirection == Direction.UP && direction == Direction.DOWN) ||
				(charDirection == Direction.RIGHT && direction == Direction.RIGHT) ||
				(charDirection == Direction.DOWN && direction == Direction.UP))
		{
			actualDirection = Direction.DOWN;
		}
		else if((charDirection == Direction.LEFT && direction == Direction.UP) ||
				(charDirection == Direction.UP && direction == Direction.LEFT) ||
				(charDirection == Direction.RIGHT && direction == Direction.DOWN) ||
				(charDirection == Direction.DOWN && direction == Direction.RIGHT))
		{
			actualDirection = Direction.LEFT;
		}
		else if((charDirection == Direction.LEFT && direction == Direction.RIGHT) ||
				(charDirection == Direction.UP && direction == Direction.UP) ||
				(charDirection == Direction.RIGHT && direction == Direction.LEFT) ||
				(charDirection == Direction.DOWN && direction == Direction.DOWN))
		{
			actualDirection = Direction.UP;
		}
		else
		{
			actualDirection = Direction.RIGHT;
		}

		// figure out the point coordinate
		float pointX;
		float pointY;
		if(actualDirection == Direction.LEFT)
		{
			pointX = x - width - 0.001f;
			pointY = y;
		}
		else if(actualDirection == Direction.UP)
		{
			pointX = x;
			pointY = y + height + 0.001f;
		}
		else if(actualDirection == Direction.RIGHT)
		{
			pointX = x + width + 0.001f;
			pointY = y;
		}
		else
		{
			pointX = x;
			pointY = y - height - 0.001f;
		}

		// return the point
		point[0] = pointX;
		point[1] = pointY;
		return point;
	}

	/**
	 * Find out if the character was attacked recently
	 * @return attacked
	 */
	public boolean wasAttacked()
	{
		return attacked;
	}

	/**
	 * Check if the character can attack
	 * @return
	 */
	public boolean canAttack()
	{
		return attackAbility.isReady();
	}

	/**
	 * Check if the character can use it's special ability
	 * @return
	 */
	public boolean canUseSpecial()
	{
		return specialAbility.isReady();
	}

	/**
	 * Set the character to be taunted or not
	 * @param taunted
	 */
	public void setTaunted(boolean taunted)
	{
		this.taunted = taunted;
	}

	/**
	 * Check whether the character is taunted or not
	 * @return taunted
	 */
	public boolean isTaunted()
	{
		return taunted;
	}

	/**
	 * Add an attacker
	 * @param newAttacker
	 */
	public void addAttacker(Character newAttacker)
	{
		if(!attackers.contains(newAttacker))
		{
			attackers.add(newAttacker);
		}
	}

	/**
	 * Get the number of attackers
	 * @return number of attackers
	 */
	public int getNumAttackers()
	{
		return attackers.size();
	}

	/**
	 * Remove an attacker
	 * @param attacker
	 */
	public void removeAttacker(Character attacker)
	{
		if(attackers.contains(attacker))
		{
			attackers.remove(attacker);
		}
	}


}
