/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package gameobject.character;

import event.CollisionEvent;
import gameobject.GameObject;
import gameobject.ObjectType;
import gameobject.Projectile;
import hud.StatusBar;

import java.util.ArrayList;

import ability.MageAttack;
import ability.MageDefend;
import ability.MageSpecial;
import ability.RogueAttack;
import ability.RogueDefend;
import ability.RogueSpecial;
import ability.WarriorAttack;
import ability.WarriorDefend;
import ability.WarriorSpecial;

import com.badlogic.gdx.graphics.g2d.Animation;

/**
 * Enemy class extends the character class
 * @author jordan
 * @version Nov 7 2013
 */
public class Enemy extends Character {

	private EnemyHealthBar healthBar;

	// ability constants

	// warrior
	final private float WARRIOR_ATTACK_RANGE = 0.02f;
	final private int WARRIOR_ATTACK_DAMAGE = 30;
	final private int WARRIOR_ATTACK_ENERGY_USE = 10;
	final private int WARRIOR_ATTACK_COOLDOWN = 30;
	final private float WARRIOR_DEFEND_ENERGY_USE = 0.1f;
	final private float WARRIOR_SPECIAL_RANGE = 1.0f;
	final private int WARRIOR_SPECIAL_ENERGY_USE = 20;
	final private int WARRIOR_SPECIAL_DURATION = 500;
	final private int WARRIOR_SPECIAL_COOLDOWN = 100;
	final private int WARRIOR_MAX_HEALTH = 200;
	final private int WARRIOR_MAX_ENERGY = 100;
	final private float WARRIOR_ENERGY_REGEN = 0.2f;

	// mage
	final private float MAGE_ATTACK_RANGE = 0.7f;
	final private float MAGE_ATTACK_RADIUS = 0.07f;
	final private int MAGE_ATTACK_DAMAGE = 40;
	final private int MAGE_ATTACK_ENERGY_USE = 30;
	final private int MAGE_ATTACK_COOLDOWN = 50;
	final private float MAGE_DEFEND_ENERGY_USE = 0.5f;
	final private float MAGE_SPECIAL_RANGE = 0.2f;
	final private int MAGE_SPECIAL_ENERGY_USE = 50;
	final private int MAGE_SPECIAL_HEALING = 50;
	final private int MAGE_SPECIAL_COOLDOWN = 100;
	final private int MAGE_MAX_HEALTH = 100;
	final private int MAGE_MAX_ENERGY = 200;
	final private float MAGE_ENERGY_REGEN = 0.2f;

	// rogue
	final private float ROGUE_ATTACK_RANGE = 1.0f;
	final private int ROGUE_ATTACK_DAMAGE = 20;
	final private int ROGUE_ATTACK_ENERGY_USE = 10;
	final private int ROGUE_ATTACK_COOLDOWN = 30;
	final private float ROGUE_DEFEND_ENERGY_USE = 0.5f;
	final private float ROGUE_SPECIAL_RANGE = 0.15f;
	final private int ROGUE_SPECIAL_ENERGY_USE = 20;
	final private int ROGUE_SPECIAL_DAMAGE = 100;
	final private int ROGUE_SPECIAL_COOLDOWN = 100;
	final private int ROGUE_MAX_HEALTH = 150;
	final private int ROGUE_MAX_ENERGY = 150;
	final private float ROGUE_ENERGY_REGEN = 0.2f;

	/**
	 * Constructor
	 * @param x
	 * @param y
	 * @param startingState
	 * @param width
	 * @param height
	 * @param objectType
	 * @param characterType
	 * @param animations
	 * @throws Exception
	 */
	public Enemy(float x, float y, AnimationState startingState, float width, float height,
			ObjectType objectType, CharacterType characterType, ArrayList<Animation> animations) throws Exception
			{
		super(x, y, startingState, width, height, objectType, characterType, animations);
		speed = 0.003f;
		if(characterType == CharacterType.WARRIOR)
		{
			name = "Enemy Warrior";
			attackAbility = new WarriorAttack(WARRIOR_ATTACK_RANGE, WARRIOR_ATTACK_DAMAGE,
					WARRIOR_ATTACK_ENERGY_USE, WARRIOR_ATTACK_COOLDOWN, this);
			defendAbility = new WarriorDefend(WARRIOR_DEFEND_ENERGY_USE, this);
			specialAbility = new WarriorSpecial(WARRIOR_SPECIAL_RANGE,
					WARRIOR_SPECIAL_ENERGY_USE, WARRIOR_SPECIAL_DURATION, WARRIOR_SPECIAL_COOLDOWN, this);
			maxHealth = WARRIOR_MAX_HEALTH;
			maxEnergy = WARRIOR_MAX_ENERGY;
			energyRegen = WARRIOR_ENERGY_REGEN;
		}
		else if (characterType == CharacterType.MAGE)
		{
			name = "Enemy Mage";
			attackAbility = new MageAttack(false, MAGE_ATTACK_RANGE, MAGE_ATTACK_RADIUS, MAGE_ATTACK_DAMAGE,
					MAGE_ATTACK_ENERGY_USE, MAGE_ATTACK_COOLDOWN, this);
			defendAbility = new MageDefend(MAGE_DEFEND_ENERGY_USE, this);
			specialAbility = new MageSpecial(false, MAGE_SPECIAL_RANGE,
					MAGE_SPECIAL_ENERGY_USE, MAGE_SPECIAL_HEALING, MAGE_SPECIAL_COOLDOWN, this);
			maxHealth = MAGE_MAX_HEALTH;
			maxEnergy = MAGE_MAX_ENERGY;
			energyRegen = MAGE_ENERGY_REGEN;
		}
		else
		{
			name = "Enemy Rogue";
			attackAbility = new RogueAttack(ROGUE_ATTACK_RANGE, ROGUE_ATTACK_DAMAGE,
					ROGUE_ATTACK_ENERGY_USE, ROGUE_ATTACK_COOLDOWN, this);
			defendAbility = new RogueDefend(ROGUE_DEFEND_ENERGY_USE, this);
			specialAbility = new RogueSpecial(ROGUE_SPECIAL_RANGE,
					ROGUE_SPECIAL_ENERGY_USE, ROGUE_SPECIAL_DAMAGE, ROGUE_SPECIAL_COOLDOWN, this);
			maxHealth = ROGUE_MAX_HEALTH;
			maxEnergy = ROGUE_MAX_ENERGY;
			energyRegen = ROGUE_ENERGY_REGEN;
		}

		health = maxHealth;
		energy = maxEnergy;

		healthBar = new EnemyHealthBar("assets/data/hud/bar-health.png", this);
			}

	/*
	 * @see gameobject.character.Character#checkForInput(float)
	 */
	@Override
	protected void checkForInput(float delta)
	{
		strategy.update(delta);
	}

	/*
	 * @see gameobject.character.Character#handleSpecialCollision(event.CollisionEvent)
	 */
	protected void handleSpecialCollision(CollisionEvent cevent)
	{
		GameObject otherObject = cevent.getCollidingObject();
		ObjectType type = otherObject.getObjectType();

		// hit by projectile
		if (type == ObjectType.ALLYPROJECTILE)
		{
			Projectile p = (Projectile)otherObject;
			attacked(p.getDamage(), p);
		}
	}

	/**
	 * Get the health bar
	 * @return healthBar
	 */
	public EnemyHealthBar getHealthBar()
	{
		return healthBar;
	}

}