/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package gameobject.character;

import event.CollisionEvent;
import game.DialogueHandler;
import game.GameScreen;
import game.GameUtility;
import game.Player;
import game.QuestMain;
import gameobject.GameObject;
import gameobject.ObjectType;
import gameobject.Projectile;
import gameobject.SolidObject;

import java.util.ArrayList;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector3;

/**
 * PartyMember extends the character class
 * @author jordan, Eyleen 
 * @version Nov 7 2013
 */
public class PartyMember extends Character {


	int clickedX = 0;
	int clickedY = 0;

	/**
	 * Constructor
	 * @param x
	 * @param y
	 * @param startingState
	 * @param width
	 * @param height
	 * @param objectType
	 * @param characterType
	 * @param animations
	 * @throws Exception
	 */
	public PartyMember(float x, float y, AnimationState startingState, float width, float height,
			ObjectType objectType, CharacterType characterType, ArrayList<Animation> animations) throws Exception
			{
		super(x, y, startingState, width, height, objectType, characterType, animations);
		if (objectType == ObjectType.SECONDARY || objectType == ObjectType.TERTIARY){
			speed = 0.004f;
		}
		if(characterType == CharacterType.WARRIOR)
		{
			name = "Warrior";
		}
		else if (characterType == CharacterType.MAGE)
		{
			name = "Mage";
		}
		else
		{
			name = "Rogue";
		}
			}

	/*
	 * @see gameobject.character.Character#checkForInput(float)
	 */
	@Override
	protected void checkForInput(float delta)
	{
		if(!dead)
		{
			// if the character is player controlled, get keyboard input
			if(objectType == ObjectType.PRIMARY)
			{
				GameScreen gameScreen = QuestMain.getCurrentScreen();
				Rectangle screenDimensions = gameScreen.getScreenDimensions();

				// movement
				if(Gdx.input.isKeyPressed(Keys.W) && y + height < screenDimensions.getY() + screenDimensions.getHeight())
				{
					movingUp = true;
				}
				else 
				{
					movingUp = false;
				}

				if(Gdx.input.isKeyPressed(Keys.S) && y > screenDimensions.getY())
				{
					movingDown = true;
				}
				else 
				{
					movingDown = false;
				}

				if(Gdx.input.isKeyPressed(Keys.D) && x + width < screenDimensions.getX() + screenDimensions.getWidth()) 
				{
					movingRight = true;
				}
				else 
				{
					movingRight = false;
				}

				if(Gdx.input.isKeyPressed(Keys.A) && x > screenDimensions.getX()) 
				{
					movingLeft = true;
				}
				else 
				{
					movingLeft = false;
				}

				// get cursor point
				float xPoint = Gdx.input.getX();
				float yPoint = Gdx.input.getY();
				float zPoint = 0;

				Vector3 pointVector = new Vector3(xPoint, yPoint, zPoint);
				OrthographicCamera camera = gameScreen.getCamera();
				camera.unproject(pointVector);

				cursorX = pointVector.x;
				cursorY = pointVector.y;

				// attacking
				attackingRight = false;
				attackingLeft = false;
				attackingUp = false;
				attackingDown = false;
				if(Gdx.input.isButtonPressed(Input.Buttons.LEFT) && !attacking && !defending &&
						energy >= attackAbility.getEnergyUse() && screenDimensions.contains(cursorX, cursorY))
				{
					clickedX = Gdx.input.getX();
					clickedY = Gdx.input.getY();

					//System.out.println(clickedX + "," + clickedY);

					// figure out direction of attack
					Direction attackDirection = GameUtility.getDirection(this, cursorX, cursorY);

					// facing right
					if(attackDirection == Direction.RIGHT)
					{
						attackingRight = true;
					}
					// facing left
					else if(attackDirection == Direction.LEFT)
					{
						attackingLeft = true;
					}
					// facing down
					else if(attackDirection == Direction.UP)
					{
						attackingUp = true;
					}
					// facing down
					else
					{
						attackingDown = true;
					}

				}

				defendingRight = false;
				defendingLeft = false;
				defendingUp = false;
				defendingDown = false;
				//defending
				if (Gdx.input.isKeyPressed(Keys.E) && energy >= defendAbility.getEnergyUse())
				{
					defending = true;

					Animation animation = getAnimation();
					animation.setPlayMode(0);

					if (characterType == CharacterType.ROGUE)
					{
						setRogueHiding(true);
					}
				}
				else
				{
					if(defending)
					{
						switch(characterState)
						{
						case DEFENDING_RIGHT:
							setCharacterState(AnimationState.END_DEFEND_RIGHT);
							break;
						case DEFENDING_LEFT:
							setCharacterState(AnimationState.END_DEFEND_LEFT);
							break;
						case DEFENDING_UP:
							setCharacterState(AnimationState.END_DEFEND_UP);
							break;
						case DEFENDING_DOWN:
							setCharacterState(AnimationState.END_DEFEND_DOWN);
							break;
						default:
							break;
						}
						if(endDefendingRightAnimation.isAnimationFinished(getStateTime()) ||
								endDefendingLeftAnimation.isAnimationFinished(getStateTime()) ||
								endDefendingUpAnimation.isAnimationFinished(getStateTime()) || 
								endDefendingDownAnimation.isAnimationFinished(getStateTime()))
						{
							defending = false;
							if (characterType == CharacterType.ROGUE)
							{
								setRogueHiding(false);
							}
						}
					}
					else
					{
						defending = false;
						if (characterType == CharacterType.ROGUE)
						{
							setRogueHiding(false);
						}
					}				
				}

				//special
				specialRight = false;
				specialLeft = false;
				specialUp = false;
				specialDown = false;
				if(Gdx.input.isKeyPressed(Keys.F) && !defending && !attacking && !usingSpecial &&
						energy >= specialAbility.getEnergyUse()){

					if((characterState == AnimationState.MOVING_LEFT || characterState == AnimationState.FACING_LEFT))
					{
						specialLeft = true;
					}

					if((characterState == AnimationState.MOVING_RIGHT || characterState == AnimationState.FACING_RIGHT))
					{
						specialRight = true;
					}

					if((characterState == AnimationState.MOVING_UP || characterState == AnimationState.FACING_UP))
					{
						specialUp = true;
					}

					if((characterState == AnimationState.MOVING_DOWN || characterState == AnimationState.FACING_DOWN))
					{
						specialDown = true;
					}
				}
			}

			// otherwise, the character is controlled by its strategy
			else
			{
				strategy.update(delta);
			}
		}
		// character is dead
		else
		{
			movingRight = false;
			movingLeft = false;
			movingUp = false;
			movingDown = false;

			attackingRight = false;
			attackingLeft = false;
			movingRight = false;
			movingLeft = false;
			movingUp = false;
			movingDown = false;

			attackingRight = false;
			attackingLeft = false;
			attackingUp = false;
			attackingDown = false;

			defending = false;

			defendingRight = false;
			defendingLeft = false;
			defendingUp = false;
			defendingDown = false;

			specialRight = false;
			specialLeft = false;
			specialUp = false;
			specialDown = false;
		}


	}

	public void revive()
	{
		dead = false;
		health = maxHealth;

		switch(characterState)
		{
		case DEAD_RIGHT:
			setCharacterState(AnimationState.REVIVE_RIGHT);
			break;
		case DEAD_LEFT:
			setCharacterState(AnimationState.REVIVE_LEFT);
			break;
		case DEAD_UP:
			setCharacterState(AnimationState.REVIVE_UP);
			break;
		case DEAD_DOWN:
			setCharacterState(AnimationState.REVIVE_DOWN);
			break;
		default:
			break;
		}
	}

	/*
	 * @see gameobject.character.Character#handleSpecialCollision(event.CollisionEvent)
	 */
	protected void handleSpecialCollision(CollisionEvent cevent)
	{
		GameObject otherObject = cevent.getCollidingObject();
		ObjectType type = otherObject.getObjectType();

		// hit by projectile
		if (type == ObjectType.ENEMYPROJECTILE)
		{
			Projectile p = (Projectile)otherObject;
			attacked(p.getDamage(), p);
		}
	}

	/**
	 * Set the party member's object type to ally or player
	 * @param newType the new ObjectType (only PLAYER and ALLY allowed)
	 */
	public void setObjectType(ObjectType newType)
	{
		if(newType == ObjectType.PRIMARY || newType == ObjectType.SECONDARY || newType == ObjectType.TERTIARY)
		{
			objectType = newType;
		}
	}

	/**
	 * Check whether the party member is player controlled
	 * @return true if player-controlled, false otherwise
	 */
	public boolean isActivePartyMember(){
		return (objectType == ObjectType.PRIMARY);
	}

	/**
	 * Increase health
	 * @param i the health to be added
	 */
	public void increaseHealth(int i) {
		health += i;

		if(health > maxHealth){
			health = maxHealth;
		}
	}

	/**
	 * Increase stamina
	 * @param i the stamina to be added
	 */
	public void increaseEnergy(int i){
		energy += i;

		if(energy > maxEnergy){
			energy = maxEnergy;
		}
	}

}
