/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package hud;

import menu.Input;
import menu.PauseOverlay;
import menu.Size;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Buttons;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.InputMultiplexer;
import com.badlogic.gdx.InputProcessor;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.Texture.TextureFilter;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;

import game.Clock;
import game.DialogueHandler;
import game.GameScreen;
import game.Player;
import game.QuestMain;
import gameobject.character.CharacterType;

/**
 * This class controls all elements of the HUD and its input
 * @author Eyleen
 * @version Jan 11 2014
 */
public class HUD implements InputProcessor{
	private final String BACKGROUND = "assets/data/backgrounds/bg_hud.png";
	
	private final String HEALTH_POTION = "assets/data/hud/potion-health.png";
	private final String ENERGY_POTION = "assets/data/hud/potion-energy.png";
	private final String PAUSE = "assets/data/hud/pause.png";
	
	private final String WARRIOR_PORTRAIT = "assets/data/hud/portrait-warrior.png";
	private final String MAGE_PORTRAIT = "assets/data/hud/portrait-mage.png";
	private final String ROGUE_PORTRAIT = "assets/data/hud/portrait-rogue.png";
	
	private final String WARRIOR_ATTACK = "assets/data/ability/warrior-attack.png";
	private final String WARRIOR_ALT_ATTACK = "assets/data/ability/warrior-attack-alt.png";
	private final String WARRIOR_DEFEND = "assets/data/ability/warrior-defend.png";
	private final String WARRIOR_ALT_DEFEND = "assets/data/ability/warrior-defend-alt.png";
	private final String WARRIOR_SPECIAL = "assets/data/ability/warrior-special.png";
	private final String WARRIOR_ALT_SPECIAL = "assets/data/ability/warrior-special-alt.png";
	private final String WARRIOR_COOLDOWN = "assets/data/ability/warrior-cooldown.png";
	private final String MAGE_ATTACK = "assets/data/ability/mage-attack.png";
	private final String MAGE_ALT_ATTACK = "assets/data/ability/mage-attack-alt.png";
	private final String MAGE_DEFEND = "assets/data/ability/mage-defend.png";
	private final String MAGE_ALT_DEFEND = "assets/data/ability/mage-defend-alt.png";
	private final String MAGE_SPECIAL = "assets/data/ability/mage-special.png";
	private final String MAGE_ALT_SPECIAL = "assets/data/ability/mage-special-alt.png";
	private final String MAGE_COOLDOWN = "assets/data/ability/mage-cooldown.png";
	private final String ROGUE_ATTACK = "assets/data/ability/rogue-attack.png";
	private final String ROGUE_ALT_ATTACK = "assets/data/ability/rogue-attack-alt.png";
	private final String ROGUE_DEFEND = "assets/data/ability/rogue-defend.png";
	private final String ROGUE_ALT_DEFEND = "assets/data/ability/rogue-defend-alt.png";
	private final String ROGUE_SPECIAL = "assets/data/ability/rogue-special.png";
	private final String ROGUE_ALT_SPECIAL = "assets/data/ability/rogue-special-alt.png";
	private final String ROGUE_COOLDOWN= "assets/data/ability/rogue-cooldown.png"; 
	
	private final float BACKGROUND_X = -0.6f;
	private final float BACKGROUND_Y = -0.5f;
	private final float BACKGROUND_SIZE_X = 1.2f;
	private final float BACKGROUND_SIZE_Y = 0.2f; 
	
	private final float WARRIOR_FRAME_X = -0.48f;
	private final float MAGE_FRAME_X = -0.24f;
	private final float ROGUE_FRAME_X = 0.0f;
	private final float FRAME_Y = -0.465f;
	
	private final float TRAY_X = 0.25f;
	private final float TRAY_Y = -0.36f;
	
	private final float POTION_X = 0.33f;
	private final float HEALTH_POTION_Y = -0.39f;
	private final float ENERGY_POTION_Y = -0.45f;
	
	private static final float PAUSE_X = 0.4f;
	private static final float PAUSE_Y = -0.445f;
	private static final float PAUSE_SIZE_X = 0.1f;
	private static final float PAUSE_SIZE_Y = 0.1f; 
	
	private final char HEALTH_KEY = '1';
	private final char ENERGY_KEY = '2';
	private final char PAUSE_KEY = 'p';

	//player frames
	private PlayerFrame warrior;
	private PlayerFrame mage;
	private PlayerFrame rogue;
	
	//ability trays
	private AbilityTray warriorTray;
	private AbilityTray mageTray;
	private AbilityTray rogueTray;
	
	//potions
	private ItemIcon healthPotion;
	private ItemIcon energyPotion;
	
	//pause button
	private TextureRegion backgroundRegion;
	private TextureRegion pauseRegion;
	
	//pause screen
	private PauseOverlay pauseOverlay;
	
	//dialogue alert
	private static Alert alert;
	
	//other variables
	private boolean init = false;
	private Player player = null;
	private Clock clock = null;
	private static HUD instance = null;
	private boolean pause;
	
	private static InputMultiplexer plex;
	
	/**
	 * Singleton constructor
	 */
	private HUD(){}
	
	/**
	 * creates a new instance of HUD
	 * WILL overwrite an existing instance
	 * @return the new instance
	 */
	public static HUD createInstance(){
		if(instance == null){
			instance = new HUD();
			plex = new InputMultiplexer();
			plex.addProcessor(DialogueHandler.getInstance());
			plex.addProcessor(instance);
			Gdx.input.setInputProcessor(plex);
		}
		return instance;
	}
	
	public static HUD getInstance(){
		return instance;
	}
	
	/**
	 * Initialize the HUD components
	 * @param g the QuestMain linked to the game
	 */
	public void initHUD(QuestMain g) throws Exception{
		player = Player.getInstance();
		clock = Clock.getInstance();
		
		warrior = new PlayerFrame(player.getWarrior(), WARRIOR_PORTRAIT,
				WARRIOR_FRAME_X, FRAME_Y, ModeType.ATTACK);
		mage = new PlayerFrame(player.getMage(), MAGE_PORTRAIT, MAGE_FRAME_X,
				FRAME_Y, ModeType.ATTACK);
		rogue = new PlayerFrame(player.getRogue(), ROGUE_PORTRAIT, ROGUE_FRAME_X,
				FRAME_Y, ModeType.ATTACK);
		
		warriorTray = new AbilityTray(WARRIOR_ATTACK, WARRIOR_ALT_ATTACK,
				WARRIOR_DEFEND, WARRIOR_ALT_DEFEND, WARRIOR_SPECIAL,
				WARRIOR_ALT_SPECIAL, WARRIOR_COOLDOWN, player.getWarrior(),
				TRAY_X, TRAY_Y);
		mageTray = new AbilityTray(MAGE_ATTACK, MAGE_ALT_ATTACK, MAGE_DEFEND,
				MAGE_ALT_DEFEND, MAGE_SPECIAL, MAGE_ALT_SPECIAL, MAGE_COOLDOWN,
				player.getMage(), TRAY_X, TRAY_Y);
		rogueTray = new AbilityTray(ROGUE_ATTACK, ROGUE_ALT_ATTACK, ROGUE_DEFEND,
				ROGUE_ALT_DEFEND, ROGUE_SPECIAL, ROGUE_ALT_SPECIAL, ROGUE_COOLDOWN,
				player.getRogue(), TRAY_X, TRAY_Y);
		
		healthPotion = new ItemIcon(HEALTH_POTION, POTION_X, HEALTH_POTION_Y,
				player.getHealthPotionNumber());
		energyPotion = new ItemIcon(ENERGY_POTION, POTION_X, ENERGY_POTION_Y,
				player.getEnergyPotionNumber());
		
		GameScreen gameScreen = QuestMain.getCurrentScreen();
		
		Texture pauseTexture = new Texture(Gdx.files.internal(PAUSE));
		gameScreen.addTexture(pauseTexture);
		pauseRegion = new TextureRegion(pauseTexture);
		
		Texture backgroundTexture = new Texture(Gdx.files.internal(BACKGROUND));
		gameScreen.addTexture(backgroundTexture);
		backgroundRegion = new TextureRegion(backgroundTexture, 0, 0,
				Size.FULLSCREEN, Size.QUARTERSCREEN);
		
		pauseOverlay = PauseOverlay.makeNewInstance(g);
		
		alert = Alert.makeNewInstance();
		
		init = true;
		pause = false;
	}
	
	/**
	 * Pause game and pass input control to pause overlay
	 */
	private void pause(){
		clock.pause();
		pause = true;
		Gdx.input.setInputProcessor(pauseOverlay);
	}
	
	/**
	 * Resume game play and HUD regains input control 
	 */
	public void unpause(){
		clock.unpause();
		pause = false;
		Gdx.input.setInputProcessor(plex);
	}
	
	public static InputProcessor getMultiplexer(){
		return plex;
	}
	
	public void draw(SpriteBatch spriteBatch){
		if(!init){
			return;
		}
		spriteBatch.draw(backgroundRegion, BACKGROUND_X, BACKGROUND_Y, BACKGROUND_SIZE_X, BACKGROUND_SIZE_Y);
		warrior.draw(spriteBatch);
		mage.draw(spriteBatch);
		rogue.draw(spriteBatch);
		warriorTray.draw(spriteBatch);
		mageTray.draw(spriteBatch);
		rogueTray.draw(spriteBatch);
		healthPotion.draw(spriteBatch);
		energyPotion.draw(spriteBatch);
		spriteBatch.draw(pauseRegion, PAUSE_X, PAUSE_Y, PAUSE_SIZE_X, PAUSE_SIZE_Y);
		
		if(clock.isPaused() && pause){
			pauseOverlay.draw(spriteBatch);
		}
		alert.draw(spriteBatch);
	}
		
	public void update() {
		if(!init || clock.isPaused()){
			return;
		}
		
		warrior.update();
		mage.update();
		rogue.update();
		warriorTray.update();
		mageTray.update();
		rogueTray.update();
		healthPotion.update(player.getHealthPotionNumber());
		energyPotion.update(player.getEnergyPotionNumber());
	}
	
	@Override
	public boolean keyTyped(char character){
		if(clock.isPaused() && DialogueHandler.getInstance().isFinished()){
			if(character == PAUSE_KEY){
				unpause();
				return true;
			}
		}
		if(character == HEALTH_KEY){
			player.useHealthPotion();
			return true;
		}
		if(character == ENERGY_KEY){
			player.useEnergyPotion();
			return true;
		}
		if(character == PAUSE_KEY && DialogueHandler.getInstance().isFinished()){
			pause();
			return true;
		}
		return false;
	}
	
	@Override
	public boolean touchUp(int screenX, int screenY, int pointer, int button) {
		if(button == Buttons.LEFT){
			float x = Input.convertScreenCoorsX(screenX);
			float y = Input.convertScreenCoorsY(screenY);
			
			if(Input.inRange(x, y, healthPotion.getX(), healthPotion.getY(),
					healthPotion.getSizeX(), healthPotion.getSizeY())){
				player.useHealthPotion();
				return true;
			}
			
			if(Input.inRange(x, y, energyPotion.getX(), energyPotion.getY(),
					energyPotion.getSizeX(), energyPotion.getSizeY())){
				player.useEnergyPotion();
				return true;
			}

			if(Input.inRange(x, y, PAUSE_X, PAUSE_Y, PAUSE_SIZE_X, PAUSE_SIZE_Y) && DialogueHandler.getInstance().isFinished()){
				pause();
				return true;
			}
			
			Player player = Player.getInstance();
			// warrior frame was clicked on
			if(Input.inRange(x, y, WARRIOR_FRAME_X, FRAME_Y, warrior.getFrameX(), warrior.getFrameY()))
			{
				player.switchCommanded(CharacterType.WARRIOR);
				return true;
			}
			// mage frame was clicked on
			if(Input.inRange(x, y, MAGE_FRAME_X, FRAME_Y, mage.getFrameX(), mage.getFrameY()))
			{
				player.switchCommanded(CharacterType.MAGE);
				return true;
			}
			// rogue frame was clicked on
			if(Input.inRange(x, y, ROGUE_FRAME_X, FRAME_Y, rogue.getFrameX(), rogue.getFrameY()))
			{
				player.switchCommanded(CharacterType.ROGUE);
				return true;
			}
		}
		return false;
	}
	
	@Override
	public boolean keyUp(int keycode){
		if(keycode == Keys.ESCAPE){
			if(clock.isPaused() && DialogueHandler.getInstance().isFinished()){
				unpause();
			}else if(DialogueHandler.getInstance().isFinished()){
				pause();
			}
			return true;
		}
		return false;
	}

	@Override
	public boolean keyDown(int keycode) {
		return false;
	}

	@Override
	public boolean touchDown(int screenX, int screenY, int pointer, int button) {
		return false;
	}

	@Override
	public boolean touchDragged(int screenX, int screenY, int pointer) {
		return false;
	}

	@Override
	public boolean mouseMoved(int screenX, int screenY) {
		return false;
	}

	@Override
	public boolean scrolled(int amount) {
		return false;
	}

	public static void alert() {
		alert.activate();
	}
}
