/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package hud;

import menu.Size;
import game.GameScreen;
import game.QuestMain;
import gameobject.ObjectType;
import gameobject.character.PartyMember;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.Texture.TextureFilter;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;

/**
 * The gui for the player characters
 * Includes:
 * 	Character portrait
 * 	Health/Stamina bars
 *  Mode icons
 * 	
 * @author Phoenix
 */
public class PlayerFrame{
	//character frame elements
	private final String FRAME = "assets/data/hud/frame.png";
	private final String FRAME_BACK = "assets/data/hud/frame-alt.png";
	private final String ACTIVE_FRAME = "assets/data/hud/frame-active.png";
	private final String ACTIVE_FRAME_BACK = "assets/data/hud/frame-active-alt.png";
	private final String DEAD_FRAME = "assets/data/hud/frame-dead.png";
	private final String DEAD_FRAME_BACK = "assets/data/hud/frame-dead-alt.png";
	private final String PORTRAIT;
	private final float FRAME_X;
	private final float FRAME_Y;
	private final float FRAME_SIZE_X = 0.23f;
	private final float FRAME_SIZE_Y = 0.12f;
	private final float PORTRAIT_SIZE_X = 0.07f;
	private final float PORTRAIT_SIZE_Y = 0.07f;
	
	private Texture texture;
	private Texture activeTexture;
	private Texture deadTexture;
	private Texture textureBack;
	private Texture activeTextureBack;
	private Texture deadTextureBack;
	private Texture portraitTexture;
	private TextureRegion region;
	private TextureRegion regionBack;
	private TextureRegion portraitRegion;
	
	//mode icons attributes
	private final String ATK_SELECTED = "assets/data/mode/atk-selected.png";
	private final String ATK_UNSELECTED = "assets/data/mode/atk-unselected.png";
	private final String ATK_DEACTIVE = "assets/data/mode/atk-deactive.png";
	private final String DEF_SELECTED = "assets/data/mode/def-selected.png";
	private final String DEF_UNSELECTED = "assets/data/mode/def-unselected.png";
	private final String DEF_DEACTIVE = "assets/data/mode/def-deactive.png";
	private final String SUPPORT_UNSELECTED = "assets/data/mode/support-selected.png";
	private final String SUPPORT_SELECTED = "assets/data/mode/support-unselected.png";
	private final String SUPPORT_DEACTIVE = "assets/data/mode/support-deactive.png";
	private final float ICON_OFFSET = 0.04f;//offset between icons
	private final float ICON_OFFSET_X = 0.097f;//offset from frame origin
	private final float ICON_OFFSET_Y = 0.03f;//offset from frame origin
	private ModeIcon atkMode;
	private ModeIcon defMode;
	private ModeIcon supportMode;
	
	//character portrait
	private final float PORTRAIT_OFFSET_X = 0.019f;
	private final float PORTRAIT_OFFSET_Y = 0.028f;
	
	//status bars
	private final String HEALTH_BAR = "assets/data/hud/bar-health.png";
	private final String ENERGY_BAR = "assets/data/hud/bar-energy.png";
	private final String BACKING = "assets/data/hud/backing.png";
	private final float BAR_OFFSET_X = 0.09f;
	private final float HEALTH_BAR_OFFSET_Y = 0.079f;
	private final float ENERGY_BAR_OFFSET_Y = 0.06f;
	private final float BACKING_OFFSET_X = 0.089f;
	private final float BACKING_OFFSET_Y = 0.059f;
	private final float BACKING_SIZE_X = 0.13f;
	private final float BACKING_SIZE_Y = 0.05f;
	private StatusBar healthBar;
	private StatusBar energyBar;
	private TextureRegion backingRegion;
	
	//controller variables
	//private ModeType currentMode;
	
	//player character
	private PartyMember character;
		
	/**
	 * Default constructor for testing proposes
	 * Uses placeholder assets
	 */
	public PlayerFrame(PartyMember c, String fileName, float x, float y,
			ModeType mode){
		FRAME_X = x;
		FRAME_Y = y;
		PORTRAIT = fileName;
		
		GameScreen gameScreen = QuestMain.getCurrentScreen();

		//frame
		texture = new Texture(Gdx.files.internal(FRAME));
		gameScreen.addTexture(texture);
		textureBack = new Texture(Gdx.files.internal(FRAME_BACK));
		gameScreen.addTexture(textureBack);
		activeTexture = new Texture(Gdx.files.internal(ACTIVE_FRAME));
		gameScreen.addTexture(activeTexture);
		activeTextureBack = new Texture(Gdx.files.internal(ACTIVE_FRAME_BACK));
		gameScreen.addTexture(activeTextureBack);
		deadTexture = new Texture(Gdx.files.internal(DEAD_FRAME));
		gameScreen.addTexture(deadTexture);
		deadTextureBack = new Texture(Gdx.files.internal(DEAD_FRAME_BACK));
		gameScreen.addTexture(deadTextureBack);
		portraitTexture = new Texture(Gdx.files.internal(PORTRAIT));
		gameScreen.addTexture(portraitTexture);
		Texture backingTexture = new Texture(Gdx.files.internal(BACKING));
		gameScreen.addTexture(backingTexture);
		
		region = new TextureRegion(texture, 0, 0, Size.QUARTERSCREEN,
				Size.PORTRAIT);
		regionBack = new TextureRegion(textureBack, 0, 0, Size.QUARTERSCREEN,
				Size.PORTRAIT);
		portraitRegion = new TextureRegion(portraitTexture, 0, 0,
				Size.QUARTERSCREEN, Size.QUARTERSCREEN);
		backingRegion = new TextureRegion(backingTexture, 0, 0, Size.PORTRAIT, Size.PORTRAIT);
		
		//mode buttons
		atkMode = new ModeIcon(ATK_DEACTIVE, ATK_UNSELECTED,
				ATK_SELECTED, c, x + ICON_OFFSET_X, y + ICON_OFFSET_Y);
		defMode = new ModeIcon(DEF_DEACTIVE, DEF_UNSELECTED,
				DEF_SELECTED, c, x + ICON_OFFSET_X + ICON_OFFSET,
				y + ICON_OFFSET_Y);
		supportMode = new ModeIcon(SUPPORT_DEACTIVE, SUPPORT_UNSELECTED,
				SUPPORT_SELECTED, c, x + ICON_OFFSET_X + (ICON_OFFSET * 2),
				y + ICON_OFFSET_Y);
		
		//status bars
		healthBar = new StatusBar(HEALTH_BAR, x + BAR_OFFSET_X,
				y + HEALTH_BAR_OFFSET_Y);
		energyBar = new StatusBar(ENERGY_BAR, x + BAR_OFFSET_X,
				y + ENERGY_BAR_OFFSET_Y);
		
		//setting up control variables
		//currentMode = mode;
		character = c;
	}
	
	public void update(){
		if(character.isDead()){
			region.setRegion(deadTexture);
			regionBack.setRegion(deadTextureBack);
		}else if(character.getObjectType() == ObjectType.PRIMARY){
			region.setRegion(activeTexture);
			regionBack.setRegion(activeTextureBack);
		}else if(character.getObjectType() == ObjectType.SECONDARY || character.getObjectType() == ObjectType.TERTIARY){
			region.setRegion(texture);
			regionBack.setRegion(textureBack);
		}
		atkMode.update();
		defMode.update();
		supportMode.update();
	}
	
	public void draw(SpriteBatch spriteBatch){
		spriteBatch.draw(regionBack, FRAME_X, FRAME_Y + 0.005f, FRAME_SIZE_X, FRAME_SIZE_Y);
		
		//draw the character portrait
		spriteBatch.draw(portraitRegion, FRAME_X + PORTRAIT_OFFSET_X,
				FRAME_Y + PORTRAIT_OFFSET_Y, PORTRAIT_SIZE_X, PORTRAIT_SIZE_Y);
		
		//draw the mode icons
		atkMode.draw(spriteBatch);
		defMode.draw(spriteBatch);
		supportMode.draw(spriteBatch);
		
		//add backing for status bars
		spriteBatch.draw(backingRegion, FRAME_X + BACKING_OFFSET_X, FRAME_Y + BACKING_OFFSET_Y, BACKING_SIZE_X, BACKING_SIZE_Y);
		
		//status bars
		healthBar.draw(spriteBatch, character.getHealth(), character.getMaxHealth());
		energyBar.draw(spriteBatch, character.getEnergy(), character.getMaxEnergy());		
		
		//add frame
		spriteBatch.draw(region, FRAME_X, FRAME_Y, FRAME_SIZE_X, FRAME_SIZE_Y);
	}

	public float getFrameX() {
		return FRAME_SIZE_X;
	}
	
	public float getFrameY(){
		return FRAME_SIZE_Y;
	}
}
