/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package manager;

import event.CollisionEvent;
import gameobject.GameObject;
import gameobject.ObjectType;

import java.util.ArrayList;
import java.util.List;

/**
 * Manages collisions between objects, following the publish/subscribe pattern
 * @author jordan
 * @version Oct 3 2013
 */
public class CollisionManager {

	private static CollisionManager instance = null;

	private List<GameObject> movingObjects;
	private List<GameObject> staticObjects;

	/**
	 * Constructor
	 */
	protected CollisionManager()
	{
		movingObjects = new ArrayList<GameObject>();
		staticObjects = new ArrayList<GameObject>();
	}

	public static CollisionManager getInstance()
	{
		return instance;
	}

	public static CollisionManager makeNewInstance()
	{
		instance = new CollisionManager();
		return instance;
	}

	/**
	 * Subscribe an object to receive collision events
	 * @param newObject
	 * @throws Exception if object is already subscribed
	 */
	public void subscribeObject(GameObject newObject) throws Exception
	{
		ObjectType type = newObject.getObjectType();
		// Object is able to move
		if(type == ObjectType.PRIMARY || type == ObjectType.ENEMY || type == ObjectType.SECONDARY || 
				type == ObjectType.TERTIARY || type == ObjectType.ALLYPROJECTILE ||
				type == ObjectType.ENEMYPROJECTILE)
		{
			addObject(movingObjects, newObject);
		}
		// object is unable to move
		else
		{
			addObject(staticObjects, newObject);
		}
	}

	/**
	 * Add the object to the given list
	 * @param list the list to be added to
	 * @param newObject the object to be added
	 * @throws Exception if object is already subscribed
	 */
	private void addObject(List<GameObject> list, GameObject newObject) throws Exception
	{
		if(!list.contains(newObject))
		{
			list.add(newObject);
		}
		else
		{
			throw new Exception("Object is already subscribed to receive collision events.");
		}
	}

	/**
	 * Un-subscribe an object
	 * @param object the object to be un-subscribed
	 * @throws Exception if object is not subscribed
	 */
	public void unsubscribeObject(GameObject object) throws Exception
	{
		ObjectType type = object.getObjectType();
		// object is able to move
		if(type == ObjectType.PRIMARY || type == ObjectType.ENEMY || type == ObjectType.SECONDARY || 
				type == ObjectType.TERTIARY || type == ObjectType.ALLYPROJECTILE ||
				type == ObjectType.ENEMYPROJECTILE)
		{
			removeObject(movingObjects, object);
		}
		// object is unable to move
		else
		{
			removeObject(staticObjects, object);
		}
	}

	/**
	 * Removes an object from the given list
	 * @param list the list to be removed from
	 * @param object the object to be removed
	 * @throws Exception if object is not subscribed
	 */
	private void removeObject(List<GameObject> list, GameObject object) throws Exception
	{
		if(list.contains(object))
		{
			list.remove(object);
		}
		else
		{
			throw new Exception(object.getObjectType() + " is not subscribed to receive collision events.");
		}
	}

	/**
	 * Check for collisions between objects
	 */
	public void checkCollisions()
	{
		// for every moving object
		for(GameObject object : movingObjects)
		{
			// check against the other moving objects
			for(int i = movingObjects.indexOf(object) + 1; i < movingObjects.size(); i++)
			{
				GameObject otherObject = movingObjects.get(i);
				if(object.getCollider().overlaps(otherObject.getCollider()))
				{
					// notify first object
					CollisionEvent collision = new CollisionEvent(otherObject);
					object.handleCollision(collision);

					// notify second object
					collision = new CollisionEvent(object);
					otherObject.handleCollision(collision);
				}
			}

			// check against static objects
			for(GameObject otherObject : staticObjects)
			{
				if(object.getCollider().overlaps(otherObject.getCollider()))
				{
					// notify first object
					CollisionEvent collision = new CollisionEvent(otherObject);
					object.handleCollision(collision);

					// notify second object
					collision = new CollisionEvent(object);
					otherObject.handleCollision(collision);
				}
			}
		}
	}

}
