/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package menu;

import game.QuestMain;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Buttons;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.Texture.TextureFilter;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;

/**
 * Screen that shows up when all player characters are dead
 * Gives the options to:
 * 
 * Exit the game
 * Retry the last screen
 * Load a different game
 * 
 * @author Eyleen
 */
public class GameOverScreen extends MenuScreen implements Screen{
	
	private final QuestMain game;
	private final LoadMenuScreen loadScreen;
	
	//initialize the audio file
			Sound selectSound = Gdx.audio.newSound(Gdx.files.internal("assets/data/sound/select.wav"));
		

	//image file names
	private final String BACKGROUND = "assets/data/menu/game_over_bg.png";
	private final String RETRY = "assets/data/menu/retry.png";
	private final String QUIT = "assets/data/menu/quit.png";
	private final String LOAD = "assets/data/menu/load.png";
	
	//image positions
	private final float RETRY_X = -0.4f;
	private final float RETRY_Y = -0.4f;
	private final float QUIT_X = 0.2f;
	private final float QUIT_Y = -0.4f;
	private final float LOAD_X = -0.1f;
	private final float LOAD_Y = -0.4f;
	
	//image sizes [screen coors]
	private final float BUTTON_SIZE_X = 0.25f;
	private final float BUTTON_SIZE_Y = 0.1f;
	
	//textures
	private TextureRegion background;
	private TextureRegion retry;
	private TextureRegion quit;
	private TextureRegion load;
	
	private QuitGameOverlay quitOverlay;
	
	public GameOverScreen(QuestMain g) throws Exception{
		game = g;
		
		Texture backgroundTexture = new Texture(Gdx.files.internal(BACKGROUND));
		backgroundTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		Texture retryTexture = new Texture(Gdx.files.internal(RETRY));
		retryTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		Texture quitTexture = new Texture(Gdx.files.internal(QUIT));
		quitTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		Texture loadTexture = new Texture(Gdx.files.internal(LOAD));
		loadTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		
		background = new TextureRegion(backgroundTexture, 0, 0,
				Size.FULLSCREEN, Size.FULLSCREEN);
		retry = new TextureRegion(retryTexture, 0, 0, Size.QUARTERSCREEN,
				Size.ICON);
		quit = new TextureRegion(quitTexture, 0, 0, Size.QUARTERSCREEN,
				Size.ICON);
		load = new TextureRegion(loadTexture, 0, 0, Size.QUARTERSCREEN,
				Size.ICON);
		
		loadScreen = LoadMenuScreen.getInstance();
		quitOverlay = new QuitGameOverlay();
	}
	
	@Override
	public boolean touchUp(int screenX, int screenY, int pointer, int button) {
		if(button == Buttons.LEFT){
			float x = Input.convertScreenCoorsX(screenX);
			float y = Input.convertScreenCoorsY(screenY);
			
			if(Input.inRange(x, y, RETRY_X, RETRY_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y)){

				//play sound
				selectSound.play(1.0f);
				
				//retry the last screen
				game.resumeGame();
				return true;
			}
			if(Input.inRange(x, y, QUIT_X, QUIT_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y)){

				//play sound
				selectSound.play(1.0f);
				
				//quit game
				quitOverlay.activate();
				Gdx.input.setInputProcessor(quitOverlay);
				return true;
			}
			if(Input.inRange(x, y, LOAD_X, LOAD_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y)){

				//play sound
				selectSound.play(1.0f);
				
				//load a game
				loadScreen.activate(this);
				Gdx.input.setInputProcessor(loadScreen);
				return true;
			}
		}
		
		return false;
	}

	@Override
	protected void draw() {
		spriteBatch.setProjectionMatrix(camera.combined);
		spriteBatch.begin();

		//draw background
		spriteBatch.draw(background, -0.5f, -0.5f, 1.0f, 1.0f);
		
		//draw buttons
		spriteBatch.draw(retry, RETRY_X, RETRY_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y);
		spriteBatch.draw(quit, QUIT_X, QUIT_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y);
		spriteBatch.draw(load, LOAD_X, LOAD_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y);
		
		loadScreen.draw(spriteBatch);
		quitOverlay.draw(spriteBatch);
		
		spriteBatch.end();
	}

	/*
	 * @see com.badlogic.gdx.Screen#dispose()
	 */
	@Override
	public void dispose() {
		spriteBatch.dispose();
	}

	/*
	 * @see com.badlogic.gdx.Screen#render(float)
	 */
	@Override
	public void render(float delta) {
		clearScreen();
		draw();
	}

	/*
	 * @see com.badlogic.gdx.Screen#resize(int, int)
	 */
	@Override
	public void resize(int width, int height) {
	}

	/*
	 * @see com.badlogic.gdx.Screen#pause()
	 */
	@Override
	public void pause(){
	}

	/*
	 * @see com.badlogic.gdx.Screen#resume()
	 */
	@Override
	public void resume() {
	}

	/*
	 * @see com.badlogic.gdx.Screen#show()
	 */
	@Override
	public void show() {
	}

	/*
	 * @see com.badlogic.gdx.Screen#hide()
	 */
	@Override
	public void hide() {
	}

	@Override
	protected void draw(SpriteBatch spriteBatch) {}
}
