/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package menu;

import game.QuestMain;
import menu.MenuScreen;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Buttons;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.Texture.TextureFilter;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;

/**
 * This class handles the main menu screen, its input, and the routing to other 
 * screens or the game itself
 * @author Eyleen
 * @version Dec 29 2013
 */
public class MainMenuScreen extends MenuScreen implements Screen{

	private static MainMenuScreen instance = null;
	private final QuestMain game;
	
	//image file names
	private final String BACKGROUND = "assets/data/menu/main_menu_bg.png";
	private final String BANNER = "assets/data/menu/title_banner.png";
	private final String NEW_GAME = "assets/data/menu/new_game.png";
	private final String LOAD_GAME = "assets/data/menu/load.png";
	private final String QUIT_GAME = "assets/data/menu/quit.png";
	
	//image positions
	private final float BANNER_X = -0.5f;
	private final float BANNER_Y = -0.4f;
	private final float NEW_GAME_X = -0.4f;
	private final float NEW_GAME_Y = -0.45f;
	private final float LOAD_GAME_X = -0.12f;
	private final float LOAD_GAME_Y = -0.45f;
	private final float QUIT_GAME_X = 0.15f;
	private final float QUIT_GAME_Y = -0.45f;
	
	//image sizes [screen space]
	private final float BANNER_SIZE_X = 1.05f;
	private final float BANNER_SIZE_Y = 1.2f;
	private final float BUTTON_SIZE_X = 0.25f;
	private final float BUTTON_SIZE_Y = 0.1f;
	
	//textures [images]
	private TextureRegion background;
	private TextureRegion banner;
	private TextureRegion newGame;
	private TextureRegion loadGame;
	private TextureRegion quitGame;
	
	//overlays
	private QuitGameOverlay quitOverlay;
	private LoadMenuScreen loadScreen;
	
	//initialize the audio file
		Sound selectSound = Gdx.audio.newSound(Gdx.files.internal("assets/data/sound/select.wav"));
		
	
	/**
	 * Singleton constructor
	 * @param g the QuestMain object representing the game
	 * @throws Exception 
	 */
	private MainMenuScreen(QuestMain g) throws Exception {
		super();
		
		game = g;
		
		//create texture regions
		Texture backgroundTexture = new Texture(Gdx.files.internal(BACKGROUND));
		backgroundTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		Texture bannerTexture = new Texture(Gdx.files.internal(BANNER));
		bannerTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		Texture newGameTexture = new Texture(Gdx.files.internal(NEW_GAME));
		newGameTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		Texture loadGameTexture = new Texture(Gdx.files.internal(LOAD_GAME));
		loadGameTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		Texture quitGameTexture = new Texture(Gdx.files.internal(QUIT_GAME));
		quitGameTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);

		background = new TextureRegion(backgroundTexture, 0, 0, Size.FULLSCREEN,
				Size.FULLSCREEN);
		banner = new TextureRegion(bannerTexture, 0, 0, Size.HALFSCREEN,
				Size.HALFSCREEN);
		newGame = new TextureRegion(newGameTexture, 0, 0, Size.QUARTERSCREEN,
				Size.ICON);
		loadGame = new TextureRegion(loadGameTexture, 0, 0, Size.QUARTERSCREEN,
				Size.ICON);
		quitGame = new TextureRegion(quitGameTexture, 0, 0, Size.QUARTERSCREEN,
				Size.ICON);
		
		loadScreen = LoadMenuScreen.makeNewInstance();
		quitOverlay = new QuitGameOverlay();
		
		loadScreen.deactivate();
		quitOverlay.deactivate();
		
		Gdx.input.setInputProcessor(this);
	}
	
	/**
	 * Creates and returns a new instance of MainMenuScreen.
	 * This WILL overwrite any existing instance.
	 * @param g the QuestMain representing the game
	 * @return the new instance
	 * @throws Exception
	 */
	public static MainMenuScreen makeNewInstance(QuestMain g) throws Exception{
		instance = new MainMenuScreen(g);
		return instance;
	}

	public static MainMenuScreen getInstance(){
		return instance;
	}
	
	@Override
	protected void draw(SpriteBatch spriteBatch){}

	@Override
	protected void draw(){
		spriteBatch.setProjectionMatrix(camera.combined);
		spriteBatch.begin();

		//draw menu background
		spriteBatch.draw(background, -0.5f, -0.5f, 1.0f, 1.0f);

		//draw menu elements
		spriteBatch.draw(banner, BANNER_X, BANNER_Y, BANNER_SIZE_X,
				BANNER_SIZE_Y);
		spriteBatch.draw(newGame, NEW_GAME_X, NEW_GAME_Y, BUTTON_SIZE_X,
				BUTTON_SIZE_Y);
		spriteBatch.draw(loadGame, LOAD_GAME_X, LOAD_GAME_Y, BUTTON_SIZE_X,
				BUTTON_SIZE_Y);
		spriteBatch.draw(quitGame, QUIT_GAME_X, QUIT_GAME_Y, BUTTON_SIZE_X,
				BUTTON_SIZE_Y);
		
		//draw overlays
		loadScreen.draw(spriteBatch);
		quitOverlay.draw(spriteBatch);

		spriteBatch.end();
	}
	
	@Override
	public boolean touchUp(int screenX, int screenY, int pointer, int button) {
		//left mouse button clicked
		if(button == Buttons.LEFT){
			//convert screen coordinates
			float x = Input.convertScreenCoorsX(screenX);
			float y = Input.convertScreenCoorsY(screenY);
			
			if(Input.inRange(x, y, NEW_GAME_X, NEW_GAME_Y, BUTTON_SIZE_X,
					BUTTON_SIZE_Y)){//new game
				
				//play sound
				selectSound.play(1.0f);
				
				//start a new game
				game.setScreen(0);
				return true;
			}
			if(Input.inRange(x, y, LOAD_GAME_X, LOAD_GAME_Y, BUTTON_SIZE_X,
					BUTTON_SIZE_Y)){//load game
				
				//play sound
				selectSound.play(1.0f);
				
				//activate load game screen
				loadScreen.activate(this);
				Gdx.input.setInputProcessor(loadScreen);
				return true;
			}
			if(Input.inRange(x, y, QUIT_GAME_X, QUIT_GAME_Y, BUTTON_SIZE_X,
					BUTTON_SIZE_Y)){//quitGame
				
				//play sound
				selectSound.play(1.0f);
				
				//activate overlay
				quitOverlay.activate();
				Gdx.input.setInputProcessor(quitOverlay);
				return true;
			}
		}	
		return false;
	}

	/*
	 * @see com.badlogic.gdx.Screen#dispose()
	 */
	@Override
	public void dispose() {
		spriteBatch.dispose();
	}

	/*
	 * @see com.badlogic.gdx.Screen#render(float)
	 */
	@Override
	public void render(float delta) {
		clearScreen();
		draw();
	}

	/*
	 * @see com.badlogic.gdx.Screen#resize(int, int)
	 */
	@Override
	public void resize(int width, int height) {
	}

	/*
	 * @see com.badlogic.gdx.Screen#pause()
	 */
	@Override
	public void pause(){
	}

	/*
	 * @see com.badlogic.gdx.Screen#resume()
	 */
	@Override
	public void resume() {
	}

	/*
	 * @see com.badlogic.gdx.Screen#show()
	 */
	@Override
	public void show() {
	}

	/*
	 * @see com.badlogic.gdx.Screen#hide()
	 */
	@Override
	public void hide() {
	}
}

