/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package menu;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Buttons;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.Texture.TextureFilter;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;

import game.QuestMain;
import hud.HUD;

/**
 * The pause screen in all its glory
 * @author Eyleen
 *
 */
public class PauseOverlay extends MenuScreen{
	private static PauseOverlay instance = null;
	
	//constants	
	private final String BACKGROUND = "assets/data/menu/shaded_bg.png";
	private final String MENU = "assets/data/menu/main_menu.png";
	private final String RESTART = "assets/data/menu/restart.png";
	private final String RESUME = "assets/data/menu/resume.png";
	private final String LOAD = "assets/data/menu/load.png";
	
	//image positions [screen coors]
	private final float RESUME_X = .3f;
	private final float RESUME_Y = -0.3f;
	private final float MENU_X = -0.45f;
	private final float MENU_Y = -0.3f;
	private final float RESTART_X = 0.3f;
	private final float RESTART_Y = -0.15f;
	private final float LOAD_X = -0.45f;
	private final float LOAD_Y = -0.15f;
	
	private final float BUTTON_SIZE_X = 0.18f;
	private final float BUTTON_SIZE_Y = 0.1f;
	
	private final char PAUSE_KEY = 'p';
	
	private TextureRegion background;
	private TextureRegion menu;
	private TextureRegion restart;
	private TextureRegion resume;
	private TextureRegion load;
	
	private ProgressJournal progress;
	private LoadMenuScreen loadScreen;
	private ReturnOverlay returnOverlay;
	private RestartOverlay restartOverlay;
	
	//initialize the audio file
			Sound selectSound = Gdx.audio.newSound(Gdx.files.internal("assets/data/sound/select.wav"));
	
	private PauseOverlay(QuestMain g) throws Exception{		
		Texture backgroundTexture = new Texture(Gdx.files.internal(BACKGROUND));
		backgroundTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		Texture menuTexture = new Texture(Gdx.files.internal(MENU));
		menuTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		Texture restartTexture = new Texture(Gdx.files.internal(RESTART));
		restartTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		Texture resumeTexture = new Texture(Gdx.files.internal(RESUME));
		resumeTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		Texture loadTexture = new Texture(Gdx.files.internal(LOAD));
		loadTexture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		
		background = new TextureRegion(backgroundTexture, 0, 0, Size.FULLSCREEN,
				Size.FULLSCREEN);
		menu = new TextureRegion(menuTexture, 0, 0,  Size.QUARTERSCREEN, Size.ICON);
		restart = new TextureRegion(restartTexture, 0, 0,  Size.QUARTERSCREEN, Size.ICON);
		resume = new TextureRegion(resumeTexture, 0, 0,  Size.QUARTERSCREEN, Size.ICON);
		load = new TextureRegion(loadTexture, 0, 0, Size.QUARTERSCREEN, Size.ICON);
		
		progress = ProgressJournal.makeNewInstance();
		try {
			loadScreen = LoadMenuScreen.makeNewInstance();
			returnOverlay = new ReturnOverlay(g);
			restartOverlay = new RestartOverlay(g);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		returnOverlay.deactivate();
		restartOverlay.deactivate();
	}
	
	public static PauseOverlay makeNewInstance(QuestMain g) throws Exception{
		instance = new PauseOverlay(g);
		return instance;
	}
	
	public static PauseOverlay getInstance(){
		return instance;
	}

	public void draw(SpriteBatch spriteBatch){		
		spriteBatch.draw(background, -0.5f, -0.5f, 1.0f, 1.0f);
		spriteBatch.draw(menu, MENU_X, MENU_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y);
		spriteBatch.draw(restart, RESTART_X, RESTART_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y);
		spriteBatch.draw(resume, RESUME_X, RESUME_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y);
		spriteBatch.draw(load, LOAD_X, LOAD_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y);
		progress.draw(spriteBatch);
		loadScreen.draw(spriteBatch);
		returnOverlay.draw(spriteBatch);
		restartOverlay.draw(spriteBatch);
	}
	
	@Override
	public boolean touchUp(int screenX, int screenY, int pointer, int button) {
		float x = Input.convertScreenCoorsX(screenX);
		float y = Input.convertScreenCoorsY(screenY);
		
		if(button == Buttons.LEFT){
			if(Input.inRange(x, y, MENU_X, MENU_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y)){

				//play sound
				selectSound.play(1.0f);
				
				//return to main menu
				returnOverlay.activate();
				Gdx.input.setInputProcessor(returnOverlay);
				return true;
			}
			if(Input.inRange(x, y, RESTART_X, RESTART_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y)){

				//play sound
				selectSound.play(1.0f);
				
				//restart the stage
				restartOverlay.activate();
				Gdx.input.setInputProcessor(restartOverlay);
				return true;
			}
			if(Input.inRange(x, y, RESUME_X, RESUME_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y)){
				//return to the game
				HUD.getInstance().unpause();
				return true;
			}
			if(Input.inRange(x, y, LOAD_X, LOAD_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y)){

				//play sound
				selectSound.play(1.0f);
				
				//go to load screen
				loadScreen.activate(this);
				Gdx.input.setInputProcessor(loadScreen);
				return true;
			}
			if(progress.inRange(x,y)){
				return true;
			}
		}
		return false;
	}
	
	@Override
	public boolean keyUp(int keycode) {
		if(keycode == Keys.ESCAPE){
			HUD.getInstance().unpause();
			return true;
		}
		return false;
	}

	@Override
	public boolean keyTyped(char character) {
		if(character == PAUSE_KEY){
			HUD.getInstance().unpause();
			return true;
		}
		return false;
	}
		
	@Override
	public boolean keyDown(int keycode) {
		return false;
	}

	@Override
	public boolean touchDown(int screenX, int screenY, int pointer, int button) {
		return false;
	}

	@Override
	public boolean touchDragged(int screenX, int screenY, int pointer) {
		return false;
	}

	@Override
	public boolean mouseMoved(int screenX, int screenY) {
		return false;
	}

	@Override
	public boolean scrolled(int amount) {
		return false;
	}

	@Override
	protected void draw() {}
}
