/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package pathfinding;

import java.util.ArrayList;
import java.util.List;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Circle;
import com.badlogic.gdx.math.Rectangle;

import game.GameScreen;
import game.GameUtility;
import game.QuestMain;
import gameobject.GameObject;
import gameobject.ObjectType;
import gameobject.SolidObject;
import gameobject.character.Character;
import gameobject.character.DeadTree;

/**
 * A graph of nodes on the screen, to be used for pathfinding
 * Is a singleton
 * @author jordan
 * @version Jan 22 2014
 */
public class Graph {

	// singleton

	// constants
	private float NODE_DISTANCE = 0.025f;
	private float OBJECT_RADIUS = 0.05f;
	private int GRAPH_WIDTH = (int) Math.round((1.0f - NODE_DISTANCE)/NODE_DISTANCE);
	private int GRAPH_HEIGHT = (int) Math.round((0.8f - NODE_DISTANCE)/NODE_DISTANCE);
	private float SCREEN_WIDTH_OFFSET = -0.5f + NODE_DISTANCE;
	private float SCREEN_HEIGHT_OFFSET = -0.3f + NODE_DISTANCE;

	// the actual graph, which is a 2D array of nodes
	private GraphNode[][] nodes;
	private ArrayList<GraphNode> checkNodes;
	private SolidObject[][] nodeImages;

	/**
	 * Private constructor
	 */
	public Graph()
	{
		// add image for testing purposes
		Texture image = new Texture(Gdx.files.internal("assets/data/node.png"));

		// declare 2D array
		nodes = new GraphNode[GRAPH_WIDTH][GRAPH_HEIGHT];
		checkNodes = new ArrayList<GraphNode>();
		nodeImages = new SolidObject[GRAPH_WIDTH][GRAPH_HEIGHT];

		// iterate along the width of the graph
		for (int i = 0; i < GRAPH_WIDTH; i++)
		{
			// iterate along the height of the graph
			for (int j = 0; j < GRAPH_HEIGHT; j++)
			{
				// calculate actual screen location
				float x = i*NODE_DISTANCE + SCREEN_WIDTH_OFFSET;
				float y = j*NODE_DISTANCE + SCREEN_HEIGHT_OFFSET;

				// make and add node
				GraphNode node = new GraphNode(x, y, i, j, false);
				nodes[i][j] = node;

				// get GameScreen instance
				GameScreen gameScreen = QuestMain.getCurrentScreen();
				List<GameObject> gameObjects = gameScreen.getAllObjects();

				boolean needToCheck = false;

				// figure out if node is initially occupied or not
				for (GameObject gameObject : gameObjects)
				{
					if(isOccupied(gameObject, node))
					{
						node.setOccupied(true);

						// check if it is occupied by a moving object
						ObjectType type = gameObject.getObjectType();
						if(type == ObjectType.PRIMARY || type == ObjectType.SECONDARY ||
								type == ObjectType.TERTIARY || type == ObjectType.ENEMY ||
								type == ObjectType.DESTRUCTABLE)
						{
							needToCheck = true;
						}

						break;
					}
				}

				// if it needs to be checked
				if(needToCheck || !node.isOccupied())
				{
					checkNodes.add(node);
				}

				// add image to game screen
				SolidObject nodeObject = new SolidObject(x, y, image, 0.005f, 0.005f, ObjectType.SHORT_OBSTACLE);
				nodeObject.setTag("Node");
				nodeImages[i][j] = nodeObject;
				if (!node.isOccupied())
				{
					//gameScreen.addNewObject(nodeImages[i][j]);
				}
			}
		}
	}

	/**
	 * Update the graph and its nodes
	 */
	public void update()
	{
		for(GraphNode node : checkNodes)
		{
			// get node position
			boolean occupied = false;

			GameScreen gameScreen = QuestMain.getCurrentScreen();
			List<Character> allies = gameScreen.getAllyList();
			List<Character> enemies = gameScreen.getEnemyList();
			List<DeadTree> deadTrees = gameScreen.getDeadTreeList();

			// check against moving objects
			for (Character ally : allies)
			{
				occupied = isOccupied(ally, node);
				if(occupied)
				{
					break;
				}
			}
			if(!occupied)
			{
				for (Character enemy : enemies)
				{
					occupied = isOccupied(enemy, node);
					if(occupied)
					{
						break;
					}
				}
			}
			if(!occupied)
			{
				for (DeadTree tree : deadTrees)
				{
					occupied = isOccupied(tree, node);
					if(occupied)
					{
						break;
					}
				}
			}

			node.setOccupied(occupied);

			// update node images
			if(occupied)
			{
				//gameScreen.removeObject(nodeImages[node.getGraphX()][node.getGraphY()]);
			}
			else
			{
				//gameScreen.addNewObject(nodeImages[node.getGraphX()][node.getGraphY()]);
			}
		}
	}

	/**
	 * Get the node nearest to the given coordinates
	 * @param x the x coordinate
	 * @param y the y coordinate
	 * @return the nearest node
	 */
	public GraphNode getNearestNode(float x, float y)
	{
		int graphX = (int) Math.round((x - SCREEN_WIDTH_OFFSET)/NODE_DISTANCE);
		int graphY = (int) Math.round((y - SCREEN_HEIGHT_OFFSET)/NODE_DISTANCE);
		
		if(graphX < 0) graphX = 0;
		if(graphX >= GRAPH_WIDTH) graphX = GRAPH_WIDTH - 1;
		if(graphY < 0) graphY = 0;
		if(graphY >= GRAPH_HEIGHT) graphY = GRAPH_HEIGHT - 1;

		return nodes[graphX][graphY];
	}

	/**
	 * Get a list of the nodes connected to the given node
	 * @param node the given node
	 * @return a list of connected nodes
	 */
	public ArrayList<GraphNode> getConnectedNodes(GraphNode node)
	{
		// add the given node to the array
		ArrayList<GraphNode> occupiedNodes = new ArrayList<GraphNode>();
		occupiedNodes.add(node);
		
		return getConnectedNodes(occupiedNodes);
	}

	/**
	 * Get a list of the nodes connected to the given object
	 * @param object the given object
	 * @return a list of connected nodes
	 */
	public ArrayList<GraphNode> getConnectedNodes(GameObject object)
	{
		// get the occupied nodes
		ArrayList<GraphNode> occupiedNodes = object.getOccupiedNodes();

		// if occupied nodes in empty, add the nearest node
		if(occupiedNodes.isEmpty())
		{
			occupiedNodes.add(getNearestNode(object.getCenterX(), object.getCenterY()));
		}
		
		return getConnectedNodes(occupiedNodes);
	}

	/**
	 * Get a list of nodes that have a direct connection to the given object
	 * 
	 * O = character
	 * 0 = connected nodes
	 * 							0	0	0
	 * 		O		-->			0	O	0
	 * 							0	0	0
	 * 
	 * 				or
	 * 
	 * 							0	0	0
	 * 		O					0	O	0
	 * 		O		-->			0	O	0
	 * 							0	0	0
	 * 
	 * 				or
	 * 
	 * 							0	0	0	0
	 * 		O	O	-->			0	O	O	0
	 * 							0	0	0	0
	 * 
	 * @param node
	 * @return
	 */
	private ArrayList<GraphNode> getConnectedNodes(ArrayList<GraphNode> occupiedNodes)
	{
		// create an empty list of connectedNodes
		ArrayList<GraphNode> connectedNodes = new ArrayList<GraphNode>();

		// get the starting x and y positions of the occupied nodes
		int startX = occupiedNodes.get(0).getGraphX();
		int endX = occupiedNodes.get(0).getGraphX();
		int startY = occupiedNodes.get(0).getGraphY();
		int endY = occupiedNodes.get(0).getGraphY();

		for(GraphNode node : occupiedNodes)
		{
			int x = node.getGraphX();
			int y = node.getGraphY();

			if(x < startX)
			{
				startX = x;
			}
			if(x > endX)
			{
				endX = x;
			}
			if(y < startY)
			{
				startY = y;
			}
			if(y > endY)
			{
				endY = y;
			}
		}

		// loop through graph and add connected nodes
		for(int i = startX - 1 ; i <= endX + 1 ; i++)
		{
			for(int j = startY - 1 ; j <= endY + 1 ; j++)
			{
				// if the index isn't out of bounds
				if(i >= 0 && i < GRAPH_WIDTH && j >= 0 && j < GRAPH_HEIGHT)
				{
					GraphNode node = nodes[i][j];
					// if the node isn't one of the occupied nodes
					if(!node.isOccupied())
					{
						connectedNodes.add(node);
					}
				}
			}
		}

		return connectedNodes;
	}

	/**
	 * Get the distance between adjacent nodes
	 * @return the distance
	 */
	public float getNodeDistance()
	{
		return NODE_DISTANCE;
	}

	/**
	 * Check if an object is a node
	 * @param object the object to check
	 * @param node the node to check
	 * @return true if it is occupying, false otherwise
	 */
	private boolean isOccupied(GameObject object, GraphNode node)
	{
		if(object.getTag() == "Node" || object.getObjectType() == ObjectType.POTION)
		{
			return false;
		}
		float radius = 1.0f * (float) Math.sqrt(OBJECT_RADIUS*OBJECT_RADIUS);
		Circle nodeCollider = new Circle(object.getCenterX(), object.getCenterY(), radius);
		if(nodeCollider.contains(node.getX(), node.getY()))
		{
			object.addOccupiedNode(node);
			return true;
		}
		else
		{
			object.removeOccupiedNode(node);
			return false;
		}
	}

	/**
	 * Find place farthest from all enemies
	 * @param character the character seeking a safe place
	 * @return the safe place as a node
	 */
	public GraphNode findSafePlace(Character character) {
		GameScreen gameScreen = QuestMain.getCurrentScreen();

		// get collection of enemies
		List<Character> enemies;
		if(character.getObjectType() == ObjectType.ENEMY)
		{
			enemies = gameScreen.getAllyList();
		}
		else
		{
			enemies = gameScreen.getEnemyList();
		}

		// get the best node
		GraphNode bestNode = null;
		float bestValue = 0;
		for(GraphNode node : checkNodes)
		{
			float value = 0;
			// we want the node that is farthest from all enemies
			for(Character enemy : enemies)
			{
				value += GameUtility.getStraightLineDistance(node.getX(), node.getY(),
						enemy.getX(), enemy.getY());
			}

			// replace if better
			if(value > bestValue)
			{
				bestValue = value;
				bestNode = node;
			}
		}

		return bestNode;
	}

}
