/*******************************************************************************
 * This file was developed for an MQP project at Worcester Polytechnic Institute.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package strategy;

import java.util.ArrayList;
import java.util.List;

import manager.AIManager;
import event.AIEvent;
import event.AIEventType;
import game.GameScreen;
import game.GameUtility;
import game.QuestMain;
import gameobject.GameObject;
import gameobject.ObjectType;
import gameobject.character.Character;
import gameobject.character.CharacterType;
import gameobject.character.DeadTree;

/**
 * This class manages the goals for a particular character
 * @author jordan
 * @version Feb 2 2014
 */
public class GoalManager {

	private Character character;

	private ArrayList<Goal> goals;

	private ArrayList<AIEvent> activeEvents;

	private List<Character> allies;
	private List<Character> enemies;
	private List<DeadTree> deadtrees;

	private Goal currentGoal;

	/**
	 * Constructor
	 * @param character the character to whom this goal manager belongs
	 */
	public GoalManager(Character character)
	{
		this.character = character;
		goals = new ArrayList<Goal>();
		activeEvents = new ArrayList<AIEvent>();

		// subscribe to receive ai events
		AIManager aiManager = AIManager.getInstance();
		aiManager.subscribeObject(this);
	}

	/**
	 * Update the goal manager
	 * @param delta the time since the last update
	 */
	public void update(float delta)
	{
		// create goals
		if(goals.isEmpty())
		{
			updateCharacterLists();

			// create an attack goal for each enemy
			for(Character enemy : enemies)
			{
				int priority = decideEnemyPriority(enemy);
				goals.add(new Goal(enemy, GoalType.ATTACK, priority));
			}
			// create an aid goal for each ally
			for(Character ally : allies)
			{
				// skip over the character that owns this goal manager
				if(ally == character)
				{
					continue;
				}
				// create and add goal
				int priority = decideAllyPriority(ally);
				goals.add(new Goal(ally, GoalType.AID, priority));
			}
			for(DeadTree tree : deadtrees)
			{
				int priority = decideTreePriority(tree);
				goals.add(new Goal(tree, GoalType.ATTACK, priority));
			}
			// create a survival goal
			/*int priority = decideSurvivalPriority();
			goals.add(new Goal(null, GoalType.SURVIVE, priority));*/

			currentGoal = goals.get(0);
		}

		ArrayList<Goal> irrelevantGoals = new ArrayList<Goal>();

		// figure out if there are any enemies left
		GameScreen gameScreen = QuestMain.getCurrentScreen();
		boolean battleOver = false;
		if(gameScreen.getNumEnemiesLeft() <= 0)
		{
			battleOver = true;
		}

		// update the goals
		for(Goal goal : goals)
		{
			// if the target dies, remove goal
			if(goal.getTarget() != null)
			{
				if(goal.getTarget().getObjectType() != ObjectType.DESTRUCTABLE)
				{
					Character target = (Character) goal.getTarget();
					if(target.isMarkedForDeath() || target.isDead())
					{
						irrelevantGoals.add(goal);
					}
				}
				else
				{
					if(goal.getTarget().isMarkedForDeath())
					{
						irrelevantGoals.add(goal);
					}
				}
			}
			// survive goal
			if(goal.getType() == GoalType.SURVIVE)
			{
				// remove survival goal if the battle is over
				if(battleOver)
				{
					irrelevantGoals.add(goal);
				}
				// otherwise calculate the priority
				else
				{
					int priority = decideSurvivalPriority();
					goal.setPriority(priority);
				}
			}
			// attack goal
			else if(goal.getType() == GoalType.ATTACK && goal.getTarget().getObjectType() != ObjectType.DESTRUCTABLE)
			{
				int priority = decideEnemyPriority(goal.getTarget());
				goal.setPriority(priority);
			}
			// attack tree
			else if(goal.getType() == GoalType.ATTACK)
			{
				int priority = decideTreePriority(goal.getTarget());
				goal.setPriority(priority);
			}
			// aid goal
			else if(goal.getType() == GoalType.AID)
			{
				int priority = decideAllyPriority(goal.getTarget());
				goal.setPriority(priority);
			}
		}

		// delete irrelevant goals
		for(Goal goal : irrelevantGoals)
		{
			goals.remove(goal);
		}

		updateCurrentGoal();
	}

	/**
	 * Update the ally and enemy lists
	 */
	private void updateCharacterLists()
	{
		GameScreen gameScreen = QuestMain.getCurrentScreen();
		// if this character is an enemy
		ObjectType type = character.getObjectType();
		if(type == ObjectType.ENEMY)
		{
			enemies = gameScreen.getAllyList();
			allies = gameScreen.getEnemyList();
		}
		// if this character is an ally
		else
		{
			enemies = gameScreen.getEnemyList();
			allies = gameScreen.getAllyList();
		}
		deadtrees = gameScreen.getDeadTreeList();
	}

	/**
	 * Update the current goal based on priorities
	 */
	private void updateCurrentGoal()
	{
		// iterate through goals to find the one with the highest priority
		Goal bestGoal = goals.get(0);
		for(Goal goal : goals)
		{
			if(goal.getPriority() > bestGoal.getPriority())
			{
				bestGoal = goal;
			}
		}

		// return the highest priority goal
		if(bestGoal != currentGoal)
		{
			// update attackers for old goal's target
			if(currentGoal.getType() == GoalType.ATTACK && currentGoal.getTarget().getObjectType() != ObjectType.DESTRUCTABLE)
			{
				GameObject target = currentGoal.getTarget();
				Character cTarget = (Character) target;
				cTarget.removeAttacker(character);
			}

			// update attackers for new goal's target
			if(bestGoal.getType() == GoalType.ATTACK && bestGoal.getTarget().getObjectType() != ObjectType.DESTRUCTABLE)
			{
				GameObject target = bestGoal.getTarget();
				Character cTarget = (Character) target;
				cTarget.addAttacker(character);
			}
		}

		currentGoal = bestGoal;
	}

	/**
	 * Get the current goal
	 * @return the current goal
	 */
	public Goal getCurrentGoal()
	{
		return currentGoal;
	}

	/**
	 * Handles ai events passed to the goal manager
	 * @param aievent the event
	 */
	public void handleEvent(AIEvent aievent)
	{
		GameObject sender = aievent.getSender();
		AIEventType type = aievent.getType();

		updateCharacterLists();

		// check that it wasn't an ally that sent the event
		for(Character ally : allies)
		{
			if(ally == sender)
			{
				return;
			}
		}

		// character has been taunted
		if(type == AIEventType.TAUNT)
		{
			boolean alreadyTaunted = false;
			for(AIEvent event : activeEvents)
			{
				if(event.getSender() == sender && event.getType() == type)
				{
					alreadyTaunted = true;
				}
			}
			// if the character hasn't already been taunted by the sender
			if(!alreadyTaunted)
			{
				activeEvents.add(aievent);
				character.setTaunted(true);
			}
		}

		// taunt has ended
		if(type == AIEventType.END_TAUNT)
		{
			for(AIEvent event : activeEvents)
			{
				// if the character was already taunted by the sender
				if(event.getSender() == sender && event.getType() == AIEventType.TAUNT)
				{
					activeEvents.remove(event);
					character.setTaunted(false);
					break;
				}
			}
		}

		// rogue is hiding
		if(type == AIEventType.HIDE)
		{
			boolean alreadyHidden = false;
			for(AIEvent event : activeEvents)
			{
				if(event.getSender() == sender && event.getType() == type)
				{
					alreadyHidden = true;
				}
			}
			// if the character hasn't already been taunted by the sender
			if(!alreadyHidden)
			{
				activeEvents.add(aievent);
			}
		}

		// rogue is no longer hiding
		if(type == AIEventType.END_HIDE)
		{
			for(AIEvent event : activeEvents)
			{
				// if the character was already taunted by the sender
				if(event.getSender() == sender && event.getType() == AIEventType.HIDE)
				{
					activeEvents.remove(event);
					break;
				}
			}
		}
	}

	/**
	 * Decide the priority of the enemy
	 * @param enemy the enemy
	 * @return the priority of the enemy
	 */
	private int decideEnemyPriority(GameObject enemy)
	{
		Character cEnemy = (Character) enemy;

		// priority starts at zero
		int priority = 0;

		// check for relevant events first
		for(AIEvent event : activeEvents)
		{
			// if there is a relevant event for this enemy
			if(event.getSender() == enemy)
			{
				// if the character has been taunted by the enemy
				if(event.getType() == AIEventType.TAUNT)
				{
					// priority is very high
					priority += 200;
				}
				// if the enemy is hiding
				if(event.getType() == AIEventType.HIDE)
				{
					// enemy has no priority
					return priority - 10;
				}
			}
		}

		// Warrior
		if(character.getCharacterType() == CharacterType.WARRIOR)
		{
			// get the distance between the character and the enemy
			float dist = GameUtility.getStraightLineDistance(character.getX(), character.getY(),
					enemy.getX(), enemy.getY());

			// closer enemies have higher priority
			priority += (1.0f - dist)*100;

		}

		// Mage
		else if(character.getCharacterType() == CharacterType.MAGE)
		{
			// enemies with more health have higher priority
			priority += ((float)cEnemy.getHealth() / (float)cEnemy.getMaxHealth())*100;
		}

		// Rogue
		else
		{
			// check if this enemy has the least health
			Character bestEnemy = enemies.get(0);
			for(Character e : enemies)
			{
				if(e.getHealth() < bestEnemy.getHealth())
				{
					bestEnemy = e;
				}
			}

			// enemy with least health has highest priority
			if(enemy.equals(bestEnemy))
			{
				priority += 100;
			}
		}

		return priority;
	}

	/**
	 * Decide the priority of the dead tree
	 * @param tree the tree
	 * @return the priority of the tree
	 */
	private int decideTreePriority(GameObject tree)
	{
		// priority starts at zero
		int priority = 0;

		// Warrior
		if(character.getCharacterType() == CharacterType.WARRIOR)
		{
			float treeDist = GameUtility.getStraightLineDistance(character.getX(), character.getY(),
					tree.getX(), tree.getY());
			boolean enemyCloser = false;
			for(Character enemy : enemies)
			{
				float enemyDist = GameUtility.getStraightLineDistance(character.getX(), character.getY(),
						enemy.getX(), enemy.getY());
				if(enemyDist < treeDist)
				{
					enemyCloser = true;
					break;
				}
			}

			if(!enemyCloser)
			{
				// closer trees have higher priority
				priority += (1.0f - treeDist)*100;
			}
		}

		// Mage
		else if(character.getCharacterType() == CharacterType.MAGE)
		{
			float treeDist = GameUtility.getStraightLineDistance(character.getX(), character.getY(),
					tree.getX(), tree.getY());
			boolean enemyCloser = false;
			for(Character enemy : enemies)
			{
				float enemyDist = GameUtility.getStraightLineDistance(character.getX(), character.getY(),
						enemy.getX(), enemy.getY());
				if(enemyDist < treeDist)
				{
					enemyCloser = true;
					break;
				}
			}

			if(!enemyCloser)
			{
				// closer trees have higher priority
				priority += (1.0f - treeDist)*100;
			}
		}

		// Rogue
		else
		{

		}

		return priority;
	}

	/**
	 * Decide the priority of the ally
	 * @param ally the ally
	 * @return the priority of the ally
	 */
	private int decideAllyPriority(GameObject ally)
	{
		Character cAlly = (Character) ally;

		// priority starts at zero
		int priority = 0;

		// Warrior
		if(character.getCharacterType() == CharacterType.WARRIOR)
		{
			if(!cAlly.isDead())
			{
				// allies with less health have higher priority
				//priority += ((float)(ally.getMaxHealth() - ally.getHealth()) / (float)ally.getMaxHealth())*50;
			}
		}

		// Mage
		else if(character.getCharacterType() == CharacterType.MAGE)
		{
			if(!cAlly.isDead())
			{
				if(!character.getName().equals("Big Boss"))
				{
					// allies with less health have higher priority
					priority += ((float)(cAlly.getMaxHealth() - cAlly.getHealth()) / (float)cAlly.getMaxHealth())*100;
				}
				else
				{
					if(character.getHealth() < 0.5*character.getMaxHealth())
					// allies with more health have higher priority
					priority += ((float)cAlly.getHealth() / (float)cAlly.getMaxHealth())*100;
				}
			}
		}

		// Rogue
		else
		{
			if(!cAlly.isDead())
			{
				// allies with less health have higher priority
				//priority += ((float)(ally.getMaxHealth() - ally.getHealth()) / (float)ally.getMaxHealth())*50;
			}
		}

		return priority;
	}

	/**
	 * Decide the priority of pure survival
	 * @return the priority of survival
	 */
	private int decideSurvivalPriority()
	{
		// priority starts at zero
		int priority = 0;

		// Warrior
		if(character.getCharacterType() == CharacterType.WARRIOR)
		{
			// higher priority if the character has lower health
			priority += ((float)(character.getMaxHealth() - character.getHealth()) / (float)character.getMaxHealth())*100;
		}

		// Mage
		else if(character.getCharacterType() == CharacterType.MAGE)
		{
			// higher priority if the character has lower health
			priority += ((float)(character.getMaxHealth() - character.getHealth()) / (float)character.getMaxHealth())*100;

			// higher priority if the character has more attackers
			priority += character.getNumAttackers()*10;
		}

		// Rogue
		else
		{
			// higher priority if the character has lower health
			priority += ((float)(character.getMaxHealth() - character.getHealth()) / (float)character.getMaxHealth())*100;

			// higher priority if the character has more attackers
			priority += character.getNumAttackers()*5;
		}

		return priority;
	}

}
